package biss;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Calendar;
import java.util.Date;

/**
 * lib class providing general static functions for File objects
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author P.C.Mehlitz
 */
public class FileLib
{
public static boolean backup ( File f ) {
	String fn = f.getAbsolutePath() + '~';
	File fOld = new File( fn);

	if ( f.exists() ) {
		if ( fOld.exists() )
			fOld.delete();
		return f.renameTo( fOld);
	}
	return false;
}

public static int copyFileData ( File file, byte[] buffer ) {
	int len = (int)file.length();
	if ( len > buffer.length ) return -1;

	try {
		FileInputStream in = new FileInputStream( file);
		in.read( buffer, 0, len);
		in.close();
		return len;
	}
	catch ( Throwable x ) {
		return -1;
	}
}

public static int copyFileData ( File file, char[] buffer ) {
	int len = (int)file.length();
	if ( len > buffer.length ) return -1;

	try {
		FileReader in = new FileReader( file);
		in.read( buffer, 0, len);
		in.close();
		return len;
	}
	catch ( Throwable x ) {
		return -1;
	}
}

public static boolean exists ( String pathName ) {
	File f = new File( pathName);
	return f.exists();
}

public static String fileInfoString ( File f, boolean fixed ) {	
	Calendar dtg = Calendar.getInstance();
	dtg.setTime( new Date( f.lastModified()));

	char [] fill = { ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ' };
	int    n, len = (int)f.length();
	String size = Integer.toString( len);

	ByteArrayOutputStream o = new ByteArrayOutputStream( 24);
	PrintWriter      s  = new PrintWriter( o, false);

	if ( fixed )
		s.write( fill, 0, 6 - size.length());
	s.print( size);
	s.print( "  ");

	n = dtg.get( Calendar.MONTH) + 1;
	if ( n < 10 ) s.print( '0');
	s.print( n);
	s.print( '/');

	n = dtg.get( Calendar.DAY_OF_MONTH) + 1;
	if ( n < 10 ) s.print( '0');
	s.print( n);
	s.print( '/');

	n = dtg.get( Calendar.YEAR);
	if ( n < 10 ) s.print( '0');
	s.print( n);
	s.print( " ");

	n = dtg.get( Calendar.HOUR);
	if ( n < 10 ) s.print( '0');
	s.print( n);
	s.print( ':');

	n = dtg.get( Calendar.MINUTE);
	if ( n < 10 ) s.print( '0');
	s.print( n);
	
	s.flush();
	s.close();

	return o.toString();
}

public static String findUpwards ( String pathName, String dirName, String postFix ) {
	File    target;
	int     i;
	String  p = pathName;
	String  t;

	if ( dirName != null ) {	
		if ( pathName.startsWith( dirName) ) {
			if ( (dirName.charAt( (i=dirName.length() - 1)) == File.separatorChar) ||
			     (pathName.charAt( i+1) == File.separatorChar) ) {
				if ( postFix != null ) {
					if ( dirName.charAt( i) != File.separatorChar) {
						if ( postFix.charAt(0) != File.separatorChar )
							t = dirName + File.separatorChar + postFix;
						else
							t = dirName + postFix;					
					}
					else {
						if ( postFix.charAt(0) == File.separatorChar )
							t = dirName + postFix.substring( 1);
						else
							t = dirName + postFix;						
					}
					
					target = new File( t);
					if ( target.exists() )
						return t;
				}
				else
					return dirName;
			}
		}	
	}
	else {
		if ( postFix != null ) {
			if ( !postFix.startsWith( File.separator) )
				postFix = File.separator + postFix;

			while ( true ) {
				i = p.lastIndexOf( File.separatorChar);
				if ( i > 0 ) {
					p = p.substring( 0, i);
					t = p + postFix;
					target = new File( t);
					if ( target.exists() )
						return t;
				}
				else
					break;
			}
		}
	}
	
	return null;
}

public static byte[] getFileData ( File file ) {
	int len = (int)file.length();

	try {
		FileInputStream in = new FileInputStream( file);
		byte buf[] = new byte[len];
		in.read( buf, 0, len);
		in.close();
		return buf;
	}
	catch ( Throwable x ){
		return null;
	}
}

public static String getFileDataAsString ( File file ) {
	byte[] buf = getFileData( file);
	if ( buf != null )
		return new String( buf, 0);
	else
		return null;
}

public static char[] getUTFFileData ( File file ) {
	int len = (int)file.length();

	try {
		FileReader in = new FileReader( file);
		char buf[] = new char[len];
		in.read( buf, 0, len);
		in.close();
		return buf;
	}
	catch ( Throwable x ){
		return null;
	}
}

public static String getUTFFileDataAsString ( File file ) {
	char[] buf = getUTFFileData( file);
	if ( buf != null )
		return new String( buf);
	return null;
}

public static boolean isSubDir ( File dir, File sDir ) {
	if ( !dir.isDirectory() || !sDir.isDirectory() )
		return false;

	String pd1 = dir.getAbsolutePath();
	String pd2 = sDir.getAbsolutePath();

	if ( pd2.startsWith( pd1)){
		int lsc = pd2.lastIndexOf( File.separatorChar);
		return lsc == pd1.length();
	}
	
	return false;
}

public static boolean setFileData ( File file, String cont, boolean backup ) {
	try {
		if ( backup)
			backup( file);
		FileWriter fw = new FileWriter( file);
		fw.write( cont);
		fw.close();
		return true;
	}
	catch( Exception e) {
		return false;
	}
}

public static boolean setFileData ( String pathName, String cont, boolean backup ) {
	return setFileData( new File( pathName), cont, backup);
}
}
