package biss;


/**
 * lib class providing general static functions on Strings 
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author P.C.Mehlitz
 */
public class StringLib
  implements CharTypes
{
public static boolean contains ( String source, char c ) {
	return (source.indexOf( c) > -1);
}

public static String[] getSubstrings ( String source, char sep ) {
	int n = source.length();
	char[] c = new char[n];
	int m, m1, i, i0, j;
	String [] a;
	String s;

	source.getChars( 0, n, c, 0);
	for ( m=1, i=0; i<n; i++ ) {
		if ( c[i] == sep ) m++;
	}
	if ( m == 2 && n == 1 ) m--; // single sep char

	a = new String[m];
	m--;
	
	for ( i0=0, i=0, j=0; i<n; i++ ) {
		if ( c[i] == sep ) {
			if ( i0 < i )
				s = new String( c, i0, i-i0);
			else
				s = null;
			i0 = i+1;
			
			a[j++] = s;
			
			if ( j == m )
				break;
		}
	}
	
	if ( j <= m ) {
		if ( i0 < n )
			s = new String( c, i0, n-i0);
		else
			s = null;
		
		a[j] = s;
	}
	
	return a;
}

public static int indexOfId ( String source, String pattern ) {
	return indexOfId( source, pattern, 0);
}

public static int indexOfId ( String source, String pattern, int iStart ) {
	int i = iStart, j;
	int n = pattern.length();
	int m = source.length();

	while ( (i=source.indexOf( pattern, i)) >= 0 ) {
		if ( i == 0 || !IsAN[ source.charAt(i-1)] ) {
			j = i+n;
			if ( j == m || !IsAN[ source.charAt(j)] )
				return i;
		}
		i += n;
	}

	return -1;
}

public static void main ( String[] args ) {
	//System.out.println( StringLib.indexOfId( args[0], args[1]));
	//System.out.println( StringLib.replaceAll( args[0], args[1], args[2]));
	//System.out.println( StringLib.valueOf( -10, 8));
	
	String a[] = getSubstrings( args[0], '/');
	for ( int i=0; i<a.length; i++ )
		System.out.println( a[i]);

	System.out.println( StringLib.toUpperCase( "aBcD123"));
	System.out.println( StringLib.toLowerCase( "aBcD123"));
	System.out.println( StringLib.toLowerCase( "ABCDEFabcdefGHIJKLghijklMNOPQmnopqRSTUVXYZrstuvwxyz"));
	System.out.println( StringLib.toUpperCase( "ABCDEFabcdefGHIJKLghijklMNOPQmnopqRSTUVXYZrstuvwxyz"));

	long t;
	t = System.currentTimeMillis();
	for ( int j=0; j<5000; j++) {		
		StringLib.toUpperCase( "aBcD123");
		StringLib.toLowerCase( "aBcD123");
		StringLib.toLowerCase( "ABCDEFabcdefGHIJKLghijklMNOPQmnopqRSTUVXYZrstuvwxyz");
		StringLib.toUpperCase( "ABCDEFabcdefGHIJKLghijklMNOPQmnopqRSTUVXYZrstuvwxyz");
	}
	System.out.println( (System.currentTimeMillis()-t) + " ms");

	t = System.currentTimeMillis();
	for ( int j=0; j<5000; j++) {		
		"aBcD123".toUpperCase();
		"aBcD123".toLowerCase();
		"ABCDEFabcdefGHIJKLghijklMNOPQmnopqRSTUVXYZrstuvwxyz".toLowerCase();
		"ABCDEFabcdefGHIJKLghijklMNOPQmnopqRSTUVXYZrstuvwxyz".toUpperCase();
	}
	System.out.println( (System.currentTimeMillis()-t) + " ms");

	t = System.currentTimeMillis();
	for ( int j=0; j<5000; j++) {		
		StringLib.toUpperCase( "aBcD123");
		StringLib.toLowerCase( "aBcD123");
		StringLib.toLowerCase( "ABCDEFabcdefGHIJKLghijklMNOPQmnopqRSTUVXYZrstuvwxyz");
		StringLib.toUpperCase( "ABCDEFabcdefGHIJKLghijklMNOPQmnopqRSTUVXYZrstuvwxyz");
	}
	System.out.println( (System.currentTimeMillis()-t) + " ms");

}

public static int occurencesOf ( String source, char c ) {
	int ocr = 0;
	int len = source.length();
	for ( int i=0; i<len; i++){
		if ( source.charAt( i) == c)
			ocr++;
	}
	return ocr;
}

public static String replaceAll ( String source, String pattern, String replace ) {
	int m = source.length();
	int n = pattern.length();
	int o = (replace != null) ? replace.length() : 0;
	StringBuffer s = new StringBuffer( m);
	int i, l;
	for ( l=0; (i = source.indexOf( pattern, l)) >= 0; l=i+n ){
		if ( i > l )
			s.append( source.substring( l, i));
		if ( o > 0 )
			s.append( replace);
	}
	if ( l < m )
		s.append( source.substring( l, m) );

	return s.toString();
}

public static String toLowerCase ( String s ) {
	char[] bytes = s.toCharArray();
	char c;
	for (int i=0; i<bytes.length;i++) {
		c = (bytes[i]);
		if ( (c >= 'A') && (c <= 'Z') ) {
			c = (char)((byte)c | (byte)32);
			bytes[i] = c;
		}
	}
	return  String.valueOf( bytes);
}

public static String toUpperCase ( String s ) {
	char[] bytes = s.toCharArray();
	char c;
	for (int i=0; i<bytes.length;i++) {
		c = (bytes[i]);
		if ( (c >= 'a') && (c <= 'z') ) {
			c = (char)((byte)c & (byte)223);
			bytes[i] = c;
		}
	}
	return  String.valueOf( bytes);
}

public static String valueOf ( int n, int len ) {
	char[] a = new char[len];
	int x = (n > 0) ? n : -n;
	int r;
	int i, j;
	for (i=len-1, r= x/10; i >= 0; i-- ) {
		if ( x > 0 ) {
			a[i] = (char) ((x - (r*10)) + '0');
			x = r;
			r = x / 10;
		}
		else {
			if ( n < 0 ) {
				a[i] = '-';
				n = -n;
			}
			else
				a[i] = ' ';
		}
	}
	return String.valueOf( a);
}
}
