package biss.awt;

import biss.FileLib;
import biss.ObserverSocket;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.util.Observable;

/* (compiler bug, second '*' causes compile error)
 * replaces java.awt.Button
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class Button
  extends Widget
  implements HotKeyProcessor, MouseMotionInterface
{
	boolean Alert = false;
	int BorderWidth = Awt.BorderWidth;
	FontMetrics Fm;
/**
 * triggered by Mouse1Up, SpaceBar or Enter key to notify about
 * an action event
 * parameter: this
 */
	public ObserverSocket OsAction = new ObserverSocket( this);
/**
 * triggered by painting just in case of at least one registered
 * observer. This observer is responsible for all inner painting
 * e.g. overrides standard painting
 * parameter: this
 */
	public ObserverSocket OsDraw = new ObserverSocket( this);
	boolean Pushed = false;
	boolean Raise = false;
	String Selector = null;
	String Text = null;
	Point PMn = null;
	char MnKey = 0;
	static FontMetrics DefFm;
	Image Img;
	boolean Carved = Awt.CarvedText;

static {
	DefFm = Awt.getFontMetrics( Awt.ButtonFont);
}

public Button() {
	this( "", null, null);
}

public Button( String text) {
	this( text, null, null);
}

public Button( String text, Font fnt){
	this( text, null, fnt);
}

public Button( String pathName, String text) {
	this( text, null, null);
	setImage( pathName, text);
}

public Button( String text, String sel, Font fnt) {

	setForeground( Awt.ButtonForeClr);
	setBackground( Awt.ButtonBackClr);
	setCursor( Cursor.getPredefinedCursor( Awt.ActionCursor));

	setFont( fnt);
	setContents( text);

	Selector = ( sel != null) ? sel : Text;
	registerHotKeys();
}

public void addNotify () {
	super.addNotify();

	if ( MnKey != 0) {
		Window w = getWindow();
		if ( w instanceof GUIHandlerInterface)
			((GUIHandlerInterface)w).getGUIHandler().addHotKey( MnKey | 32, Awt.GlobalKeyMod, this, "trigger");
	}

}

void animate ( boolean upd) {

	if ( Img != null){
		redraw();
		return;
	}
		
	Pushed = true;
	Raise = true;
	redrawImmediate();

	try { Thread.sleep(100);}
	catch ( InterruptedException x ){}

	Pushed = false;
	Raise = false;
	if ( upd)
		redrawImmediate();
}

public void charInput( char c, int mod) {
	if ( c == ' ')
		trigger();
}

void drawImage( Graphics g) {
	int w = ! Pushed  ? Width : Width + 2 * BorderWidth;
	int h = ! Pushed  ? Height : Height + 2 * BorderWidth;
	GraphicsLib.drawImageCentered( g, Img, w, h, this);
}

void drawImageText( Graphics g ) {    
	int iw = Img.getWidth( this);
	int ih = Img.getHeight( this);

	//prevent flicker ( will be drawn on image complete)
	if ( (iw <= 0) || (ih <= 0) )
		return;

	int th = Fm.getHeight() + ih;
	int fd = Fm.getDescent();
	int dof = Pushed ? 1 : 0;
	int y = Height - ( Height - th +1) / 2 - fd; 
	int x = (Width - Fm.stringWidth( Text)) / 2;

	if ( ! Alert && Carved) {
		g.setColor( Awt.ShadowBrightClr);
		g.drawString( Text, x+dof+1, y+dof+1);
	}

	if ( EventHandler.IsDisabled )
		g.setColor( Awt.InactiveClr);
	else
		g.setColor( Alert ? Awt.ButtonAlertForeClr : Awt.ButtonForeClr);

	g.drawString( Text, x+dof, y+dof);

	if ( PMn != null){
		g.setColor( Color.blue);
		g.drawLine( x+dof+PMn.x, y+dof+1, x+dof+PMn.y, y+dof+1);
	}

	g.drawImage( Img, (Width-iw) / 2 + dof, y+fd-th+dof, this);
}

void drawText( Graphics g){
	int y = Height - ( Height - Fm.getHeight() + 1) / 2 - Fm.getDescent();
	int x = (Width - Fm.stringWidth( Text)) / 2;

	if ( !Raise && Pushed ) {
		x += 1; y += 1;
	}

	drawText( g, x, y);
}

void drawText( Graphics g, int x, int y ) {    

	g.setFont( getFont() );

	if ( ! Raise ) {
		if ( ! Alert && Carved ) {
			g.setColor( Awt.ShadowBrightClr);
			g.drawString( Text, x+1, y+1);
		}
		if ( EventHandler.IsDisabled )
			g.setColor( Awt.InactiveClr);
		else
			g.setColor( Alert ? Awt.ButtonAlertForeClr : Awt.ButtonForeClr);
		g.drawString( Text, x, y);
	}
	else {
		GraphicsLib.flashText( g, Text, x, y);
	}

	if ( PMn != null){
		g.setColor( Color.blue);
		g.drawLine( x+PMn.x, y+1, x+PMn.y, y+1);
	}
}

public String getContents(){
	return Text;
}

public Dimension getMinimumSize(){
	int w, h;

	if ( Text != null && Fm != null ) {
		w = 15 + Fm.stringWidth( Text);
		h = 10 + Fm.getHeight();
	}
	else
		w = h = 4 * Awt.DlgXUnit;
	
	return new Dimension( w, h);
}

public String getSelector(){
	return Selector;
}

public void gotFocus(){
	updateFocusMark();
}

public void lostFocus(){
	updateFocusMark();
}

public int minWidth() {
	if ( Text == null)
		return 0;
	return Fm.stringWidth( Text);
}

public void mouse1Down(){
	if ( isEnabled() ) {
		Pushed = true;
		redraw();
	}
}

public void mouse1Drag(){
	if ( ! isEnabled() )
		return;
	int x = Mouse.Pos.x;
	int y = Mouse.Pos.y;
	if ( (x > 0) && (x < Width) &&
	     (y > 0) && (y < Height)){
		if ( ! Pushed){
			Pushed = true;
			redraw();
		}
	}
	else if ( Pushed){
		Pushed = false;
		redraw();	
	}					
}

public void mouse1Up(){
	if ( Pushed){
		Pushed = false;
		trigger();
	}
}

public void mouse2Drag(){
}

public void mouseMove(){
}

public void processKey( Object sel){
	if ( "trigger".equals( sel))
		trigger();
}

protected void redraw ( Graphics g) {
	int b = BorderWidth;
	int b2 = b*2;

	g.setFont( getFont() );

	g.setColor( Alert ? Awt.ButtonAlertBackClr : Awt.ButtonBackClr);
	g.fillRect( b, b, Width-b2, Height-b2);

	if ( ! OsDraw.isEmpty() )	OsDraw.notifyObservers( this);
	else if ( Img  != null) {
		if ( Text != null) drawImageText( g);
		else               drawImage( g);
	}
	else if ( Text != null)	drawText( g);

	GraphicsLib.paintButton( g, 0, 0, Width, Height, Pushed, null);

	if ( EventHandler.HasFocus )
		updateFocusMark( g);
}

void registerHotKeys() {
	addHotKey( Key.Newline, HotKey.Lone, this, "trigger");
}

public void setAlert ( boolean alert ) {
	Alert = alert;
	redraw();
}

public void setCarved( boolean c){
	Carved = c;
	if ( isShowing() )
		redraw();
}

public void setContents( String cont){
	if ( cont != null){
		if ( cont.endsWith( ".gif"))	setImage( cont);
		else													setLabel( cont);
	}
}

public synchronized void setFont( Font fnt) {
	Font uf = ( fnt != null) ? fnt : Awt.ButtonFont;

	if ( getFont() == uf )
		return;

	super.setFont( uf);

	if ( uf.equals( Awt.ButtonFont) )
		Fm = DefFm;
	else
		Fm = Awt.DefToolkit.getFontMetrics( uf);

	if ( isShowing() )
		redraw();
}

public void setImage( Image img){
	setImage( img, null);
}

public void setImage( Image img, String label){
	Text = label;
	Img  = img;
	if ( Text != null ){
		setFont( Awt.ButtonImageFont);
		splitLabel();
	}
	if ( isShowing() )
		redraw();
}

public boolean setImage( String pathName){
	return setImage( pathName, null);
}

public boolean setImage( String pathName, String label){
	Image img = Awt.getImage( pathName);
	if ( img != null){
		setImage( img, label);
		return true;
	}
	System.out.println( "cannot find imagefile: " + pathName);
	return false;
}

public boolean setImageClass( String imgCls){
	return setImageClass( imgCls, null);
}

public boolean setImageClass( String imgCls, String label){
	Image img = ClassImageSource.getImage( imgCls);
	if ( img != null){
		setImage( img, label);
		return true;
	}
	return false;
}

public void setImageText( String label){
	Text = label;
	setFont( Awt.ButtonImageFont);
	splitLabel();
	if ( isShowing() )
		redraw();
}

public void setLabel( String label){
	Img  = null;
	Text = label;
	splitLabel();
	setFont( Awt.ButtonFont);
}

public void setSelector( String sel){
	Selector = (sel != null) ? sel : Text;
}

public void setTextImage( Image img){
	Img  = img;
	setFont( Awt.ButtonImageFont);
}

void splitLabel() {
	char mn = '~';
	String rStr;
	int xb, xe, idx = Text.indexOf( mn);

	if ( idx > -1){   
		rStr = Text.substring( 0, idx);
		xb = Fm.stringWidth( rStr);
		Text = rStr + Text.substring( idx+1);
		xe = xb + Fm.charWidth( Text.charAt( idx));
		MnKey = Text.charAt( idx);
		PMn = new Point( xb, xe);
	}

}

public void trigger (){
	if ( isEnabled() ) {
		animate( true);
		OsAction.notifyObservers( this);
	}
}

protected void updateFocusMark () {
	Graphics g = getGraphics();
	if ( g != null) {
		updateFocusMark( g);
		g.dispose();
	}
}

protected void updateFocusMark ( Graphics g) {
	if ( EventHandler.HasFocus)
		GraphicsLib.drawFocusMark( g, Awt.BorderWidth, Awt.BorderWidth,
	                           Awt.FocusMarkWidth);
	else
		GraphicsLib.clearFocusMark( g, Awt.BorderWidth, Awt.BorderWidth,
	                           Awt.FocusMarkWidth, getBackground() );
}
}
