package biss.awt;

import biss.XorSet;
import biss.XorSetElement;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Graphics;
import java.util.Vector;

/**
 * replaces java.awt.Checkbox
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class Checkbox
  extends Button
  implements XorSetElement
{
	boolean LastState;
	XorSet ExOrs = null;

public Checkbox () {
	this( "");
}

public Checkbox ( String text ) {
	super( text);
	setBackground( Awt.WindowBackClr);
}

public Checkbox ( String pathName, String text ) {
	super( pathName, text);
	setBackground( Awt.WindowBackClr);
}

public Checkbox ( String text, boolean checked ) {
	this( text);
	Pushed = checked;
}

public Dimension getMinimumSize() {
	return getPreferredSize();
}

public Dimension getPreferredSize() {
	int w = 30;
	int h = 21;
	
	if ( Text != null && Fm != null ) {
		w += Fm.stringWidth( Text);
		h = 8 + Fm.getHeight();
	}

	return new Dimension( w, h);
}

public boolean isChecked () {
	return Pushed;
}

public void mouse1Down(){
	if ( ! isEnabled() )
		return;
	LastState = Pushed;
	Pushed = true;
//	redraw();
}

public void mouse1Up(){
	int x = Mouse.Pos.x;
	int y = Mouse.Pos.y;
	if ( (x > 0) && (y > 0) && (x < Width) && (y < Height) ){
		trigger();
	}
}

protected void redraw ( Graphics g ) {
	if ( Img != null){
		super.redraw( g);
		return;
	}

	int d = (ExOrs != null) ? Awt.CheckboxWidth + 3 : Awt.CheckboxWidth;
	int y = (Height -d) / 2;
	int x = d + 2*y + Awt.SysFontWidth / 2;

	blank( g);

	if ( EventHandler.HasFocus )
		updateFocusMark( g);

	if ( ! OsDraw.isEmpty() ){
		OsDraw.notifyObservers( this);
		GraphicsLib.paintButton( g, 0, 0, Width, Height, Pushed, null);
		return;
	}

	if ( ExOrs != null )
		GraphicsLib.paintRadio( g, y, y, d, d, Pushed);
	else
		GraphicsLib.paintInstepBox( g, y, y, d, d,
	                            Pushed ? Awt.CheckboxSetClr : Awt.CheckboxUnsetClr );

	y = (Height + Fm.getHeight()) / 2 - Fm.getDescent(); 
	drawText( g, x, y);
}

public void setChecked ( boolean checked, boolean fire ) {

	if ( (! isEnabled()) || ( LastState == checked) )
		return;

	Pushed = checked;
	LastState = Pushed;
	redraw();

	if ( Pushed && (ExOrs != null) )
		ExOrs.setSelfSelected( this);

	if ( fire)
		OsAction.notifyObservers( this);
}

public void setExclusive ( XorSet exclusiveSet ) {
	ExOrs = exclusiveSet;
	ExOrs.remember( this);
}

public void setState( boolean state){
	//for java.awt.Checkbox compatibility
	if ( state != LastState )
		setChecked( state, true);
}

public void trigger(){
	boolean an = (ExOrs == null) || (LastState == false);

	if ( ! isEnabled() )	return;
	if ( an)							animate( false);

	if ( ExOrs == null)
		setChecked( !LastState, true);
	else if ( LastState == false)
		setChecked( true, true);

}

protected void updateFocusMark ( Graphics g ) {
	int y = Height / 2;
	int d = (int)( Math.sqrt( 0.5) * Awt.FocusMarkWidth) + 1;
	
	int u[] = { Width-1, Width-d-1,  Width-1 };
	int v[] = { y+d, y, y-d };

	if ( EventHandler.HasFocus){
		g.setColor( Awt.FocusClr);
		g.fillPolygon( u, v, 3);
		g.setColor( Color.gray);
		g.drawLine( u[2], v[2], u[0], v[0]);
	}
	else {
		g.setColor( getBackground() );
		g.fillPolygon( u, v, 3);
		g.drawLine( u[2], v[2], u[0], v[0]);
	}
}

protected void updateFocusMarkX ( Graphics g ) {
	int d = (ExOrs != null) ? Awt.CheckboxWidth + 3 : Awt.CheckboxWidth;
	int y = (Height -d) / 2;
	int x = d + 2*y + Awt.SysFontWidth / 2;
	int df = Awt.FocusMarkWidth;
	int w  = Fm.stringWidth( Text);

	if ( EventHandler.HasFocus)
		GraphicsLib.drawFocusMark( g, Math.min( Width -df-2, x + Awt.SysFontWidth + w),
		                           Height-df-2, df );
	else
		GraphicsLib.clearFocusMark( g, Math.min( Width -df-2, x + Awt.SysFontWidth + w),
		                           Height-df-2, df, getBackground() );
}

public void xorSelect () {

}

public void xorUnselect () {
	setChecked( false, false);
}
}
