package biss.awt;

import java.awt.LayoutManager;
import java.util.Observable;

/**
 * ColumnList widget. Shows a label view ( ListHeader ) on top
 * of the List, to get a 'columnized' item impression.
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class ColumnList
  extends List
{
public ColumnList () {
	this( new ListHeader() );
}

public ColumnList ( ListHeader h) {
	super( SB_BOTH, false);

	Header = h;
	
	//add scroll observer before canvas, for paint operations
	//depend on Header positions
	Horz.OsScroll.addObserver( this);
	Header.OsWidthChange.addObserver( this);

	Cv = new ListCanvas( Horz, Vert, this);
	setCanvas( Cv);
}

public ColumnList ( String labels) {
	this( new ListHeader( labels));
}

protected ColumnList ( int type) {
	super( type, false);
}

public void addColumn ( String label) {
	Header.addColumn( label);
}

public void addColumn ( String label, int wid) {
	Header.addColumn( label, wid);
}

public void addColumns ( String labels) {
	Header.addColumns( labels);
}

public void doLayout() {
	LayoutManager lm = getLayout();

	if ( lm != null)
		lm.layoutContainer( this);
	else {
		int bh = Header.preferredHeight();
		Header.setBounds( 0, 0, Width, bh);
		updateBounds( 0, bh, 0, 0);
	}
}

void setCanvas( ScrollableCanvas cv) {
	add( Header);
	super.setCanvas( cv);
}

public void update ( Observable obs, Object arg ) {
	if ( obs == Header.OsWidthChange)
		Cv.redraw();
	else if (( Horz != null) && ( obs == Horz.OsScroll) )
		Header.setOffset( Horz.getPosition() );
	else
		super.update( obs, arg);
}
}
