package biss.awt;

import biss.ObserverSocket;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Panel;

/**
 * java.awt.Panel replacement used as base for various biss.awt
 * container widgets
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class Compound
  extends Container
  implements FocusSetInterface
{
	Component FocusForward;
	Border FBorder = new Border();
	int Width = 0;
	int Height = 0;
	public ObserverSocket OsFocusGot = new ObserverSocket( this);
	public ObserverSocket OsFocusLost = new ObserverSocket( this);
	public boolean Enabled = true;

public boolean canGetFocus () {
	return isEnabled() && isShowing();
}

public boolean contains ( int x, int y) {
	if ( isShowing() )
		return super.contains( x, y);
	return false;
}

public boolean focusNext () {
	return false;
}

public boolean focusPrev () {
	return false;
}

public Frame getFrame() {
	for ( Container so = getParent(); so != null; so =so.getParent() ) {
		if ( so instanceof Frame)
			return (Frame)so;
	}
	return null;
}

public Graphics getGraphics(){
	/* resolved to prevent drawing in invisible state */
	if ( isShowing() )
		return super.getGraphics();
	return null;
}

public Dimension getPreferredSize() {
	return getMinimumSize();
}

public boolean hasFocus () {
	return false;
}

public boolean isEnabled () {
	return Enabled;
}

public void paint() {
	if ( isShowing() ) {
		Graphics g = getGraphics();
		if ( g != null){
			paint( g);
			g.dispose();
		}
	}
}

public void paint( Graphics g) {
	super.paint( g);
	FBorder.drawBorder( g, Width, Height);
	redraw( g);
}

public void redraw () {
	Graphics g = getGraphics();
	if ( g != null ) {
		redraw( g);
		g.dispose();
	}
}

public void redraw( Graphics g) {
}

public void requestFocus() {
	if ( FocusForward != null)
		FocusForward.requestFocus();
	else
		super.requestFocus();
}

public void setBackground ( Color clr ) {
}

public void setBorder( int bs){
	FBorder.setBorder( bs);
}

public void setBounds( int x, int y, int width, int height) {

	Width = width;
	Height = height;

	super.setBounds( x, y, width, height);
	doLayout();
}

void setCursor(){
	setCursor( getCursor());
}

public void setEnabled ( boolean b) {
	Component[] ca = getComponents();
	for ( int i=0; i<ca.length; i++ )
		ca[i].setEnabled( b);
	Enabled = b;
}

public boolean setFocus () {
	return false;
}

public void setFont ( Font fnt ) {
}

public void setForeground ( Color clr ) {
}

public void setVisible ( boolean b) {
	Component[] ca = getComponents();
	for ( int i=0; i<ca.length; i++ )
		ca[i].setVisible( b);
	super.setVisible( b);
}
}
