package biss.awt;

import biss.DateLib;
import biss.ObserverSocket;
import java.awt.Color;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.util.Calendar;
import java.util.Date;
import java.util.Observable;
import java.util.Observer;
import java.util.TimeZone;

/**
 * high level widget to display month overview
 *
 * (C) 1996,97,98 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz, M.Brauch
 */
public class DatePane
  extends Compound
  implements Observer
{
	Date CurDate;
	public ObserverSocket OsDateChanged = new ObserverSocket( this);
	DecoBox Box = null;
	DecoText Text = null;
	DayTable Table;
	int Style;
	public static int ST_PLAIN = 0;
	public static int ST_BOX = 1;
	public static int ST_COMPOUND = 2;
	Button Next = null;
	Button Prev = null;
	private static Calendar Cal = Calendar.getInstance();

public DatePane ( ) {
	this( DateLib.today(), ST_BOX);
}

public DatePane ( Date d) {
	this( d, ST_BOX);
}

public DatePane ( Date d, int style) {
	Style = style;
	setLayout( null);
	
	if ( Style != ST_PLAIN) {
		if ( Style == ST_BOX) {
			Box = new DecoGroup();
			add( Box);
		} else if ( Style == ST_COMPOUND) {
			Box = new DecoBox();
			add( Box);
			Text = new DecoText();
			Text.setAlign( DecoText.DTA_CENTER);
			add( Text);
			Next = new Button( ">");
			Next.OsAction.addObserver( this);
			add( Next);
			Prev = new Button( "<");
			Prev.OsAction.addObserver( this);
			add( Prev);
		}
	}
	
	Table = new DayTable( this);
	add( Table);
	setContents( d);
}

public DatePane ( String edfDate) {
	this( DateLib.dateFromEDF( edfDate), ST_BOX);
}

public DatePane ( int day, int month, int year) {
	this( new Date( year, month, day), ST_BOX);
}

public void doLayout () {
	if ( Box != null) {
		Box.setBounds( 0, 0, Width, Height);
	}

	int xTable, yTable, wTable, hTable;
	
	if ( Style == ST_COMPOUND) {
		int wG = Awt.toPixelXValue(  30);
		int hG = Awt.toPixelYValue(  30);

		int wB = Awt.toPixelXValue( 120);
		int hB = Awt.toPixelYValue( 100);
		

		int wT = Width - 2* wB - 4 * wG;
		int hT = Awt.toPixelYValue( 100);

		int xT = wB + 2 * wG;
		int xB1 = wG;
		int xB2 = wB + 3 * wG + wT;
		int yT = hG;
		int yB = yT;

		Text.setBounds(  xT, yT, wT, hT);
		Prev.setBounds(  xB1, yB, wB, hB);
		Next.setBounds(  xB2, yB, wB, hB);
		
		xTable = wG;
		yTable = 3*hG+hT;
		wTable = Width - 2*xTable;
		hTable = Height-yTable-hG;
	} else {
		xTable = Awt.toPixelXValue( 20);
		yTable = Awt.toPixelYValue( 120);
		wTable = Width-2*xTable;
		hTable = Height-yTable-Awt.toPixelYValue( 20);
	}
	
	Table.setBounds( xTable, yTable, wTable, hTable);
}

public Date getCurrentDate () {
	return CurDate;
}

void newMonth(){
	String pre = DateLib.MonthShorts[CurDate.getMonth()]+" "+
	Integer.toString( 1900+CurDate.getYear());
	if      ( Style == ST_BOX) ((DecoGroup)Box).setName( pre);
	else if ( Style == ST_COMPOUND) Text.setContents( pre ); 
	if ( isShowing() ){
		Table.blank();
		Table.redraw();
	}
}

/**
 * triggered by selecting a day from the DayTable
 * parameter: current date
 */
public void notifyChanged(){
	OsDateChanged.notifyObservers( CurDate);
}

public void setColors( Color clrDays, Color clrToday, Color clrDayNames, Color clrSunday) {
	Table.setColors( clrDays, clrToday, clrDayNames, clrSunday);
}

public void setContents ( Date dt) {
	CurDate = new Date( dt.getTime() );
	DateLib.trim( CurDate);
	Table.newDayTable();
}

public void setContents ( int day, int month, int year) {
	setContents( new Date( year-1900, month-1, day));
}

public void setFont ( Font fnt) {
	Table.setFont( fnt);
}

public void update ( Observable obs, Object arg ) {
	if ( Style == ST_COMPOUND){
		Cal.setTime( getCurrentDate());
		if ( obs == Next.OsAction) {
			Cal.add( Calendar.MONTH, 1);
		} else if ( obs == Prev.OsAction) {
			Cal.add( Calendar.MONTH, -1);
		}
		setContents( Cal.getTime());
		notifyChanged();
	}
}
}

class DayTable
  extends Widget
  implements HotKeyProcessor
{
	int[][] DayArray;
	FontMetrics Fm;
	DatePane Master;
	static FontMetrics DefFm;
	Color ClrDayNames = Color.blue;
	Color ClrSunday = Color.red;
	Color ClrDays = Color.black;
	Color ClrToday = Color.green;

static {
	DefFm = Awt.getFontMetrics( Awt.ButtonFont);
}

public DayTable ( DatePane master) {
	Master = master;
	setFont( null);
	registerHotKeys();
}

public void mouse1Down () {

	int cw  = 2*Fm.charWidth( '0');
	int cd  = (Width-7*cw)/8;
	int rh  = Fm.getHeight();
	int rd  = (Height-7*rh)/8;
	int xIdx = Math.min( Mouse.Pos.x/(cw+cd), 6);
	int yIdx = Math.min( (Mouse.Pos.y-rh-rd)/(rh+rd), 5 );
	int dn;

	if ( (dn=DayArray[xIdx][yIdx])!=0) {
		Master.CurDate.setDate( dn);
		Master.notifyChanged();
		redraw();
	} 
}

void newDayTable(){
	DayArray = DateLib.getDayArray( Master.CurDate.getMonth()+1,
	                                Master.CurDate.getYear() );
	Master.newMonth();
}

public void paint( Graphics g) {
	blank( g);
	redraw( g);
}

public void processKey (Object sel) {
	Date d = Master.CurDate;
	int mn = d.getMonth();

	if ( sel.equals( "next"))
		Master.CurDate = DateLib.nextDay( d);
	else if ( sel.equals( "prev"))
		Master.CurDate = DateLib.prevDay( d);

	if ( mn!=Master.CurDate.getMonth() )
		newDayTable();
	else
		redraw();

	Master.notifyChanged();
}

public void redraw ( Graphics g) {

	int zw  = Fm.charWidth( '0');
	int cw  = 2*zw;
	int cd  = (Width-7*cw)/8;
	int dc  = Fm.getDescent();
	int rh  = Fm.getHeight();
	int rd  = (Height-7*rh)/8;
	int dn, dof, dim = DateLib.dayInMonth( Master.CurDate);

	g.setColor( ClrDayNames);
	g.setFont( getFont() );
	for ( int dy=0; dy<7; dy++)
		g.drawString( DateLib.DayShorts[dy], cd+dy*(cw+cd), rd+rh-dc);

	for ( int dy=0; dy<7; dy++) {
		for ( int wk=0; wk<6; wk++) {
			if ( (dn=DayArray[dy][wk])!=0){
				if ( dn<10) dof=zw;
				else        dof = 0;
				if (dn==dim) g.setColor( ClrToday);
				else if ( dy==6) g.setColor( ClrSunday);
				else g.setColor( ClrDays);
				g.drawString( Integer.toString(dn), dof+cd+dy*(cd+cw), (wk+2)*(rd+rh)-dc );
			}
		}
	}
}

void registerHotKeys(){
	addHotKey( 'n', HotKey.Control, this, "next");
	addHotKey( 'p', HotKey.Control, this, "prev");
}

public void setColors( Color clrDays, Color clrToday, Color clrDayNames, Color clrSunday) {
	ClrDays     = clrDays;
	ClrToday    = clrToday;
	ClrDayNames = clrDayNames;
	ClrSunday   = clrSunday;
}

public void setFont ( Font fnt) {
	Font uf = ( fnt != null) ? fnt : Awt.ButtonFont;
	super.setFont( uf);

	if ( uf.equals( Awt.ButtonFont))
		Fm = DefFm;
	else
		Fm = Awt.getFontMetrics( uf);

	if ( isShowing() )
		redraw();
}
}
