package biss.awt;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;

/**
 * static decoration image
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class DecoImage
  extends DecoBox
{
	Image Img;
	final public static int TILED = 2;
	int Style;
	final public static int CENTERED = 0;
	final public static int STRETCHED = 1;

public DecoImage () {
	Style = CENTERED;
}

public DecoImage (String gifPath) {
	Style = CENTERED;
	setImage ( gifPath);
}

public DecoImage (String gifPath, boolean stretch) {
	Style = STRETCHED;
	setImage ( gifPath);
}

public DecoImage (String gifPath, int style) {
	Style = style;
	setImage ( gifPath);
}

public void blank (Graphics g, Color cb) {
	g.setColor( cb);
	g.fillRect( 0, 0, Width+1, Height+1);
}

public void paint (Graphics g) {
	int iw, ih, bw;
	
	if ( ! isShowing() )
		return;
	if ( Img != null){
		switch ( Style ) {
			case CENTERED:
				iw = Img.getWidth( this);
				ih = Img.getHeight( this);
				if ( (iw > 0) && (ih > 0) ){
					bw = BoxBorder.Ext;
					GraphicsLib.drawImageCentered( g, Img, iw, ih, 0, 0,
		   			                            Width-2*bw, Height-2*bw, this);
				}
				break;
			case STRETCHED:
				bw = BoxBorder.Ext;
				g.drawImage( Img, bw, bw, Width-2*bw, Height-2*bw, this);
				break;
			case TILED:
				iw = Img.getWidth( this);
				ih = Img.getHeight( this);
				if ( (iw > 0) && (ih > 0) ) {
					for ( int y=0; y<Height; y+=ih ) {
						for ( int x=0; x<Width; x+=iw)
							g.drawImage( Img, x, y, iw, ih, this);
					}
				}
		}
	}
	drawBorder( g);
}

public void setImage ( Image img) {
	if ( img != null) {
		Img = img;

		if ( isShowing() ){
			Graphics g = getGraphics();
			if ( g != null ) {
				blank( g, getParent().getBackground() );
				paint(g);
				g.dispose();
			}
		}
	}
}

public void setImage (String gifPath) {
	setImage( Awt.getImage( gifPath));
}

public void setImageClass( String imgCls){
	setImage( ClassImageSource.getImage( imgCls));
}

public void setStyle ( int style) {
	Style = style;
	if ( isShowing() )
		repaint();
}
}
