package biss.awt;

import biss.CommandQueue;
import biss.ObserverSocket;
import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.*;
import java.awt.peer.ActiveEvent;
import java.util.Observable;
import java.util.Observer;

/**
 *
 * (C) 1996-98 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz, M.Brauch
 */
public class DialogWindow
  extends Dialog
  implements Observer, GUIHandlerInterface, WindowListener
{
	Layouter Lo = new Layouter( this);
	boolean WasCanceled;
	Component DefCom;
	GUIHandler EventHandler = new GUIHandler( this);
	public ObserverSocket OsCommand = new ObserverSocket( this);
	public ObserverSocket OsOpen = new ObserverSocket( this);
	public ObserverSocket OsClose = new ObserverSocket( this);
	final public static int MODAL = 2;
	final public static int DISPATCHED = 1;
	final public static int OWNED = 0;
	int Mode;
	LocalDispatcher Dsp;
	Initializer Init;
	boolean DisableDispose = false;

public DialogWindow( Frame owner, String title, boolean modal){
	this( owner, title, modal ? MODAL : DISPATCHED, null);
}

public DialogWindow( Frame owner, String title, boolean modal, Initializer init){
	this( owner, title, modal ? MODAL : DISPATCHED, init);
}

public DialogWindow( Frame owner, String title, int mode){
	this( owner, title, mode, null);
}

public DialogWindow( Frame owner, String title, int mode, Initializer init){
	super( owner, title, mode == MODAL);
	Init = init;
	Mode = mode;
	EventHandler.Wnd = this;
	addWindowListener( this);
	setBackground( Awt.WindowBackClr);
}

public void disableDispose() {
	DisableDispose = true;
}

public void dispose() {

	DisableDispose = false;
	OsClose.notifyObservers( this);
	
	if ( DisableDispose)
		return;

	if ( Dsp != null){
		Dsp.terminate();
		Awt.enableComponents( getParent() );
	}

	super.dispose();
}

public Rectangle getDlgUnitBounds( int x, int y, int w, int h){
	return Awt.toPixelUnits( new Rectangle ( x, y, w, h));
}

public GUIHandler getGUIHandler() {
	return EventHandler;
}

protected void openIn( Rectangle r) {
	setBounds( r);
	//start local dispatching
	setVisible( true);
}

protected void openIn( int x, int y, int width, int height ) {
  setBounds( x, y, width, height);
  setVisible( true);
}

protected void openInDlgUnits( int x, int y, int w, int h){
	openIn( getDlgUnitBounds( x, y, w, h));
}

protected void process() {
	dispose();
}

public void setCenterBounds(){
	Dimension scrd = Awt.DefToolkit.getScreenSize();

	//pack to get insets for terminal getPreferredSize() methods
	pack();

	Dimension dd = getSize();
	setBounds( (scrd.width-dd.width)/2, (scrd.height-dd.height)/2,
							dd.width, dd.height);
}

public void setDlgUnitCenterBounds( int w, int h) {
	Rectangle r = getDlgUnitBounds( 0, 0, w, h);
	Dimension scrd = Awt.DefToolkit.getScreenSize();
	r.setLocation( (scrd.width-r.width)/2, (scrd.height-r.height)/2);
	setBounds( r);
}

protected void setInitialFocus(){
	if ( DefCom != null)
		CommandQueue.append( new SetFocusCmd( DefCom));
	else if ( getComponentCount() > 0)
		CommandQueue.append( new SetFocusCmd( getComponent(0)) );
}

public void setVisible( boolean isVisible) {
	if ( ! isVisible) {
		if ( Mode == OWNED)
			super.setVisible( isVisible);
		return;
	}

	if ( isShowing() )
		toFront();
	else {
		super.setVisible( true);
		if ( Mode == DISPATCHED){
			Dsp = new LocalDispatcher();
			Dsp.dispatchTilTerminate();
		}
	}
}

public void update( Observable obs, Object arg) {
}

public boolean wasCanceled() {
	return WasCanceled;
}

public void windowActivated(WindowEvent e){
}

public void windowClosed(WindowEvent e){
}

public void windowClosing(WindowEvent e){
	WasCanceled = true;
	process();
}

public void windowDeactivated(WindowEvent e){
}

public void windowDeiconified(WindowEvent e){
}

public void windowIconified(WindowEvent e){
}

public void windowOpened( WindowEvent e) {

	if ( Mode == DISPATCHED ) Awt.disableComponents( getParent() );

	if ( Init != null) Init.init( this);

	OsOpen.notifyObservers( this);
	requestFocus();
	setInitialFocus();
}
}
