package biss.awt;

import biss.DirEnumerator;
import biss.FileExtFilter;
import biss.StringLib;
import biss.VectorLib;
import biss.VectorSorter;
import biss.awt.Awt;
import biss.awt.Button;
import biss.awt.Choice;
import biss.awt.DecoText;
import biss.awt.DialogWindow;
import biss.awt.Layouter;
import biss.awt.List;
import biss.awt.TextField;
import java.awt.Color;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

/**
 * Implementation of a FileDialog by means of biss.awt classes (NOT
 * native).
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
class DirEntry
  implements SelfDrawingObject
{
	File Dir;
	String Name;
	static Image DirImg = ClassImageSource.getImage( "biss.awt.DirImage");

public DirEntry ( File f) {
	Dir = f;
	Name = f.getName();
}

public DirEntry ( File f, String s) {
	Dir = f;
	Name = s;
}

public int drawHeight () {
	return Awt.SysFontHeight;
}

public void drawSelfIn (Object pane) {
	List op			= (List) pane;
	Graphics g	= op.DrawGraphics;
	Rectangle r	= op.DrawRect;
	int x0 			= r.x + Awt.SysFontHeight / 2;

	GraphicsLib.drawImageVCentered( g, DirImg, x0, r.y, r.height, op);

	x0 += Awt.SysFontHeight;
	g.setFont( Awt.SysFont);
	g.drawString( Name, x0,
	              r.y + r.height - (r.height - Awt.SysFontHeight) / 2 -
	              Awt.SysFontDesc );
}

boolean isExpandPos (Point pt) {
	return pt.x < 3 * Awt.SysFontHeight / 2;
}

public String label () {
	return Name;
}

public int maxDrawWidth () {
	return 300;
}
}

public class FileDialog
  extends DialogWindow
  implements Observer
{
	DecoText PathText = new DecoText("File");
	TextField PathEntry = new TextField();
	DecoText DriveText = new DecoText("Drive");
	List DrivePane = new List();
	DecoText DirText = new DecoText("Directory");
	List DirPane = new List();
	DecoText FileText = new DecoText("Files");
	List FilePane = new List();
	Button OkBtn = new Button("~Ok");
	Button CancelBtn = new Button("~Cancel");
	DecoText FilterText = new DecoText("Filter");
	File CurPath;
	Layouter Lo = new Layouter( this);
	FilenameFilter Filter;
	Choice FilterChoice = new Choice();

public FileDialog ( Frame owner) {
	this( owner, "FileDialog", null);
}

public FileDialog ( Frame owner, String initPath) {
	this ( owner, "FileDialog", initPath);
}

public FileDialog ( Frame owner, String title, String initPath) {
	super ( owner, title, false);

	if ( initPath == null)
		initPath = System.getProperty( "user.home", ".");
	if ( ! initPath.endsWith( File.separator) )
		initPath += File.separator;

	CurPath = new File( initPath);
	open();
}

void checkExpand (Point p) {
	DirEntry de = (DirEntry)DirPane.getSelection();
	if ( ( p == null) || (de.isExpandPos( p) )){
		CurPath = new File( de.Dir.getPath());
		updateDirPane();
	}
}

public String getDirectory ( ) {
	// java.awt.FileDialog interface
	return ( (CurPath != null) ? CurPath.toString() : null );
}

public String getFile ( ) {
	// java.awt.FileDialog interface
	File f = getSelection();
	return ((f != null) ? f.toString() : null);
}

public static File getFile ( Frame owner ) {
	FileDialog fd = new FileDialog( owner);
	return fd.getSelection();
}

public static File getFile ( Frame owner, String initPath ) {
	FileDialog fd = new FileDialog( owner, initPath);
	return fd.getSelection();
}

public FilenameFilter getFilenameFilter( ) {
	// java.awt.FileDialog interface
	return Filter;
}

public File getSelection ( ) {
	if ( WasCanceled)
		return null;
	return new File( PathEntry.getContents() );
}

void init () {
	DirPane.OsSelection.addObserver( this);
	DirPane.OsMouseSelection.addObserver( this);
	DirPane.OsProcess.addObserver( this);
	FilePane.OsSelection.addObserver( this);
	FilePane.OsProcess.addObserver( this);
	DrivePane.OsSelection.addObserver( this);
	PathEntry.OsEnter.addObserver( this);
	FilterChoice.OsSelection.addObserver( this);
	OkBtn.OsAction.addObserver( this);
	CancelBtn.OsAction.addObserver( this);

	initDrives();
	initFilter();
	updateDirPane();

	setDlgUnitCenterBounds( 2500, 2000);
	DefCom = PathEntry;
}

void initDrives () {
	Vector v = new Vector();
	if( (Awt.AwtToolkit == Awt.TK_WIN) ||
	    (Awt.AwtToolkit == Awt.TK_OS2) ){
		String[] das = { "C:", "D:", "E:", "F:", "G:", "H:" };
		for ( int i=0; i<das.length; i++){
			File da = new File( das[i] + File.separator);
			if ( da.exists() )
				v.addElement( das[i]);
		}
	}
	else
		v.addElement( File.separator);

	DrivePane.setContents( v);    
}

void initFilter () {
	String[] fa = { ".*", ".java", ".class" };
	FilterChoice.setContents( VectorLib.vectorFromArray( fa));
}

public static void main ( String[] args ) {
	Frame fr = new Frame( "OwnerFrame");
	fr.setBounds( 100, 100, 100, 100);
	fr.show();
	
	String iPath = args.length > 0 ? args[0] : null;
	FileDialog fd = new FileDialog( fr, iPath);

	System.out.println( fd.getSelection() );
	System.exit( 0);
}

public void open() {
	Lo.add( PathText, 0, 0, 0, 0, true, 50, 50, 300, 250, 0);
	Lo.add( PathEntry, 0, 0, 1000, 0, true, 300, 50, -50, 250, 0);
	Lo.add( DriveText, 0, 0, 0, 0, true, 50, 250, 300, 350, 0);
	Lo.add( DrivePane, 0, 0, 0, 1000, true, 50, 350, 300, -300, 0);
	Lo.add( DirText, 0, 0, 600, 0, true, 300, 250, 0, 350, 0);
	Lo.add( DirPane, 0, 0, 600, 1000, true, 300, 350, 0, -300, 4);
	Lo.add( FileText, 600, 0, 1000, 0, true, 0, 250, -50, 350, 0);
	Lo.add( FilePane, 600, 0, 1000, 1000, true, 0, 350, -50, -300, 0);
	Lo.add( OkBtn, 0, 1000, 333, 1000, true, 50, -250, -20, -50, 0);
	Lo.add( CancelBtn, 333, 1000, 666, 1000, true, 20, -250, -20, -50, 0);
	Lo.add( FilterText, 700, 1000, 700, 1000, true, 0, -250, 300, -50, 0);
	Lo.add( FilterChoice, 700, 1000, 1000, 1000, true, 300, -250, -50, -50, 0);

	init();

	Rectangle bounds = getBounds();

	if ( bounds.width > 0)
		setVisible( true);
	else {
		bounds.setBounds( 370, 903, 2496, 1996);
		Awt.toPixelUnits( bounds);
		setBounds( bounds);
		setVisible( true);
	}
}

public void setDirectory (String dir ) {
	// java.awt.FileDialog interface
	File f = new File( dir);
	if ( f.exists() && f.isDirectory() && f.canRead() )
		CurPath = f;
	updateDirPane();
}

public void setFile (String file ) {
	// java.awt.FileDialog interface
	File f = new File( file);
	if ( f.exists() && f.canRead() )
		CurPath = new File( f.getPath());
	updateDirPane();
	DirPane.setSelection( CurPath, true);
	FilePane.setSelection( f.getName(), true );
}

public void setFilenameFilter( FilenameFilter filter ) {
	// java.awt.FileDialog interface
	Filter = filter;
	if ( filter instanceof FileExtFilter){
		FilterChoice.setSelection( ((FileExtFilter)filter).extension() );
		updateFiles();
	}
}

public void update ( Observable obs, Object arg ) {
	if ( obs == PathEntry.OsEnter){
		File f = new File( PathEntry.getContents() );
		if ( f.exists() ){
			CurPath = f;
			if ( f.isDirectory() )
				updateDirPane();
			else
				process();
		} 
	}
	else if ( obs == FilterChoice.OsSelection)
		updateFiles();
	else if ( obs == DrivePane.OsSelection){
		updateDrive();
	}
	else if ( obs == DirPane.OsSelection)
		updateFiles();
	else if ( obs == DirPane.OsProcess)
		checkExpand( null);
	else if ( obs == DirPane.OsMouseSelection)
		checkExpand( (Point)arg);
	else if ( obs == FilePane.OsSelection)
		updatePath();
	else if ( obs == FilePane.OsProcess)
		process();
	else if ( obs == OkBtn.OsAction)
		process();
	else if ( obs == CancelBtn.OsAction){
		WasCanceled = true;
		process();
	}
}

void updateDirPane ( ) {
	Vector v = new Vector();
	DirEnumerator de = new DirEnumerator( CurPath.getPath(), 1);
	for ( Enumeration e = de.elements(); e.hasMoreElements();)
		v.addElement( new DirEntry( (File)e.nextElement()) );

	v.setElementAt( new DirEntry( new File( CurPath.getPath()), "."), 0);
	if ( ! CurPath.getPath().equals( File.separator)){
		String pd = CurPath.getParent();
		if ( pd == null) pd = File.separator;
		v.insertElementAt( new DirEntry( new File( pd), ".."), 1);
	}

	DirPane.setContents( v);
	FilePane.setContents( null);
	PathEntry.setContents( CurPath.getPath() );
}

void updateDrive () {
	CurPath = new File( (String)DrivePane.getSelection() + File.separator);
	updateDirPane();
	updatePath();
	updateFiles();
}

void updateFiles () {
	DirEntry de = (DirEntry)DirPane.getSelection();
	if ( de != null) {
		CurPath = de.Dir;
		String[] fls = CurPath.list( new FileExtFilter( (String)FilterChoice.getSelection()));
		Vector v = VectorLib.vectorFromArray( fls);
		VectorLib.quickSort( v, new FileSorter() );
		PathEntry.setContents( CurPath.getPath() );
		FilePane.setContents( v);
	}
}

void updatePath () {
	String file = (String)FilePane.getSelection();
	if ( file != null)
		PathEntry.setContents( CurPath.getPath() + File.separator + file);
}
}

class FileSorter
  implements VectorSorter
{
public int compare (Object a, Object b) {
	String f1 = (String)a;
	String f2 = (String)b;
	return f1.compareTo( f2);
}
}
