package biss.awt;

import biss.Biss;
import biss.FileLib;
import biss.awt.Awt;
import biss.awt.BarMenu;
import biss.awt.Layouter;
import biss.awt.TextArea;
import biss.awt.TopWindow;
import java.awt.FileDialog;
import java.awt.Rectangle;
import java.io.File;
import java.util.Observable;
import java.util.Observer;

/**
 * simple file editor ( to be extended). Example of a combined
 * physical and logical view class
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class FileEditor
  extends TopWindow
  implements Observer
{
	BarMenu BMenu = new BarMenu();
	TextArea FPane = new TextArea();
	String FileName;
	Layouter Lo = new Layouter( this);
	String PathName;

public FileEditor () {
	open();
}

public FileEditor ( String pathName) {
	File f = Biss.getEnvFile( pathName);
	if ( f != null)
		PathName = f.getAbsolutePath();
	FileName = pathName;
	open();
}

public void init () {
	setBarMenu();
	BMenu.OsCommand.addObserver( this);
	if ( PathName != null)
		openFile( PathName);
}

public static void main ( String[] args) {
	String pathName = (args.length > 0) ? args[0] : null;
	new FileEditor( pathName );
}

public void open() {
	Lo.add( BMenu, 0, 0, 1000, 0, true, 0, 0, 0, 150, 0);
	Lo.add( FPane, 0, 0, 1000, 1000, true, 0, 150, 0, 0, 0);

	init();

	Rectangle bounds = getBounds();

	if ( bounds.width > 0)
		setVisible( true);
	else {
		bounds.setBounds( 650, 1090, 1963, 2430);
		Awt.toPixelUnits( bounds);
		setBounds( bounds);
		setVisible( true);
	}
}

void openFile () {
	FileDialog fd = new FileDialog( this, "Open file:", FileDialog.LOAD);
	fd.setVisible( true);
	String d = fd.getDirectory();
	String f = fd.getFile();
	if ( (d != null) && (f != null)){
		FPane.repaint();
		openFile( d + f);
	}
}

public void openFile ( String pathName) {
	File f = new File( pathName);
	FileName = pathName;
	updateTitle();

	if ( f.exists() )
		FPane.setContents( FileLib.getUTFFileDataAsString( f));
	else
		FPane.setContents( (String)null);
}

void saveFileAs () {
	FileDialog fd = new FileDialog( this, "Save file:", FileDialog.SAVE);
	fd.setVisible( true);
	String fn = fd.getDirectory() + fd.getFile();
	FPane.repaint();
	saveFileAs( fn);
}

boolean saveFileAs ( String pathName) {
	if ( pathName == null)
		return false;

	FileName = pathName;
	updateTitle();

	return FileLib.setFileData( pathName, FPane.getContents(), true);
}

void setBarMenu () {
	Menu m = new Menu();
	m.add( "~Open");
	m.add( "Save ~As");
	m.add( "~Save.");
	m.add( "~Close.");
	BMenu.setMenu( m);
}

public void update ( Observable slot, Object arg) {
	if ( slot == BMenu.OsCommand ){
		if ( "Open".equals( arg))
			openFile();
		else if ( "Close.".equals( arg))
			dispose();
		else if ( "Save.".equals( arg))
			saveFileAs( FileName);
		else if ( "Save As".equals( arg))
			saveFileAs();
	}
	else
		super.update( slot, arg);
}

void updateTitle () {
	setTitle( "Edit: " + FileName);
}
}
