package biss.awt;

import java.awt.Component;
import java.awt.Container;

/**
 * delegation object class to implement a focus switching mechanism for
 * arbitrary container widgets
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class FocusHandler
{
public static boolean focusNext ( Container co) {
	Component ca[] = co.getComponents();
	int       nc   = co.getComponentCount();
	FocusSetInterface bc, fbc = null;
	boolean set = false;

	for ( int i=0; i<nc; i++){
		Component c = ca[i];
		if ( c == null)
			continue;
 		if ( ! (c instanceof FocusSetInterface) ) {
			if ( c instanceof Container) {
				if ( focusNext( (Container)c) )
					return true;
			}
			continue;
		}
		bc = (FocusSetInterface) c;
		//first component that can get focus
		if ( (fbc == null) && (bc.canGetFocus() ) ) 
			fbc = bc;

		if ( set){
			if ( bc.setFocus()){
				return true;
			}
		}
		else if ( bc.hasFocus() ){
			if ( bc instanceof Compound) {
				if ( ((Compound)bc).focusNext() )
					return true;
			}
			set = true;
		}
	}

	if ( fbc != null) {
		fbc.setFocus();
		return true;
	}
	
	return false;
}

public static void focusNextX ( Container co) {
	Component ca[] = co.getComponents();
	int       nc   = co.getComponentCount();
	FocusSetInterface bc, fbc = null;
	boolean set = false;

	for ( int i=0; i<nc; i++){
		Component c = ca[i];
		if ( (c == null) || ! (c instanceof FocusSetInterface) )
			continue;
		bc = (FocusSetInterface) c;
		//first component that can get focus
		if ( (fbc == null) && (bc.canGetFocus() ) ) 
			fbc = bc;

		if ( set){
			if ( bc.setFocus()){
				return;
			}
		}
		else if ( bc.hasFocus() ){
			if ( bc instanceof Compound) {
				if ( ((Compound)bc).focusNext() )
					return;
			}
			set = true;
		}
	}

	if ( fbc != null)
		fbc.setFocus();
}

public static boolean focusPrev ( Container co) {
	Component ca[] = co.getComponents();
	int       nc   = co.getComponentCount();
	FocusSetInterface bc, lbc = null;
	boolean set = false;

	for ( int i=nc - 1; i>=0; i--){
		Component c = ca[i];
		if (c == null)
			continue;
		if ( ! (c instanceof FocusSetInterface) ) {
			if ( c instanceof Container ){
				if ( focusPrev( (Container)c) )
					return true;
			}
			continue;
		}
		bc = (FocusSetInterface) c;
		if ( (lbc == null) && ( bc.canGetFocus() ) )
			lbc = bc;
		if ( set){
			if ( bc.setFocus())
				return true;
		}
		else if ( bc.hasFocus() ){
			if ( bc instanceof Compound){
				if ( ((Compound)bc).focusPrev() )
					return true;
			}
			set = true;
		}
	}

	if ( lbc != null){
		lbc.setFocus();
		return true;
	}
	
	return false;
}

public static void focusPrevX ( Container co) {
	Component ca[] = co.getComponents();
	int       nc   = co.getComponentCount();
	FocusSetInterface bc, lbc = null;
	boolean set = false;

	for ( int i=nc - 1; i>=0; i--){
		Component c = ca[i];
		if ( (c == null) || ! (c instanceof FocusSetInterface) )
			continue;
		bc = (FocusSetInterface) c;
		if ( (lbc == null) && ( bc.canGetFocus() ) )
			lbc = bc;
		if ( set){
			if ( bc.setFocus())
				return;
		}
		else if ( bc.hasFocus() ){
			if ( bc instanceof Compound){
				if ( ((Compound)bc).focusPrev() )
					return;
			}
			set = true;
		}
	}

	if ( lbc != null)
		lbc.setFocus();
}

public static boolean includesFocusPart ( Container co) {
	Component ca[] = co.getComponents();
	int       nc   = co.getComponentCount();

	for ( int i=0; i<nc; i++){
		Component c = ca[i];
		if ( c != null) {
			if ( c instanceof FocusSetInterface){
				FocusSetInterface bc = (FocusSetInterface) c;
				if ( bc.hasFocus() )
					return true;
			}
		}
	}

	return false;
}
}
