package biss.awt;

import biss.VectorLib;
import biss.awt.Awt;
import biss.awt.Button;
import biss.awt.Checkbox;
import biss.awt.Choice;
import biss.awt.DecoText;
import biss.awt.DialogWindow;
import biss.awt.Label;
import biss.awt.Layouter;
import biss.awt.SpinButton;
import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

/**
 * Implementation of a FontDialog by means of biss.awt classes (NOT
 * native).
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class FontDialog
  extends DialogWindow
  implements Observer
{
	Font Selection = null;
	DecoText FamText = new DecoText("Family");
	Choice FamChoice = new Choice();
	DecoText SizeText = new DecoText("Size");
	SpinButton SizeSpin = new SpinButton();
	Checkbox BoldBtn = new Checkbox("Bold");
	Checkbox ItalBtn = new Checkbox("Italic");
	DecoText PreText = new DecoText("Preview");
	Label Preview = new Label("The quick brown Fox ...");
	Button OkBtn = new Button("~Ok");
	Button CancelBtn = new Button("~Cancel");

public FontDialog ( Frame owner) {
	super( owner, "FontDialog", DISPATCHED);
	open();
}

public Font getSelection() {
	if ( WasCanceled )
		return null;
	return Selection;
}

void init() {
	queryFonts();

	FamChoice.OsSelection.addObserver( this);
	SizeSpin.OsSpinChanged.addObserver( this);
	BoldBtn.OsAction.addObserver( this);
	ItalBtn.OsAction.addObserver( this);
	OkBtn.OsAction.addObserver( this);
	CancelBtn.OsAction.addObserver( this);

	Preview.setCarved( false);
	Preview.setBackground( Color.white);
	Selection = new Font( (String)FamChoice.getSelection(),
	                           (BoldBtn.isChecked() ? Font.BOLD : Font.PLAIN) |
	                           (ItalBtn.isChecked() ? Font.ITALIC : 0 ),
	                           SizeSpin.getSelection() );
	Preview.setFont( Selection);

	setDlgUnitCenterBounds( 2000, 1266);
	DefCom = OkBtn;
}

public static void main ( String[] args ) {
	Frame fr = new Frame ( "OwnerWnd");
	fr.setBounds( 100, 100, 100, 100);
	fr.show();
	FontDialog fd = new FontDialog( fr);
	System.out.println( fd.getSelection() );
	System.exit( 0);
}

public void open() {
	Lo.add( FamText, 0, 0, 0, 0, true, 53, 53, 400, 206, 0);
	Lo.add( FamChoice, 0, 0, 1000, 0, true, 420, 53, -46, 206, 0);
	Lo.add( SizeText, 0, 0, 0, 0, true, 53, 220, 400, 373, 0);
	Lo.add( SizeSpin, 0, 0, 0, 0, true, 420, 220, 840, 373, 0);
	Lo.add( BoldBtn, 0, 0, 0, 0, true, 900, 220, 1366, 373, 0);
	Lo.add( ItalBtn, 0, 0, 0, 0, true, 1420, 220, 1886, 373, 0);
	Lo.add( PreText, 0, 0, 0, 0, true, 53, 400, 400, 800, 0);
	Lo.add( Preview, 0, 0, 1000, 0, true, 420, 400, -46, 800, 0);
	Lo.add( OkBtn, 0, 0, 0, 0, true, 420, 840, 1020, 1026, 0);
	Lo.add( CancelBtn, 0, 0, 0, 0, true, 1080, 840, 1680, 1026, 0);

	init();

	Rectangle bounds = getBounds();

	if ( bounds.width > 0)
		setVisible( true);
	else {
		bounds.setBounds( 3, 2283, 1996, 1263);
		Awt.toPixelUnits( bounds);
		setBounds( bounds);
		setVisible( true);
	}
}

void queryFonts () {
	String[] sa = Toolkit.getDefaultToolkit().getFontList();
	FamChoice.setContents( VectorLib.vectorFromArray( sa));
	SizeSpin.setRange( 2, 50);
	SizeSpin.setSelection( 10);
}

public void update ( Observable obs, Object arg ) {
	Font f = Preview.getFont();

	if ( obs == FamChoice.OsSelection){
		Selection = new Font( (String)arg, f.getStyle(), f.getSize() );
		Preview.setFont( Selection);
	}
	else if ( obs == SizeSpin.OsSpinChanged){
		Selection = new Font( f.getName(), f.getStyle(), SizeSpin.getSelection() );
		Preview.setFont( Selection);
	}
	else if ( obs == BoldBtn.OsAction){
		Selection = new Font( f.getName(),
	                           BoldBtn.isChecked() ? (f.getStyle() | Font.BOLD) :
	                           (f.getStyle() &~ Font.BOLD),
	                           f.getSize() );
		Preview.setFont( Selection);
	}
	else if ( obs == ItalBtn.OsAction){
		Selection = new Font( f.getName(),
	                           ItalBtn.isChecked() ? (f.getStyle() | Font.ITALIC) :
	                           (f.getStyle() &~ Font.ITALIC),
	                           f.getSize() );
		Preview.setFont( Selection);
	}
	else if ( obs == OkBtn.OsAction){
		process();
	}
	else if ( obs == CancelBtn.OsAction){
		WasCanceled = true;
		process();
	}
}
}
