package biss.awt;

import biss.ObserverSocket;
import java.applet.Applet;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Vector;

/* (/** causes javac to fail)
 * generic event handling support for biss.awt classes.
 * Adapts to specific clients by examining their interfaces
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class GUIHandler
  implements KeyListener, MouseListener, MouseMotionListener, FocusListener
{
	boolean HasFocus;
	HotKeyHashtab HotKeys;
	Component OC;
	Window Wnd;
	CmdInterface Ci;
	FocusInterface Fi;
	KbdInterface Ki;
	MouseInterface Mi;
	MouseMotionInterface Mmi;
	GUIHandler Super;
	boolean IsDisabled;
	static SingleInstInterface SingleInstWnd;
	static boolean SkipTyped;
	static GUIHandler CurrentHandler;
	boolean ProtectFocus;
	Container DefKeyHandler;

public GUIHandler (Component owner) {
	OC = owner;
	Key.ActiveKey = 0;
	Key.ActiveKeyMods = 0;

	if ( owner instanceof KbdInterface){
		owner.addKeyListener( this);
		Ki = (KbdInterface)owner;
	}
	if ( owner instanceof CmdInterface)
		Ci = (CmdInterface)owner;
	if ( owner instanceof FocusInterface){
		owner.addFocusListener( this);
		Fi = (FocusInterface)owner;
	}
	if ( owner instanceof MouseInterface){
		owner.addMouseListener( this);
		Mi = (MouseInterface)owner;
	}
	if ( owner instanceof MouseMotionInterface){
		owner.addMouseMotionListener( this);
		Mmi = (MouseMotionInterface)owner;
	}
	
	if ( Awt.IsApplet) {
		for ( Container c = OC.getParent();	c != null; c = c.getParent() ) {
			if ( c instanceof Applet) {
				DefKeyHandler = c;
				break;
			}
		}
	}
}

public GUIHandler ( Window w) {
	this( (Component) w);
	Wnd = w;
}

public boolean addHotKey( HotKey hk){
	if ( HotKeys == null)
		HotKeys = new HotKeyHashtab( 20, 2);
	if ( HotKeys.containsKey( hk.HashCode))
		return false;
	HotKeys.put( hk.HashCode, hk);
	return true;
}

public boolean addHotKey( int c, int mods, HotKeyProcessor p, Object sel){
	int key = Key.isShiftDown( mods) ? (c | 32) : c;
	return addHotKey( new HotKey( key, mods, p, sel));
}

public void addHotKeys( Vector v){
	int s = v.size();
	for ( int i=0; i<s; i++)
		addHotKey( (HotKey)v.elementAt( i));
}

void checkDefaultKey( int key, int mod ){
	Container c = ( DefKeyHandler != null) ? DefKeyHandler : Wnd;
	
	switch( key) {
	case Key.Tab:
	case Key.Tab+96:  // ctrl offset
		// (lone/alt/ctrl) + tab to step forward
		if (mod != InputEvent.SHIFT_MASK )
			FocusHandler.focusNext( c);
		// shift + tab to step backwards
		else
			FocusHandler.focusPrev( c); 
		break;
	}
}

boolean checkHotKey( int key, int mod ){
	int ik;
	HotKey  hk;

	if ( (HotKeys != null) && ( !HotKeys.isEmpty())){
		ik = key + mod*256;
		if ( HotKeys.containsKey( ik)) {
			hk = (HotKey)HotKeys.get( ik);
			if ( ! hk.process() ) {
				if ( Ci != null)
					Ci.getOsCommand().notifyObservers( hk.Selector);
			}
			return true;
		}
	}

	if ( Super != null)
		return Super.checkHotKey( key, mod);

	return false;
}

public boolean disposeSingleInstWnd () {
	return disposeSingleInstWnd( OC );
}

public static boolean disposeSingleInstWnd ( Component co) {
	Awt.DefToolkit.sync();

	if ( SingleInstWnd != null){
		if ( SingleInstWnd.disposeRequest( co)) {
			SingleInstWnd = null;
			return true;
		}
	}
	return false;
}

public void focusGained( FocusEvent evt){
	if ( CurrentHandler != null)
		CurrentHandler.focusLost( null);
	CurrentHandler = this;
	HasFocus = true;
	if ( Fi != null){
		Fi.gotFocus();
		Fi.getOsFocusGot().notifyObservers();
	}
}

public void focusLost( FocusEvent evt){
	HasFocus = false;
	if ( Fi != null){
		Fi.lostFocus();
		Fi.getOsFocusLost().notifyObservers();
	}
	CurrentHandler = null;
}

void getMouseState(MouseEvent e) {
	Mouse.LastEvent = e;
	Mouse.Pos.x = e.getX();
	Mouse.Pos.y = e.getY();
}

public static boolean hasSingleInstWnd () {
	return (SingleInstWnd != null);
}

public HotKey hotKeyAt( int key, int mod ){
	int ik;

	if ( (HotKeys == null) || (HotKeys.isEmpty()) )
		return null;

	ik = key + mod * 256;

	return ((HotKey)HotKeys.get( ik));
}

public static boolean isSingleInstWndOwner ( Window owner ) {
	if ( SingleInstWnd == null )
		return false;
	return ( ((Window)SingleInstWnd).getParent() == owner );
}

public void keyPressed(KeyEvent e) {
//System.out.println( e);

	int altGrMask = InputEvent.CTRL_MASK | InputEvent.ALT_MASK;
	int kco  = e.getKeyCode();
	int mods = e.getModifiers();
	int kch  = e.getKeyChar();
	int key  = kco;
		
	//Motif workaround ( alt or ctrl key combinations generate keyTyped() )
	SkipTyped = (mods > InputEvent.SHIFT_MASK) && (mods != altGrMask);
			
	if ( (kch != 0) && (! Key.isVKey( e))) {
		if ( (mods == 0) || (mods == altGrMask) ||(mods == InputEvent.SHIFT_MASK) ){
			e.consume();
			return;
		}
		else key = kch;
	}

	if ( (mods & InputEvent.CTRL_MASK) != 0) {
		if ( (key >= 1) && (key <= 26))
			key += 96;
	}
		
	//the mod vKeys does not update modifiers
	if ( kco == KeyEvent.VK_SHIFT )			mods |= InputEvent.SHIFT_MASK;
	else if ( kco == KeyEvent.VK_CONTROL)	mods |= InputEvent.CTRL_MASK;
	else if ( kco == KeyEvent.VK_ALT)		mods |= InputEvent.ALT_MASK;
		
	Key.ActiveKey = key;
	Key.ActiveKeyMods |= mods;

	if ( ! IsDisabled) {
		if ( ( Ki == null) || (! Ki.vKeyInput( key, mods) )) {
			if ( ! checkHotKey( key, mods) )
				checkDefaultKey( key, mods);
		}
	}

	e.consume();
}

public void keyReleased(KeyEvent e) {

	int mods = e.getModifiers();
	int kco  = e.getKeyCode();

	//the mod vKeys does not update modifiers
	if ( kco == KeyEvent.VK_SHIFT )			mods |= InputEvent.SHIFT_MASK;
	else if ( kco == KeyEvent.VK_CONTROL)	mods |= InputEvent.CTRL_MASK;
	else if ( kco == KeyEvent.VK_ALT)		mods |= InputEvent.ALT_MASK;

	Key.ActiveKey = 0;
	Key.ActiveKeyMods &= ~mods;

	e.consume();
}

public void keyTyped(KeyEvent e) {
	//Motif workaround
	if ( SkipTyped) {
		e.consume();
		return;
	}
	
	int mods = e.getModifiers();
	char kc  = e.getKeyChar();

	int altGrMask = InputEvent.CTRL_MASK | InputEvent.ALT_MASK;
	if ( (mods != 0) && (mods != altGrMask) && (mods != InputEvent.SHIFT_MASK) )
		return;

	Key.ActiveKey = kc;
	Key.ActiveKeyMods |= mods;

	if ( (! IsDisabled) && ( Ki != null) && (! Key.isVKey( e)))
		Ki.charInput( kc, mods);
	
	e.consume();
}

public void mouseClicked(MouseEvent e) {
	
	Mouse.LastEvent = e;
	Mouse.Pos.x = e.getX();
	Mouse.Pos.y = e.getY();

	if ( Mi != null) {
		int mod = e.getModifiers();
		int cc = e.getClickCount();
		
		Mouse.updateActiveButton( mod );
		Mi.getOsMouseBtn().notifyObservers();
		Mouse.reset();

		switch ( Mouse.getActiveButton( mod) ) {
			case 1:	if ( cc == 1) 			Mi.mouse1Click();
							else if ( cc == 2)	Mi.mouse1DblClick();
							break;
			default:if ( cc == 1)				Mi.mouse2Click();
							else if ( cc == 2)	Mi.mouse2DblClick();
							break;
		}
	}
	
	e.consume();
	Mouse.reset();
}

public void mouseDragged(MouseEvent e) {
	Mouse.LastEvent = e;
	Mouse.Pos.x = e.getX();
	Mouse.Pos.y = e.getY();

	if ( Mmi != null) {
		switch( Mouse.ActiveButton) {
			case 1: Mmi.mouse1Drag(); break;
			default: Mmi.mouse2Drag(); break;
		}
	}		
	e.consume();
}

public void mouseEntered(MouseEvent e) {
	Mouse.LastEvent = e;
	Mouse.Pos.x = e.getX();
	Mouse.Pos.y = e.getY();

	if ( Mi != null) {
		Mi.getOsMouseEnter().notifyObservers();
		Mi.mouseEnter();
	}
	e.consume();
}

public void mouseExited(MouseEvent e) {
	Mouse.LastEvent = e;
	Mouse.Pos.x = e.getX();
	Mouse.Pos.y = e.getY();
	if ( Mi != null) {
		Mi.getOsMouseExit().notifyObservers();
		Mi.mouseExit();
	}
	e.consume();
}

public void mouseMoved(MouseEvent e) {
	Mouse.LastEvent = e;
	Mouse.Pos.x = e.getX();
	Mouse.Pos.y = e.getY();
	if ( Mmi != null)
		Mmi.mouseMove();
	e.consume();
}

public void mousePressed(MouseEvent e) {
	e.consume();

	if ( IsDisabled)
		return;

	if ( SingleInstWnd != null) {
		if ( !(SingleInstWnd instanceof Window) || ( ! ((Window)SingleInstWnd).isAncestorOf( OC) )) {
			if ( !isSingleInstWndOwner( Wnd) ) {
				disposeSingleInstWnd();
				return;
			}
			disposeSingleInstWnd();
		}
	}

	//temporary workaround for focusGained() / focusLost()
	//sync problems
//	if ( !HasFocus)
	if ( ProtectFocus)
		focusGained( null);
	else if ( Fi != null)
		Fi.setFocus();

	Mouse.LastEvent = e;
	Mouse.Pos.x = e.getX();
	Mouse.Pos.y = e.getY();
	Mouse.down( e);

	if ( Mi != null) {
		Mi.getOsMouseBtn().notifyObservers();

		switch ( Mouse.ActiveButton) {
			case 1: Mi.mouse1Down(); break;
			default: Mi.mouse2Down(); break;
		}
	}

	e.consume();
}

public void mouseReleased(MouseEvent e) {
	Mouse.LastEvent = e;
	Mouse.Pos.x = e.getX();
	Mouse.Pos.y = e.getY();

	if ( Mi != null) {
		Mi.getOsMouseBtn().notifyObservers();
		Mouse.reset();
		
		switch ( Mouse.getActiveButton( e.getModifiers() )) {
			case 1:	Mi.mouse1Up(); break;
			default: Mi.mouse2Up(); break;
		}
	}		

	e.consume();
}

public void protectFocus( boolean b){
	ProtectFocus = b;
}

public boolean removeHotKey( int c, int mods) {
	int   key;

	if ( HotKeys == null)
		return false;

	key = c + mods * 256;

	if ( ! HotKeys.containsKey( key))
		return false;

	HotKeys.remove( key);
	return true;
}

public void setCursor( int idx){
	OC.setCursor( Cursor.getPredefinedCursor( idx));
}
}
