package biss.awt;

import biss.CommandQueue;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Scrollbar;
import java.awt.event.MouseEvent;

/**
 * horizontal scrollbar ( replaces java.awt.Scrollbar)
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class HScrollBar
  extends ScrollBar
{
public HScrollBar () {
	super( 0, 0, 0);
}

public HScrollBar ( int min, int max, int pos) {
	super( min, max, pos);
}

public Dimension getPreferredSize(){
	return new Dimension( 10*Awt.SysFontWidth, Awt.ScrollBarWidth);
}

public void mouse1Drag() {

	if ( DragOffs > 0){
		int h  = Height - FBorder.Ext;
		int tl = Width - 2*h;
		int x  = Mouse.Pos.x - DragOffs;
		
		setPosition( Min + ((x - h)*(Max - Min)) / (tl - SliderRect.width) );
	}
}

public void mouseDownScroll () {

	int x = Mouse.Pos.x;
	int y = Mouse.Pos.y;

	if ( SliderRect.contains( x, y)){
		DragOffs = x - SliderRect.x;
		ScrollOp = 5;
		LastOp = TRACK;
	}
	else if ( (x < Height) && (Pos > Min) ){
		ScrollOp = 1;
		LastOp = UNIT_DECREMENT;
		redrawSpins();
		Repeater.start();
	}
	else if ( (x > Width - Height) && (Pos < Max) ){
		ScrollOp = 2;
		LastOp = UNIT_INCREMENT;
		redrawSpins();
		Repeater.start();
	}
	else if ( x < SliderRect.x){
		ScrollOp = 3;
		LastOp = BLOCK_DECREMENT;
	}
	else if ( x > SliderRect.x + SliderRect.width){
		ScrollOp = 4;
		LastOp = BLOCK_INCREMENT;
	}

}

protected void redrawSlider( Graphics g) {
	if ( Width <= 0 )
		return;

	int b = FBorder.Ext;
	int h = Height - b;
	int w = Width - 2*h;

	if ( SliderRect.width > 0 ) {
		g.setColor( getBackground());
		int x, d = SliderRect.x - h;
		int hh = h-b;
		if ( d > 0 )
			g.fillRect( h, b, d, hh);
		x = SliderRect.x + SliderRect.width;
		d = Width - h - x;
		if ( d > 0 )
			g.fillRect( x, b, d, hh);

		GraphicsLib.paint3dBox( g, SliderRect.x, SliderRect.y,
		                        SliderRect.width, SliderRect.height,
		                        false, Awt.ButtonBackClr);
	}
	else {
		g.setColor( getBackground() );
		g.fillRect( h, b, w, h-b);
	}
}

protected void redrawSpins( Graphics g) {
	int b = FBorder.Ext;
	int h = Height - 2*b;

	if ( Mouse.isAnyActive() && (ScrollOp == UNIT_INCREMENT) )
		GraphicsLib.paintButton( g, b, b, h, h, false, Awt.ButtonBackClr);
	else
		GraphicsLib.paint3dBox( g, b, b, h, h, false, Awt.ButtonBackClr);

	if ( Mouse.isAnyActive() && (ScrollOp == UNIT_DECREMENT) )
		GraphicsLib.paintButton( g, Width-b-h, b, h, h, false, Awt.ButtonBackClr);
	else
		GraphicsLib.paint3dBox( g, Width-b-h, b, h, h, false, Awt.ButtonBackClr);
}

protected void updateSliderPos() {
	int h = Height - FBorder.Ext;
	int tl = Width - 2*h;
	SliderRect.x = h + (Pos-Min) * (tl-SliderRect.width)/(Max-Min);

}

protected void updateSliderRect() {
	if ( (Height == 0) || (Width == 0) )
		return;

	int b = FBorder.Ext;
	int h = Height - b;
	int tl = Width - 2*h;

	if ( Max - Min <= 0){
		SliderRect.setBounds( 0, 0, 0, 0);
	}
	else {
		int s = Math.max((int)((double)tl * PageInc / (PageInc+Max-Min)), Height);
		int xPos = h + (Pos-Min) * (tl-s)/(Max-Min);
		SliderRect.setBounds( xPos, b, s, h-b);
	}

}
}
