package biss.awt;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.Enumeration;

/**
 * horizontal splitbar (moveable boundary between widgets to change the
 * layout of a Container)
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author P.C.Mehlitz
 */
public class HSplitBar
  extends SplitBar
{
public HSplitBar () {
	setCursor( Cursor.getPredefinedCursor( Cursor.E_RESIZE_CURSOR));
}

public HSplitBar ( int id ) {
	this();
	Id = id;
}

void computeSiblings () {
	Container c = getParent();
	int cl = c.getComponentCount();
	Component childs[] = c.getComponents();
	Rectangle b = getBounds();
	Rectangle r;

	for ( int i=0; i<cl; i++ ) {
		if ( (childs[i] != this) && (childs[i].isVisible()) ) {
			r = childs[i].getBounds();
			if ( r.y < (b.y + b.height) && (r.y + r.height) > r.y ) {
				if ( (Math.abs( b.x - (r.x + r.width)) <= 1) ||
				     (Math.abs( r.x - (b.x + b.width)) <= 1) ) {
					addSibling( childs[i]);
				}
			}
		}
	}
}

public Dimension getPreferredSize () {
	return new Dimension( 5, Parent.getSize().height);
}

public void mouseDragged ( MouseEvent e) {
	int x = e.getX();
	int x1 = XPos;
	int y2 = YPos+Height - 1;

	if ( Y < 0 ) {
		Y = 0;
	}
	else {
		x1 += X;
		PGr.drawLine( x1, YPos, x1, y2 );
	}

	x1 = XPos + x;
	if ( x1 <= Max && x1 >= Min ) {
		X = x;
		PGr.drawLine( x1, YPos, x1, y2 );
	}

}

public void paint ( Graphics g ) {
	g.setColor( getBackground());
	g.fillRect( 0, 0, Width, Height);

	int b = 12;
	int w = Width -1;
	g.setColor( Color.white);
	g.drawLine( 0, 0, w, 0);
	g.drawLine( 0, 0, 0, b);
	g.setColor( Color.black);
	g.drawLine( 0, b, w, b);
	g.drawLine( w, 0, w, b);
}

void rearrange () {
	Component c;
	Rectangle r;
	int       xNew, wNew;
	int       x = XPos + X;
	int       x2 = XPos + Width;
	int       d;

	setVisible( false);

	for ( Enumeration e=Siblings.elements(); e.hasMoreElements(); ) {
		c = (Component) e.nextElement();
		r = c.getBounds();

		if ( r.x >= x2 ) {   // right
			d    = r.x - x2;
			xNew = x + Width + d;
			wNew = r.width + (r.x - xNew);
		}
		else {              // left
			d    = XPos - (r.x + r.width);
			xNew = r.x;
			wNew = x - r.x - d;
		}
		c.setBounds( xNew, r.y, wNew, r.height);

		/*** panel bug workaround ***/
		if ( c instanceof Container)
			c.doLayout();

		c.setVisible( true);
	}
	setLocation( x, YPos);
	setVisible( true);
}

void updateMoveBoundaries ( Rectangle r ) {
	if ( r.x >= XPos ) {
		int x = r.x+r.width - Width;
		if ( x > Max )
			Max = x;
	}
	if ( r.x < Min )
		Min = r.x;
}
}
