package biss.awt;

import biss.PrimHashtab;
import java.awt.event.InputEvent;

/**
 * class to implement function keys (both topwindow global and widget local)
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class HotKey
{
	final public static int Alt = InputEvent.META_MASK;
	final public static int Control = InputEvent.CTRL_MASK;
	final public static int Lone = 0;
	final public static int Shift = InputEvent.SHIFT_MASK;
	int Key;
	int HashCode;
	int Mods;
	Object Selector = null;
	HotKeyProcessor Proc = null;

HotKey () {
}

public HotKey ( int key, int mods, HotKeyProcessor p, Object sel) {  
	Key = key;
	Mods = mods;
	HashCode = key + mods * 256;
	Selector = sel;
	Proc = p;
}

public HotKeyProcessor getProcessor(){
	return Proc;
}

public Object getSelector() {
	return Selector;
}

public int key() {
	return Key;
}

public int mods() {
	return Mods;
}

public boolean process(){
	if ( Proc == null)
		return false;
	Proc.processKey( Selector);
	return true;
}

public void setProcessor( HotKeyProcessor p){
	Proc = p;
}
}

class HotKeyHashtab
  extends PrimHashtab
{
	HotKey[] HKeys;
	static HotKey RecycledKey = new HotKey();

HotKeyHashtab ( int entryGuess, int shiftLeft) {
	super( entryGuess, shiftLeft);
}

public boolean containsKey ( int key ) {
	return (indexOf( key, false) >= 0);
}

protected Object createKeyArray ( int n ) {
  return null;
}

protected Object createRecycledEntry () {
	return RecycledKey;
}

protected Object[] createValueArray ( int n ) {
  HKeys = new HotKey[n];
	return HKeys;
}

public HotKey get ( long key ) {
	int i = indexOf( key, false);
	return (i >= 0) ? HKeys[i] : null;
}

boolean grow () {
	int  m = HKeys.length;
	int  n = m * 3 / 2;
	int  i, k;
	HotKey v;

	if ( m > 14983 ) return false;

	HotKeyHashtab newHt = new HotKeyHashtab( n, LeftShift);
	
	for ( i=0, k=Count; i<m; i++ ) {
		if ( (v = HKeys[i]) != null && v != RecycledKey ) {
			newHt.put( v.HashCode, v);
			if ( --k == 0 ) break;
		}
	}

	Values = HKeys = newHt.HKeys;
	
	return true;
}

final private int indexOf ( long key, boolean newOne ) {
	long   k = key;
	int    m = HKeys.length, m2 = m >> 1;
	int    i, j = 0, d = 1;
	HotKey   v;

	if ( LeftShift > 0 ) k <<= LeftShift;
	i = (int) (k % m);

  do {
		if ( (v = HKeys[i]) == null ){
			return (newOne) ? i : -1;
		}
		else if ( v == RecycledKey ) {
			if ( newOne ) return i;
		}
		else if ( v.HashCode == key ) {
			return i;
		}
		
		i += d; d += 2;
		if ( i >= m )
			i %= m;
	}
	while ( ++j <= m2 );
	
	return -1;
}

public HotKey put ( long key, HotKey value ) {
	int     i;
	HotKey  v;

	do {
		if ( (i = indexOf( key, true)) >= 0 ) {
			v = HKeys[i];
			HKeys[i] = value;
			if ( v != null )
				return v;
			Count++;
			return null;
		}
	}
	while ( grow() ); // no space available any more
	
	throw new NullPointerException();
}

public HotKey remove ( long key ) {
	int i = indexOf( key, false);

	if ( i >= 0 ) {
		HotKey v = HKeys[i];
		HKeys[i] = RecycledKey;
		Count--;
		return v;
	}
	return null;
}
}
