package biss.awt;

import biss.ObserverSocket;
import biss.VectorLib;
import biss.VectorSorter;
import biss.awt.Awt;
import biss.awt.Button;
import biss.awt.Checkbox;
import biss.awt.DialogWindow;
import biss.awt.Layouter;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Rectangle;
import java.util.BitSet;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

class CBSorter
  implements VectorSorter
{
public int compare ( Object a, Object b ) {
	return ((Checkbox)a).getContents().compareTo( ((Checkbox)b).getContents());
}
}

/**
 * generic dialog to choose from a set of Checkboxs (used to
 * set type/field/method modifiers)
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author P.C.Mehlitz
 */
public class ItemCheckDlg
  extends DialogWindow
{
	final static int H = 16;
	Object[] Items;
	BitSet Displayed;
	BitSet Checked;
	Vector CB;
	Hashtable Id;
	Button AllBtn = new Button("~All");
	Button NoneBtn = new Button("~None");
	Layouter GM = new Layouter( this);
	public ObserverSocket OsSet = new ObserverSocket( this);
	Button SetBtn = new Button("~Set");
	Button ExitBtn = new Button("E~xit");

public ItemCheckDlg ( Frame owner, String title, boolean isModal,
	       String[] labels, BitSet display, BitSet check,
	       Observer obs ) {
	super( owner, title, isModal ? DialogWindow.MODAL : DialogWindow.OWNED);
	setButtons( labels, display);
	setChecked( check);

	setBounds( computeBounds());

	if ( obs != null ) {
		OsClose.addObserver( obs);
		OsSet.addObserver( obs);
	}
	
	open();
}

public ItemCheckDlg ( Frame owner, String title, boolean isModal,
	       String[] labels, BitSet check, Observer obs ) {
	this( owner, title, isModal, labels, null, check, obs);
}

public ItemCheckDlg ( Frame owner, boolean isModal) {
	super( owner, "ItemCheckDlg", isModal ? DialogWindow.MODAL : DialogWindow.OWNED);
	open();
}

Rectangle computeBounds () {
	int w = 22 * Awt.SysFontWidth;
	int h = (CB.size() + 5) * Awt.SysFontHeight * 16 / 10;

	return new Rectangle( 0, 0, w, h);
}

public BitSet getChecked () {
	return Checked;
}

public String[] getCheckedLabels () {
	int i, n;
	String s[];
	Checkbox btn;

	for ( i=0, n=0; i<CB.size(); i++ )
		if ( ((Checkbox)CB.elementAt( i)).isChecked() ) n++;

	s = new String[n];
	
	if ( n > 0 ) {
		for ( i=0, n=0; i<CB.size(); i++ ) {
			if ( (btn = (Checkbox)CB.elementAt( i)).isChecked() ) {
				s[i] = btn.getContents();
			}
		}
	}
	
	return s;
}

public void init() {
	SetBtn.OsAction.addObserver( this);
	ExitBtn.OsAction.addObserver( this);
	
	AllBtn.OsAction.addObserver( this);
	NoneBtn.OsAction.addObserver( this);
}

public static void main ( String[] args ) {
	String[] mod1 = { "private", "protected", "public", "static", "blabla" };
	BitSet disp = new BitSet( mod1.length);
	for ( int i=0; i<mod1.length; i++ ) disp.set(i);
	BitSet chk = new BitSet( mod1.length);
	chk.set(2);

	ItemCheckDlg dlg = new ItemCheckDlg( null, "CheckDlg", false, mod1, disp, chk, null);

	try { Thread.sleep( 2000); } catch ( Exception x ) {}
	chk.clear(2); chk.set(1);
	dlg.setChecked( chk);

	try { Thread.sleep( 5000); } catch ( Exception x ) {}

	dlg.setContents( mod1, disp);
	/*
	  String[] mod2 = { "synchronized", "transient", "volatile" };
	  disp = new BitSet( 2); disp.set(0); disp.set(1);
	  dlg.setContents( mod2, disp);
	 */
}

public void open() {
	GM.add( AllBtn, 0, 1000, 500, 1000, true, 20, -340, -13, -186, 0);
	GM.add( NoneBtn, 500, 1000, 1000, 1000, true, 20, -340, -13, -186, 0);
	GM.add( SetBtn, 0, 1000, 500, 1000, true, 20, -173, -13, -20, 0);
	GM.add( ExitBtn, 500, 1000, 1000, 1000, true, 20, -173, -13, -20, 0);

	init();

	Rectangle bounds = getBounds();

	if ( bounds.width > 0)
		setVisible( true);
	else {
		bounds.setBounds( 3, 2283, 1330, 1330);
		Awt.toPixelUnits( bounds);
		setBounds( bounds);
		setVisible( true);
	}
}

public void setAll ( boolean checked ) {
	for ( Enumeration e=CB.elements(); e.hasMoreElements(); ) {
		Checkbox cb = (Checkbox) e.nextElement();
		cb.setChecked( checked, true);
	}
}

void setButtons ( Object[] items, BitSet displayed ) {
	int i, m;
	Checkbox c;

	if ( CB != null ) {
		for ( i=0; i<CB.size(); i++ )
			remove( (Checkbox)CB.elementAt(i) );
	}

	Items = items;
	Displayed = displayed;

	if ( displayed != null ) 
		m = Math.min( displayed.size(), items.length);
	else
		m = items.length;
		
	CB = new Vector( m);
	Id = new Hashtable( m);
	for ( i=0; i<m; i++ ) {
		if ( (displayed == null) || displayed.get(i) ) {
			c = new Checkbox( Items[i].toString());
			c.OsAction.addObserver( this);
			CB.addElement( c);
			Id.put( c, new Integer( i));
		}
	}

	VectorLib.quickSort( CB, new CBSorter());

	for ( i=0; i<CB.size(); i++ ){
		c = (Checkbox)CB.elementAt(i);
		GM.add( c, 500, 0, 500, 0, true, -300, 160*(i+1), 600, 160*(i+2),  0);
	}
}

void setCheckState ( Checkbox cb ) {
	if ( Checked == null )
		Checked = new BitSet( CB.size());

	int i = ((Integer)Id.get( cb)).intValue();
	if ( cb.isChecked() )
		Checked.set(i);
	else
		Checked.clear(i);
}

public void setChecked ( BitSet checked ) {
	Checked = checked;

	for ( Enumeration e=CB.elements(); e.hasMoreElements(); ) {
		Checkbox cb = (Checkbox) e.nextElement();
		int i = ((Integer)Id.get( cb)).intValue();
		cb.setChecked( (checked != null && checked.get(i)), false);
	}
}

public void setContents ( Object[] items) {
	setContents( items, null);
}

public void setContents ( Object[] items, BitSet display ) {
	setButtons( items, display);
	setBounds( computeBounds());
	GM.forceLayout();
}

public void update ( Observable slot, Object arg ) {
	if ( slot == ExitBtn.OsAction ) 
		process();
	else if ( slot == SetBtn.OsAction )
		OsSet.notifyObservers( this);
	else if ( slot == AllBtn.OsAction ) {
		setAll( true);
		OsSet.notifyObservers( this);
	}
	else if ( slot == NoneBtn.OsAction ) {
		setAll( false);
		OsSet.notifyObservers( this);
	}
	else if ( arg instanceof Checkbox )
		setCheckState( (Checkbox) arg);
	else
		super.update( slot, arg);
}
}
