package biss.awt;

import biss.ObserverSocket;
import biss.Queue;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.util.Observable;
import java.util.Vector;

/**
 * List widget which is capable of displaying/processing hierarchical
 * data (directory trees etc.). Used in conjunction with LeveledObject
 * (wrapper for hierarchically organized object) and Leveler (generator
 * for LeveledObject collections).
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class LeveledList
  extends List
{
	Leveler Lvl = null;
	public LeveledObject DrawLvlObject;
	public ObserverSocket OsShrink = new ObserverSocket( this);
	public ObserverSocket OsExpand = new ObserverSocket( this);
	boolean PlainMode;

public LeveledList () {
	this( SB_BOTH);
}

public LeveledList (int type) {
	super( type, false);

	Cv = new LeveledListCanvas(Horz, Vert, this);  
	setCanvas( Cv);
}

public void addAllElements ( Vector list) {
}

public void addElement ( Object obj) {
}

public Vector allItems () {
	return ((LeveledListCanvas)Cv).AllItems;
}

LeveledListCanvas canvas () {
	return (LeveledListCanvas)Cv;
}

public void expandAll () {
	LeveledObject.expandAll( allItems());
	canvas().updateViewItems();
}

public Rectangle getDrawRectFor ( Object o) {
	LeveledObject lo = getLvlObj( o, false);
	if ( lo != null)
		return Cv.drawRectFor( lo);
	return null;
}

public LeveledObject getLvlObj ( Object o, boolean all) {
	Vector v = all ? allItems() : Cv.Lines;

	if ( v == null)
		return null;
	int s = v.size();

	for ( int i=0; i<s; i++){
		LeveledObject lo = (LeveledObject)v.elementAt( i);
		if ( lo.Obj == o)
			return lo;
	}
	return null;
}

public Object getSelection () {
	if ( Cv.Selection != null)
		return ((LeveledObject)Cv.Selection).Obj;
	return null;
}

public Vector getSelections () {
	Vector list = Cv.Selections;
	int s = list.size();
	Vector sList = new Vector( s);
	
	for ( int i=0; i<s; i++){
		LeveledObject lo = (LeveledObject)list.elementAt( i);
		sList.addElement( lo.Obj);
	}
	
	return sList;
}

public void redrawObject ( Object obj) {
	Cv.redrawObject( getLvlObj( obj, false));
}

public void removeAllElements ( Vector list) {
}

public void removeElement ( Object obj) {
}

public void replaceElementWith ( Object oldObj, Object newObj) {
}

public int setContents ( Object levelObj) {
	int n = 0;

	if ( Lvl != null) {
		Vector v = Lvl.getLeveledList( levelObj);
		if ( v != null ) {
			Cv.setContents( v);
			n = v.size();
		}
	}
	else {
		Cv.setContents( null);
	}
	
	return n;
}

public int setContents ( Object levelObj, Leveler lvl) {
	Lvl = lvl;
	return setContents( levelObj);
}

public boolean setDrawEnvironment ( Object obj) {
	return Cv.setDrawEnvironment( getLvlObj( obj, false));
}

public void setExpandImage ( String pathName) {
	canvas().setExpandImage( Awt.getImage( pathName) );
}

public void setExpandImageClass ( String imgCls) {
	try {
		Class c = Class.forName( imgCls);
		ClassImageSource cis = (ClassImageSource) c.newInstance();
		canvas().setExpandImage( createImage( cis) );
	}
	catch( Throwable t) { System.out.println( t);}
}

public void setPlainMode ( boolean p, boolean update) {
	PlainMode = p;
	if ( update) {
		reset();
		if ( p) expandAll();
		else		shrinkAll();
	}
}

public boolean setSelection ( Object obj, boolean fireSlot) {
	LeveledObject lo = getLvlObj( obj, false);
	if ( lo == null){
		lo = getLvlObj( obj, true);
		((LeveledListCanvas)Cv).expandPath( lo);
	}
	return Cv.setSelection( lo, fireSlot);
}

public boolean setSelection ( int idx, boolean fireSlot) {
	Vector v = allItems();
	if ( v == null)
		return false;
	return setSelection( ((LeveledObject)v.elementAt( idx)).Obj, fireSlot);
}

public void setShrinkImage ( String pathName) {
	canvas().setShrinkImage( Awt.getImage( pathName) );
}

public void setShrinkImageClass ( String imgCls) {
	try {
		Class c = Class.forName( imgCls);
		ClassImageSource cis = (ClassImageSource) c.newInstance();
		canvas().setShrinkImage( createImage( cis) );
	}
	catch( Throwable t) { System.out.println( t);}
}

public boolean showObject ( Object obj) {
	return Cv.showObject( getLvlObj( obj, false));
}

public void shrinkAll () {
	if ( ! PlainMode) {
		LeveledObject.shrinkAll( allItems());
		canvas().updateViewItems();
	}
}

public void updateContents ( Vector newList) {
}
}

class LeveledListCanvas
  extends ListCanvas
{
	Vector AllItems;
	Dimension LevelDim = new Dimension( Awt.SysFontHeight/3+1, Awt.SysFontHeight/3+1);
	Image ExpandImg = null;
	Image ShrinkImg = null;
	int LevelDist;
	LeveledList LvlMaster;

LeveledListCanvas( HScrollBar h, VScrollBar v, List cmpd) {
	super(h, v, cmpd);
	
	LvlMaster = (LeveledList) Master;
}

void drawLevelSign () {
	LeveledObject lo = LvlMaster.DrawLvlObject;
	Rectangle r = LvlMaster.DrawRect;
	int lvl     = lo.level();
	int dy      = (r.height - LevelDim.height) / 2;

	r.x += (lvl+1)*LevelDim.width;

	if ( lo.isExpandable() ){
		if ( ExpandImg != null)
			GraphicsLib.drawImageVCentered( ResGraph, ExpandImg,
		                                r.x, r.y, r.height, this );
		else {
			ResGraph.setColor( Awt.ListExpandClr);
			ResGraph.fillArc( r.x, r.y+dy, LevelDim.width, LevelDim.height, 0, 360);
			ResGraph.setColor( Color.blue);
			ResGraph.drawOval( r.x, r.y+dy, LevelDim.width, LevelDim.height);
		}
	}
	else if ( lo.isShrinkable() ){
		if ( ShrinkImg != null)
			GraphicsLib.drawImageVCentered( ResGraph, ShrinkImg,
		                                r.x, r.y, r.height, this );
		else {
			ResGraph.setColor( Color.blue);
			ResGraph.drawOval( r.x, r.y+dy, LevelDim.width, LevelDim.height );
		}
	}

	r.x += 3*LevelDim.width/2 + 2;
}

void drawRenderedObject () {
	Object        obj   = Master.DrawObject;
	Rectangle     dRect = Master.DrawRect;

	LeveledList   lop   = LvlMaster;
	LeveledObject lo    = (LeveledObject) obj;
	Object        dro   = lo.Obj;

	ResGraph.setColor( lop.DrawClrBack);
	ResGraph.fillRect( dRect.x, dRect.y, dRect.width, dRect.height);
	ResGraph.setColor( lop.DrawClrFore);
	
	lop.DrawLvlObject = lo;
	lop.DrawObject = dro;

	if ( ! LvlMaster.PlainMode) {
		drawLevelSign();
		drawTreeInfo();
	}
	
	if ( ! lop.OsDrawObject.isEmpty() )
		lop.OsDrawObject.notifyObservers();
	else if ( dro instanceof SelfDrawingObject)
		((SelfDrawingObject) dro).drawSelfIn( lop);
	else {
		ResGraph.setFont( Awt.SysFont);
		ResGraph.drawString( dro.toString(), dRect.x + 5,
		                     dRect.y + dRect.height -
		                     (dRect.height - Awt.SysFontHeight) / 2 -
		                     Awt.SysFontDesc );
	}  
}

void drawTreeInfo () {
	int ld2 = LevelDim.width/2;
	int x = XOffset + ld2;
	int xd;
	int y = Master.DrawRect.y;
	int cy = y + (LineHeight >> 1);
	int lvl;
	long k;
	LeveledObject lo = LvlMaster.DrawLvlObject;

	ResGraph.setColor( Color.blue);

	for ( lvl = 0, k=1; lvl < lo.Level; lvl++, k <<= 1 ) {
		if ( (lo.FollowLevels & k) > 0 )
			ResGraph.drawLine( x, y, x, y+LineHeight);
		if ( lvl<lo.Level)
			x += LevelDim.width;
	}

	ResGraph.drawLine( x, y, x, cy);
	ResGraph.drawLine( x, cy, x + ld2, cy);
	
	if ( (lo.FollowLevels & k) > 0 )
		ResGraph.drawLine( x, cy, x, y + LineHeight);

	x += LevelDim.width;
	xd = x + LevelDim.width;

	if ( lo.isShrinkable() ){
		ResGraph.drawLine( x, y + LineHeight - (LineHeight - LevelDim.height) / 2, x, y + LineHeight);
		ResGraph.drawLine( x+ld2 , cy, xd, cy);
	}
	else if ( lo.isExpandable() )
		ResGraph.drawLine( x+ld2, cy, xd, cy);
	else
		ResGraph.drawLine( x-ld2, cy, xd, cy);
}

boolean expand (LeveledObject lo) {
	if ( LvlMaster.PlainMode || ! lo.isExpandable() )
		return false;

	int s = AllItems.size();
	lo.setExpanded( true);

	for ( int i=AllItems.indexOf(lo)+1; i<s; i++) {
		LeveledObject lc = (LeveledObject)AllItems.elementAt( i);
		if ( lc.Level <= lo.Level)
			break;
		if ( lc.Level == lo.Level+1)
			lc.setVisible( true);
	}

	LvlMaster.OsExpand.notifyObservers( lo);
	return true;
}

boolean expandPath (LeveledObject lo) {
	if ( AllItems == null)
		return false;
	LeveledObject.expandPath( AllItems, lo);
	updateViewItems();
	return true;
}

Vector getViewItems () {
	Vector v = new Vector();
	
	if ( AllItems != null ) {
		int s = AllItems.size();
		for( int i=0; i<s; i++){
			LeveledObject lo = (LeveledObject)AllItems.elementAt( i);
			if ( lo.isVisible() )
				v.addElement( lo);
		}
	}

	return v;
}

public boolean imageUpdate(Image img, int flags, int x, int y, int w, int h) {
	updateDims( img);
	return super.imageUpdate( img, flags, x, y, w, h);
}

boolean isLevelPos ( int x, LeveledObject lo) {
	if ( LvlMaster.PlainMode)
		return false;
	return ( (x - XOffset) < (lo.Level+2)*LevelDim.width );
}

public void mouse1Down() {
	int x = Mouse.Pos.x;
	int y = Mouse.Pos.y;
	Object ns = objectAt( x, y);
	LeveledObject lo = (LeveledObject) ns;
	SelSource = OP_SEL_MOUSE;

	if ( (lo != null) && (lo.isExpandable() || lo.isShrinkable()) ){
		if ( isLevelPos( x, lo)){
			toggleExpand( lo);
			return;
		}
	}

	if ( Key.isCtrlDown() ) {
		toggleMarker( ns);
		Master.notifySelect();
	}
	else {
		selectObject( ns, false);
	}
}

public void processKey( Object sel){
	if ( sel.equals( "toggleExpand")){
		LeveledObject lo = (LeveledObject) Selection;
		if ( (lo != null) && (lo.isExpandable() || lo.isShrinkable()) )
			toggleExpand( lo);
	}
	else
		super.processKey( sel);
}

void registerHotKeys() {
	EventHandler.addHotKey( Awt.LeveledListExpToggleKeys[0], Awt.LeveledListExpToggleKeys[1], this, "toggleExpand");
	super.registerHotKeys();
}

synchronized int setContents( Vector data) {
	AllItems = data;
	if ( LvlMaster.PlainMode)
		LeveledObject.expandAll( AllItems);
	Lines = getViewItems();
	LeveledObject.computeFollowLevels( Lines);

	super.setContents( Lines);
	return Lines.size();
}

void setExpandImage (Image img) {
	ExpandImg = img;
	updateDims( img);
	if ( isShowing() )
		redraw();
}

void setLineDimensions ( Object refObj) {
	if ( refObj != null) {
		LeveledObject lo = (LeveledObject) refObj;
		super.setLineDimensions( lo.Obj);
	}
	else
		super.setLineDimensions( null);

	if ( LevelDim.height > LineHeight)
		LineHeight = LevelDim.height;
}

void setShrinkImage (Image img) {
	ShrinkImg = img;
	updateDims( img);
	if ( isShowing() )
		redraw();
}

boolean shrink (LeveledObject lo) {
	if ( LvlMaster.PlainMode || ! lo.isShrinkable() )
		return false;

	int s = AllItems.size();
	lo.setExpanded( false);

	for ( int i=AllItems.indexOf(lo)+1; i<s; i++) {
		LeveledObject lc = (LeveledObject)AllItems.elementAt( i);
		if ( lc.Level <= lo.Level)
			break;
		lc.setVisible( false);
		lc.setExpanded( false);
	}

	LvlMaster.OsShrink.notifyObservers( lo);
	return true;
}

boolean toggleExpand (LeveledObject lo) {
	if ( LvlMaster.PlainMode)
		return false;
	if ( lo.isExpandable() ){
		expand( lo);
		return updateViewItems();
	}

	if ( lo.isShrinkable() ) {
		shrink( lo);
		return updateViewItems();
	}

	return false;
}

void updateDims (Image img) {
	int h = img.getHeight( this);
	int w = img.getWidth( this);
	
	if ( ( w <= 0) || ( h <= 0) )
		return;
		
	if ( LevelDim.width < w)
		LevelDim.width = w;
	if ( LevelDim.height < h)
		LevelDim.height = h;

	if ( LineHeight < LevelDim.height)
		LineHeight = 5*LevelDim.height/4;
}

synchronized boolean updateViewItems () {
	if ( Lines == null || AllItems == null )
		return false;
		
	int s = AllItems.size();
	Lines.removeAllElements();
	for( int i=0; i<s; i++){
		LeveledObject lo = (LeveledObject)AllItems.elementAt( i);
		if ( lo.isVisible() )
			Lines.addElement( lo);
	}

	LeveledObject.computeFollowLevels( Lines);
	
	updateContents();
	return true;
}
}
