package biss.awt;

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.MenuComponent;
import java.awt.peer.ActiveEvent;

class LocalDispatchEvent
  extends AWTEvent
  implements ActiveEvent
{
	LocalDispatcher Dsp;

public LocalDispatchEvent (LocalDispatcher ld) {
	super( ld, 0);
	Dsp = ld;
}

public void dispatch () {
	consume();
	Dsp.dispatchLocal( this, true);
}
}

public class LocalDispatcher
{
	boolean Terminate;
	int[] BreakIds;
	ActiveEvent BreakEvent;

public LocalDispatcher () {
}

void dispatchLocal ( LocalDispatchEvent lde, boolean sync) {

	int bil = (BreakIds != null) ? BreakIds.length : 0;
	int id = 0;
	
	try {
		AWTEvent evt;
		EventQueue eq = Awt.DefToolkit.getSystemEventQueue();
			
		while ( !Terminate) {
			evt = eq.getNextEvent();
			id = evt.getID();
			Object src = evt.getSource();

			if ( src instanceof Component)
				((Component)src).dispatchEvent( evt);
			else if ( src instanceof MenuComponent)
				((MenuComponent)src).dispatchEvent( evt);
			else if ( evt instanceof ActiveEvent)
				((ActiveEvent)evt).dispatch();

			if ( evt == BreakEvent){
				if ( sync) {
					synchronized( lde){
						lde.notify();
					}
				}
				return;
			}
			for ( int i=0; i<bil; i++){
				if ( BreakIds[i] == id){
					if ( sync) {
						synchronized( lde){
							lde.notify();
						}
					}
					return;
				}
			}
		}	
		if ( sync) {
			synchronized( lde){
				lde.notify();
			}
		}
		return;
	}	
	catch( Exception e) {
		System.err.println( "error dispatching event queue:");
		e.printStackTrace();
		if ( sync) {
			synchronized( lde){
				lde.notify();
			}
		}
		return;
	}

}

public void dispatchTilTerminate () {
	dispatchUntil( (int[])null, null);
}

public void dispatchUntil ( ActiveEvent aEvt) {
	dispatchUntil( (int[])null, aEvt);
}

public void dispatchUntil ( LocalDispatchEvent lde) {
	EventQueue q = Awt.DefToolkit.getSystemEventQueue();
	q.postEvent( lde);
	synchronized( lde) {
		try { lde.wait(); }
		catch( Exception e) {}
	}
}

public void dispatchUntil ( int breakId) {
	int[] ia = { breakId };
	dispatchUntil( ia, null);
}

public void dispatchUntil ( int breakId, ActiveEvent aEvt) {
	int[] ia = { breakId };
	dispatchUntil( ia, aEvt);
}

public void dispatchUntil ( int[] breakIds) {
	dispatchUntil( breakIds, null);
}

public void dispatchUntil ( int[] breakIds, ActiveEvent aEvt) {
	BreakIds = breakIds;
	BreakEvent = aEvt;

	LocalDispatchEvent lde = new LocalDispatchEvent( this);	
	
	if ( Thread.currentThread().getName().startsWith( "AWT-EventQueue-" ))
		dispatchLocal( lde, false);
	else
		dispatchUntil( lde);
}

public void terminate () {
	Terminate = true;
}
}
