package biss.awt;

import java.util.Observable;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * replaces java.awt.Menu
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class Menu
  extends MenuItem
  implements MenuItemContainer
{
	Vector Items = new Vector();

public Menu(){
	this( "");
}

public Menu( String label){
	super( label);
}

public Menu( String label, String items){
	super( label);
	StringTokenizer tok = new StringTokenizer( items, ",:;");
	while( tok.hasMoreTokens() )
		add( tok.nextToken() );
}

public synchronized MenuItem add( MenuItem mi){
	Items.addElement( mi);
	return mi;
}

public void add( String label){
	add( new MenuItem( label));
}

public void add( String label, String sel){
	add( new MenuItem( label, sel));
}

public void addItem( String label){
	//for backward compatibility
	add( label);
}

public void addItem( String label, String sel){
	//for backward compatibility
	add( label, sel);
}

public void addSeparator(){
	add( MenuItem.Separator);
}

public Vector getHotKeys( HotKeyProcessor p, int mods){
	Vector v = new Vector();
	getHotKeys( v, p, mods);
	return v;
}

void getHotKeys( Vector v, HotKeyProcessor p, int mods){
		int s = Items.size();
		for ( int i=0; i<s; i++) {
			MenuItem mi = (MenuItem)Items.elementAt( i);
			mi.getHotKeys( v, p, mods);
		}
}

public MenuItem getItem( String sel){
	int s = Items.size();
	for ( int i=0; i<s; i++) {
		MenuItem mi = (MenuItem)Items.elementAt( i);
		if ( mi.Selector.equals( sel))
			return mi;
	}
	return null;
}

public MenuItem getItem( int index){
	try { return (MenuItem)Items.elementAt( index); }
	catch ( Throwable t) { return null; }
}

public int getItemCount(){
	return Items.size();
}

public Vector getItems(){
	return Items;
}

public boolean includesItem( MenuItem mi){
	return Items.contains( mi);
}

public void insert( MenuItem mi, int index){
	try {
		Items.insertElementAt( mi, index);
		notifyChange();
	}
	catch ( Throwable t){}
}

public void insert( String label, int index){
	try {
		Items.insertElementAt( new MenuItem( label), index);
		notifyChange();
	}
	catch ( Throwable t){}
}

public void insertSeparator( int index){
	try {
		Items.insertElementAt( MenuItem.Separator, index);
		notifyChange();
	}
	catch ( Throwable t){}
}

void notifyChange(){
//	setChanged();
//	notifyObservers( this);
}

public void remove( MenuItem mi){
	try {
		Items.removeElement( mi);
		notifyChange();
	}
	catch ( Throwable t){}
}

public void remove( int index){
	try {
		Items.removeElementAt( index);
		notifyChange();
	}
	catch ( Throwable t){}
}

public void removeAll(){
	Items.removeAllElements();
	notifyChange();
}

public void setChecked( MenuItem mi, boolean state){
	mi.setChecked( state);
}

public boolean setChecked( String sel, boolean state){
	try {
		setChecked( getItem( sel), state);
		return true;
	}
	catch( Throwable t) {
		return false;
	}
}

public void setEnabled( MenuItem mi, boolean state){
	mi.setEnabled( state);
}

public boolean setEnabled( String sel, boolean state){
	try {
		setEnabled( getItem( sel), state);
		return true;
	}
	catch( Throwable t) {
		return false;
	}
}

public void setEnabled( boolean state){
	int s = Items.size();
	for ( int i=0; i<s; i++)
		((MenuItem)Items.elementAt( i)).setEnabled( state);

	super.setEnabled( state);
}
}
