package biss.awt;

import biss.ObserverSocket;
import java.awt.Color;
import java.awt.Component;
import java.awt.Event;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MenuComponent;
import java.awt.MenuShortcut;
import java.awt.Point;
import java.util.Vector;

/**
 * replaces java.awt.MenuItem
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class MenuItem
  extends java.awt.MenuComponent
{
	String Selector;
	public static MenuItem Separator = new MenuItem( "---");
	int State;
	Point PMn;
	char MnKey;
	static FontMetrics Fm = Awt.getFontMetrics( Awt.MenuFont);
	final public static int CHECKED = 2;
	final public static int DISABLED = 1;
	String Text;
	final public static int EXPANDED = 4;

public MenuItem( String txt ){
	this( txt, null);
}

public MenuItem( String label, String sel ){
	Text = label;
	splitLabel();
	Selector = (sel != null) ? sel : Text;
}

public MenuShortcut createShortcut() {
	if ( MnKey != 0)
		return new MenuShortcut( MnKey, false);
	return null;
}

void drawAt( Graphics g, int x, int y, boolean highLight ) {

	if ( highLight ) {
		GraphicsLib.flashText( g, Text, x, y);
	}
	else {
		if ( isChecked() ){
			int w = Fm.getAscent()-Fm.getDescent();
			GraphicsLib.paintInstepBox( g, x, y-w, w, w, Awt.CheckboxSetClr);
			x += 3*w/2;
		}
		g.setColor( isEnabled() ? Awt.MenuForeClr : Awt.InactiveClr);
		g.drawString( Text, x, y);
	}

	if ( PMn != null)
		g.drawLine( x+PMn.x, y+2, x+PMn.y, y+2);
}

void getHotKeys( Vector v, HotKeyProcessor p, int mods) {
	if ( MnKey != 0) {
		HotKey hk = new HotKey( MnKey | 32, mods, p, this);
		v.addElement( hk);
	}
}

public String getLabel(){
	return Text;
}

public String getSelector(){
	return Selector;
}

public boolean isChecked(){
	return ( (State & CHECKED) != 0);
}

public boolean isEnabled(){
	return ( (State & DISABLED) == 0);
}

public boolean isExpanded(){
	return ( (State & EXPANDED) != 0);
}

public boolean isSeparator(){
//	return Separator.equals( this);
	// java.awt MenuItem compatibility ( separator is '-' )
	return ( (Text != null) ? (Text.charAt( 0) == '-') : false );
}

public void process (){
}

public void setChecked( boolean state){
	if ( state) State |= CHECKED;
	else        State &= ~CHECKED;
}

public void setEnabled( boolean state){
	if ( state) State &= ~DISABLED; 
	else        State |= DISABLED;
}

public void setExpanded( boolean state){
	if ( state) State |= EXPANDED;
	else        State &= ~EXPANDED;
}

public void setLabel ( String label ){
	boolean updSel = ( Selector == Text);
	
	Text = label;
	splitLabel();

	if ( updSel)
		Selector = Text;
}

public void setSelector( String sel){
	Selector = sel;
}

void splitLabel() {
	char mn = '~';
	String rStr;
	int xb, xe, idx = Text.indexOf( mn);

	if ( idx > -1){
		rStr = Text.substring( 0, idx);
		xb = Fm.stringWidth( rStr);
		Text = rStr + Text.substring( idx+1);
		xe = xb + Fm.charWidth( Text.charAt( idx));
		MnKey = Text.charAt( idx);
		PMn = new Point( xb, xe);
	}
}
}
