package biss.awt;

import biss.awt.Awt;
import biss.awt.DialogWindow;
import biss.awt.Label;
import biss.awt.Layouter;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.Rectangle;
import java.util.Observable;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * generic dialog to display messages
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class MessageBox
  extends DialogWindow
{
	Vector Btns = new Vector(5);
	Button CancelBtn = null;
	public static String NO = "~No";
	Button Selection = null;
	Label TP = new Label();
	public static String YES = "~Yes";
	int BtnPCW;

public MessageBox( String title, String text, String buttons, Frame owner){
	super( owner, title, true);
	StringTokenizer tk = new StringTokenizer( buttons, ",;:");
	while( tk.hasMoreTokens() ){
		String tok = tk.nextToken();
		if ( tok.startsWith( "@"))
			addCancelButton( tok.substring( 1, tok.length()));
		else
			addButton( tok);
	}
	TP.setContents( text);
	open();
}

public Button addButton( String label ) {

	Button btn = new Button( label );
	Btns.addElement( btn);
	btn.OsAction.addObserver( this);

	return btn;
}

public Button addCancelButton( ) {
	return addCancelButton( "~Cancel");
}

public Button addCancelButton( String label ) {
	return (CancelBtn = addButton( label ));
}

int computeWidth(){

	int  mw = 15*Awt.DlgXUnit;
	int  bw = maxBtnWidth();
	int  s  = Btns.size();
	int gtw = s*bw + (s+1)*Awt.DlgXUnit;

	BtnPCW = (1000 - (s+1)*100) / s;

	return (int)Math.max( gtw, mw);
}

public static boolean confirm( String text, Frame owner){
	MessageBox mb = new MessageBox( "Please confirm:", text, "~Yes,~No", owner);
	return (mb.getSelectionIdx() == 0);
}

public Dimension getPreferredSize() {
	Insets in = getInsets();
	int bh = Awt.SysFontHeight;
	int tw = computeWidth();
	int mh = TP.minHeight( tw) + 2*Awt.SysFontHeight;

	try { bh += ((Button)Btns.firstElement()).getPreferredSize().height; }
	catch ( Exception e) { bh += 2*Awt.SysFontHeight; }

	return new Dimension( tw + in.left + in.right, mh + bh + in.top + in.bottom);
}

public Button getSelection(){
	return Selection;
}

public int getSelectionIdx(){
	return Btns.indexOf( Selection);
}

public String getSelectionLabel(){
	return Selection.getContents();
}

void init() {
	TP.wrapContents();
	TP.setCarved( false);
	TP.setBorder( Border.BS_NONE);
	TP.setAlignment( Label.HA_CENTER, Label.VA_CENTER);

	try { DefCom = (Button)Btns.firstElement(); }
	catch ( Exception e) {}

	computeWidth();
	packButtons();
	setCenterBounds();
}

public static void main( String args[]){

	String s = "This is a quite\nlong text\n" +
	"this line might even be too long to fit in one\n" +
	"another break. Ok?";

	Frame fr = new Frame( "DlgOwner");
	fr.setBounds( 100, 100, 100, 100);
	fr.show();

//	System.out.println( notify( "This is a big one", fr));
	System.out.println( confirm( "Are you sure ?", fr));
//	MessageBox mb = new MessageBox( "MessageBox", s, "~Yes,~Maybe,@~Never", fr);
//	System.out.println( mb.getSelectionIdx() );

	System.exit( 0);
}

int maxBtnWidth(){
	int  w = 0;
	int  s = Btns.size();

	for ( int i=0; i<s; i++)
		w = Math.max( w, ((Button)Btns.elementAt(i)).minWidth());

	return w + 2*Awt.SysFontWidth;
}

public static void notify( String text, Frame owner){
	MessageBox mb = new MessageBox( "Please notify:", text, "~Ok", owner);
}

public void open() {
	Lo.add( TP, 0, 0, 1000, 1000, true, 0, 0, 0, -250, 0);

	init();

	Rectangle bounds = getBounds();

	if ( bounds.width > 0)
		setVisible( true);
	else {
		bounds.setBounds( 3, 2283, 1303, 830);
		Awt.toPixelUnits( bounds);
		setBounds( bounds);
		setVisible( true);
	}
}

void packButtons(){
	int bs = Btns.size();

	for ( int i=0; i<bs; i++) {
		Button btn = (Button)Btns.elementAt( i);
		int ip = (i+1)*100 + i*BtnPCW;
		Lo.add( btn, ip, 1000, ip+BtnPCW, 1000, true, 0, -230, 0, -30, 0);
	}

}

public void update ( Observable o, Object arg ) {
	if ( arg instanceof Button){
		Selection = (Button)arg;
		if ( Selection == CancelBtn)
			WasCanceled = true;
		process();
	} 
}

public boolean wasConfirmed () {
	return !WasCanceled && Selection.getContents().equals( YES);
}

public static int yesNoCancel( String title, String text, Frame owner){
	MessageBox mb = new MessageBox( title, text, "~Yes,~No,@~Cancel", owner);
	return mb.getSelectionIdx();
}
}
