package biss.awt;

import biss.ObserverSocket;
import java.awt.Color;
import java.awt.Event;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.LayoutManager;
import java.awt.Rectangle;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * auxiliary class for objects representing single notebook tabs
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
class BookTab
{
	String TabLabel;
	int TabWidth;
	static FontMetrics Fm;
	static int TabHeight;
	PageSpec Master;
	static FontMetrics FmSmall;

static {
	Fm = Awt.getFontMetrics( Awt.BookTabFont);
	FmSmall = Awt.getFontMetrics( Awt.BookSmallFont);
	TabHeight = 3*Fm.getHeight()/2;
}

public BookTab ( String tabText, PageSpec bp) {
	Master = bp;
	TabLabel = tabText;
	TabWidth = Fm.stringWidth( tabText) + 10;
}

void drawAt (Graphics g, Color c, int x, int ori) {
	switch ( ori) {
		case Notebook.TABS_TOP:
			drawAtTop( g, c, x);
			break;
		case Notebook.TABS_BOTTOM:
			drawAtBottom( g, c, x);
			break;
	}
}

void drawAtBottom (Graphics g, Color c, int x) {
	int dx=0, dy=0, i;
	int bw = 1;
	int rad = TabHeight/5;
	boolean isTop = Master.IsOnTop;
	int height = Master.Master.Height;

	g.setColor( c );
	g.fillRect( x, height-TabHeight, TabWidth, TabHeight);

	if ( isTop ){
		g.setColor( Color.white);
		bw = Awt.BorderWidth;
	}
	else {
		g.setColor( Color.black);
		dx = 1; dy = 1;
	}

	// bottomleft
	for ( i=0; i<bw; i++){
		g.drawLine( x+rad, height-i-1, x+TabWidth-rad-1, height-i-1);  //bottom
		g.drawLine( x, height-i-rad-1, x+rad, height-i-1);          //bottomleft
		g.drawLine( x+TabWidth-rad-1, height-i-1, x+TabWidth-1, height-i-rad-1);  //bottomright
		g.drawLine( x+i, height-TabHeight-2, x+i, height-rad-1);   //left
	}

	// bottomshadow
	if ( !isTop ) {
		g.setColor( Color.white);
		g.drawLine( x+rad, height-bw-1, x+TabWidth-rad-1, height-bw-1);
	}

	// right
	g.setColor( Color.black);
	for ( i=0; i<bw; i++)
		g.drawLine( x+TabWidth-1-i, height-TabHeight-2, x+TabWidth-1-i, height-rad-1);

	if ( ! isTop )
		g.setFont( Awt.BookSmallFont);
	// page-connector
	else {
		g.setFont( Awt.BookTabFont);
		g.setColor( c);
		for ( i=1; i<=bw; i++)
			g.drawLine( x+bw, height-TabHeight-i, x+TabWidth-bw-1, height-TabHeight-i);
		g.setColor( Color.black);
	}

	// label
	g.setColor( (Master.isEnabled() ? Awt.MenuForeClr : Awt.InactiveClr));
	g.drawString( TabLabel, x + 5 + dx, height - TabHeight/4 - dy );

}

void drawAtTop (Graphics g, Color c, int x) {
	int dx=0, dy=0, i;
	int bw = 1;
	int rad = TabHeight/5;
	boolean isTop = Master.IsOnTop;

	g.setColor( c );
	g.fillRect( x, 0, TabWidth, TabHeight);

	if ( isTop ){
		g.setColor( Color.white);
		bw = Awt.BorderWidth;
	}
	else {
		g.setColor( Color.black);
		dx = 1; dy = 1;
	}

	// topleft
	for ( i=0; i<bw; i++){
		g.drawLine( x+rad, i, x+TabWidth-rad-1, i);             //top
		g.drawLine( x, rad+i, x+rad, i);                        //topleft
		g.drawLine( x+TabWidth-rad-1, i, x+TabWidth-1, rad+i);  //topright
		g.drawLine( x+i, rad, x+i, TabHeight-1);                //left
	}

	// topshadow
	if ( !isTop ) {
		g.setColor( Color.white);
		g.drawLine( x+rad, bw, x+TabWidth-rad-1, bw);
	}

	// right
	g.setColor( Color.black);
	for ( i=0; i<bw; i++)
		g.drawLine( x+TabWidth-1-i, rad, x+TabWidth-1-i, TabHeight-1);

	if ( ! isTop )
		g.setFont( Awt.BookSmallFont);
	// page-connector
	else {
		g.setFont( Awt.BookTabFont);
		g.setColor( c);
		for ( i=0; i<bw; i++)
			g.drawLine( x+bw, TabHeight+i, x+TabWidth-bw-1, TabHeight+i);
		g.setColor( Color.black);
	}

	// label
	g.setColor( (Master.isEnabled() ? Awt.MenuForeClr : Awt.InactiveClr));
	g.drawString( TabLabel, x + 5 + dx, 3*TabHeight/4 + dy );

}

void drawBreakAtBottom (Graphics g, Color c, int tw, int x) {
	int cw = 6;
	int pikes = 6;
	int height = Master.Master.Height;

	g.setColor( c);
	g.fillRect( tw-cw, height-TabHeight, cw, TabHeight);

	g.setColor( Color.black);
	int xn, xo=tw-cw;
	for ( int i=0; i<pikes; i++) {
		xn = tw - (int)( Math.random() * cw) - 1;
		g.drawLine( xo, height-i*TabHeight/pikes, xn, height-(i+1)*TabHeight/pikes);
		xo = xn;
	}  
}

void drawBreakAtTop (Graphics g, Color c, int tw, int x) {
	int cw = 6;
	int pikes = 6;

	g.setColor( c);
	g.fillRect( tw-cw, 0, cw, TabHeight);

	g.setColor( Color.black);
	int xn, xo=tw-cw;
	for ( int i=0; i<pikes; i++) {
		xn = tw - (int)( Math.random() * cw) - 1;
		g.drawLine( xo, i*TabHeight/pikes, xn, (i+1)*TabHeight/pikes);
		xo = xn;
	}  
}

void drawBreakTab (Graphics g, Color c, int tw, int x, int ori) {
	switch ( ori) {
		case Notebook.TABS_TOP:
			drawBreakAtTop( g, c, tw, x);
			break;
		case Notebook.TABS_BOTTOM:
			drawBreakAtBottom( g, c, tw, x);
			break;
	}
}
}

/**
 * class to implement a notebook widget (basically a container with
 * variable contents/pages). Rather than implementing a notebook class
 * as a another container widget (the usual way), we have choosen to
 * implement it just as a control widget that can be used to switch
 * between different layouts (you can think of it as a visual front end
 * for a LayoutManager like the CardLayout). This design implies that
 * neither notebook- "pages" nor widgets within these "pages" are childs
 * of their notebook. And this implies that you can have widgets
 * appearing on more than a single "page". This, in turn, reliefs you
 * of the burden to sync several widgets displaying the same data.
 * And - best of all - this can be implemented MUCH cleaner than the
 * usual (container-) notebook. Period.
 *
 * 
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class Notebook
  extends Widget
  implements HotKeyProcessor
{
	Vector Pages = new Vector();
	Layouter FrameLayouter;
	PageSpec TopPage;
	int TabOffs = 0;
	int TabEnd;
	static String Shift;
	static String BackShift;
	static int ShiftLen;
/**
 * triggered by TopPage change
 * the observer may call the lock() method to disable page change
 * parameters: int[] ( previous top index, next top index)
 */
	public ObserverSocket OsTopPage = new ObserverSocket( this);
	boolean Locked = false;
	final public static int TABS_TOP = 0;
	final public static int TABS_BOTTOM = 1;
	int Orientation = TABS_TOP;
	Rectangle TabRect = new Rectangle();
	Rectangle FrameRect = new Rectangle();

static {
	Shift = ">>";
	BackShift = "<<";
	ShiftLen = BookTab.Fm.stringWidth( Shift);
}

public Notebook () {
	setFont( Awt.BookTabFont);
	registerHotKeys();
}

public Notebook ( String labels) {
	this();
	StringTokenizer tok = new StringTokenizer( labels, ",:;");
	while( tok.hasMoreTokens() ){
		String s = tok.nextToken();
		int idx = s.indexOf( '@');
		if ( idx > -1)
			addPage( s.substring( 0, idx), s.substring( idx+1));
		else
			addPage( s, null );
	}
	setTopPage( 0);
}

public void addNotify () {
	super.addNotify();
	LayoutManager lom = getParent().getLayout();
	if ( lom instanceof Layouter)
		FrameLayouter = (Layouter) lom;
	if ( TopPage == null)
		setTopPage( 0);
}

/**
 * use tabText of <null, "*" or " "> 
 * to build a tabless page ( minor)
 */
public int addPage ( String tabText, String status) {
	addPage( tabText, status, Pages.size() );
	return Pages.size();
}

public boolean addPage ( String tabText, String status, int layoutIdx) {
	return insertPage( layoutIdx, tabText, status);
}

void drawBookFrame ( Graphics g) {

	if ( Awt.PlainBook) {
		g.setColor( Color.white);
		g.drawLine( 0, FrameRect.y, Width, FrameRect.y);
		g.drawLine( 0, FrameRect.y+1, Width, FrameRect.y+1);
		g.setColor( Color.black);
		g.drawLine( 0, FrameRect.y+FrameRect.height-1, Width, FrameRect.y+FrameRect.height-1);
	}
	else {
		GraphicsLib.paint3dBox( g, 0, FrameRect.y,
		                        Width, FrameRect.height,
		                        false, getBackground() );
	}

	drawSpins( g);
	drawStatus( g);
}

void drawSpins ( Graphics g) {
	int ext = BookTab.Fm.getHeight();
	int th = FrameRect.y;
	int maxExt = FrameRect.height-2*Awt.BorderWidth;
	int x = 4;
	int y = th+Awt.BorderWidth+(maxExt-ext)/2+BookTab.Fm.getAscent();

	g.setFont( Awt.BookTabFont );
	g.setColor( Color.black);
	g.drawString( BackShift, x, y);
	g.drawString( Shift, Width-ShiftLen-x, y);

	ext = Math.min( ext, maxExt);
	y = th+Awt.BorderWidth+(maxExt-ext)/2;

	x += 4 + ShiftLen;
	Color c = EventHandler.HasFocus ? Awt.FocusClr : Color.white;
	GraphicsLib.fillTri( g, x, y+ext, x+ext, y+ext, x, y, c);
	g.setColor( Color.black);
	g.drawRect( x, y, ext-1, ext-1);
	g.drawLine( x, y, x+ext-1, y+ext-1);

	x = Width - ShiftLen - 8 - ext;
	GraphicsLib.fillTri( g, x, y+ext, x+ext, y+ext, x+ext, y, Color.white);
	g.setColor( Color.black);
	g.drawRect( x, y, ext-1, ext-1);
	g.drawLine( x, y+ext-1, x+ext-1, y);  
}

boolean drawStatus ( Graphics g) {

	if ( (TopPage == null) || (TopPage.Status == null) )
		return false;

	int ma = BookTab.FmSmall.getMaxAscent();
	int fh = BookTab.FmSmall.getHeight();
	int th = FrameRect.y;
	int sh = FrameRect.height - Awt.BorderWidth;
	int y = th + ( sh - fh) / 2 + ma;

	int x1 = 2*(ShiftLen+4);
	int x2 = Width - x1; 
	int sw =	BookTab.FmSmall.stringWidth( TopPage.Status);
	
	g.setFont( Awt.BookSmallFont);
	g.drawString( TopPage.Status, x1 + ((x2-x1)-sw) / 2, y);

	return true;
}

void drawTabs ( Graphics g) {
	int   w = 0;
	Color c = getBackground();

	for ( TabEnd=TabOffs; TabEnd<Pages.size(); TabEnd++) {
		PageSpec pg = (PageSpec) Pages.elementAt( TabEnd);
		if ( pg.Tab != null){
			pg.Tab.drawAt( g, c, w, Orientation);
			w += pg.Tab.TabWidth;
			if ( w > Width) {
				pg.Tab.drawBreakTab( g, c, Width, w, Orientation);
				break;
			}
		}
	}
	TabEnd--;
	
	if ( w < Width) {
		g.setColor( c);
		switch ( Orientation) {
			case TABS_TOP:
				g.fillRect( w, 0, Width - w, BookTab.TabHeight);
				break;
			case TABS_BOTTOM:
				g.fillRect( w, Height - BookTab.TabHeight, Width - w, BookTab.TabHeight);
				break;
		}
	}  
}

public int getIndex ( String label) {
	for ( int i=0; i<Pages.size(); i++){
		PageSpec p = (PageSpec)Pages.elementAt( i);
		if ( p.hasLabel( label) )
			return i;
	}
	return -1;
}

public int getOrientation () {
	return Orientation;
}

public void gotFocus() {
	Graphics g = getGraphics();
	if ( g != null) {
		drawSpins( g);
		g.dispose();
	}
}

public boolean insertPage ( int insIdx, String tabText, String status) {
	return insertPage( insIdx, tabText, status, insIdx);
}

public boolean insertPage ( int insIdx, String tabText, String status, int layoutIdx) {
	try {
		PageSpec pg = new PageSpec( layoutIdx, tabText, status, this);
		Pages.insertElementAt( pg, insIdx);
		if ( isShowing() )
			redraw();
		return true;
	}
	catch( Exception e) {
		return false;
	}
}

public void lock() {
	Locked = true;
}

public void lostFocus() {
	Graphics g = getGraphics();
	if ( g != null) {
		drawSpins( g);
		g.dispose();
	}
}

public void mouse1Up () {

	if ( !isEnabled()) return;
	
	
	// Windows sometimes messes up the ect.x/y coordinates in MOUSE_DOWN events, so
	// let us do Button/Menu like page switching

	int mx = Mouse.Pos.x;
	int my = Mouse.Pos.y;
	int w = 0;

	if ( TabRect.inside( mx, my) ) {
		for ( int i=TabOffs; i<=TabEnd; i++) {
			PageSpec pg = (PageSpec)Pages.elementAt( i);
			if ( pg.Tab == null)
				continue;
			if ( w + pg.Tab.TabWidth > mx){
				setTopPage( pg);
				break;
			}
			w += pg.Tab.TabWidth;
		}
	}
	else {
		int dw = 4+ShiftLen;

		if ( mx < dw)
			shiftLeft();
		else if ( mx < 2*dw )
			prevPage();
		else if ( mx > Width-dw)
			shiftRight();
		else if ( mx > Width-2*dw)
			nextPage();
	}
}

public void nextPage() {
	if ( TopPage == null)
		setTopPage( 0);
	else
		setTopPage( Pages.indexOf( TopPage) +1);
}

protected void posChanged () {
	int th = BookTab.TabHeight;
	
	switch ( Orientation) {
		case TABS_BOTTOM:
			TabRect.reshape( 0, Height-th, Width, th);
			FrameRect.reshape( 0, 0, Width, Height-th);
			break;
		default:
			TabRect.reshape( 0, 0, Width, th);
			FrameRect.reshape( 0, th, Width, Height-th);
			break;
	}
}

public void prevPage() {
	if ( TopPage == null)
		setTopPage( 0);
	else
		setTopPage( Pages.indexOf( TopPage) -1);
}

public void processKey( Object sel) {
	if ( sel.equals( "nextPage"))
		nextPage();
	else if ( sel.equals( "prevPage"))
		prevPage();
	else if ( sel.equals( "shiftLeft"))
		shiftLeft();
	else if ( sel.equals( "shiftRight"))
		shiftRight();
	else
		super.processKey( sel);
}

public void redraw ( Graphics g) {
	drawBookFrame( g);
	drawTabs( g);
}

void registerHotKeys () {
	addHotKey( Awt.NotebookNextKeys[0], Awt.NotebookNextKeys[1], this, "nextPage");
	addHotKey( Awt.NotebookPrevKeys[0], Awt.NotebookPrevKeys[1], this, "prevPage");
	addHotKey( Awt.NotebookLeftKeys[0], Awt.NotebookLeftKeys[1], this, "shiftLeft");
	addHotKey( Awt.NotebookRightKeys[0], Awt.NotebookRightKeys[1], this, "shiftRight");
}

public boolean removePage ( int pageIdx ) {
	try {
		PageSpec ps = (PageSpec)Pages.elementAt( pageIdx);
		if ( ps == TopPage)
			TopPage = null;
		Pages.removeElement( ps);
		if ( isShowing() )
			redraw();
		return true;
	}
	catch( Exception e) {
		return false;
	}

}

public void setEnabled ( boolean b) {

	for ( int i=0; i<Pages.size(); i++)
		((PageSpec)Pages.elementAt( i)).setEnabled( b);

	super.setEnabled( b);
}

public boolean setLabel ( int idx, String label) {
	try {
		PageSpec pg = (PageSpec)Pages.elementAt( idx);
		pg.setLabel( label);
		if ( isShowing() )
			redraw();
		return true;
	}
	catch( Throwable t) {
		return false;
	}
}

public void setOrientation ( int ori) {
	Orientation = ori;
	if ( isShowing() )
		redraw();
}

public boolean setStatus ( int idx, String status) {
	try {
		PageSpec pg = (PageSpec)Pages.elementAt( idx);
		pg.Status = status;
		if ( isShowing() )
			redraw();
		return true;
	}
	catch( Throwable t) {
		return false;
	}
}

boolean setTopPage ( PageSpec bp) {
	if ( (bp == TopPage) || (bp == null) )
		return false;

	int ci = (TopPage != null) ? Pages.indexOf( TopPage) : -1;
	int ni = Pages.indexOf( bp);

	Locked = false;
	int[] ia = { ci, ni}; 
	OsTopPage.notifyObservers( ia );

	if ( Locked)
		return false;

	if ( ci > -1)
		TopPage.IsOnTop = false;
	TopPage = bp;
	TopPage.IsOnTop = true;
	redraw();

	if ( FrameLayouter != null){
		if ( FrameLayouter.countLayouts() > TopPage.LayoutIndex)
			FrameLayouter.setLayout( TopPage.LayoutIndex);
	}

	return true;
}

public boolean setTopPage ( int idx) {
	try {
		PageSpec pg = (PageSpec)Pages.elementAt( idx);
		shiftShow( idx);
		return setTopPage( pg);
	}
	catch( Throwable t) {
		return false;
	}
}

void shiftLeft() {
	if ( TabOffs > 0){
		TabOffs--;
		redraw();
	}
}

void shiftRight() {
	if ( TabEnd < Pages.size()-1){
		TabOffs++;
		redraw();
	}
}

/**
 * shift pagetab to visible region ( -> setTopPage() )
 */
void shiftShow ( int idx) {
	if ( idx < TabOffs)
		TabOffs = idx;
	else if ( idx > TabEnd) {
		int tw = 0;
		for ( TabOffs=idx; TabOffs>-1; TabOffs--) {
			PageSpec pg = (PageSpec)Pages.elementAt( TabOffs);
			if ( pg.Tab != null)
				tw += pg.Tab.TabWidth;
			if ( tw > Width){
				TabOffs++;
				break;
			}
		}
	}
}

/**
 * This one is meant for updating the Notebook after
 * externally setting the layout (useful for notebook
 * pages that have different layouts)
 *
 */
public boolean updtBook ( int idx) {
	try {
		PageSpec pg = (PageSpec)Pages.elementAt( idx);
		shiftShow( idx);
		
		if ( (pg == TopPage) || (pg == null) ) return false;

		int ci = (TopPage != null) ? Pages.indexOf( TopPage) : -1;


		if ( ci > -1) TopPage.IsOnTop = false;
		TopPage = pg;
		TopPage.IsOnTop = true;
		redraw();

		return true;
	}
	catch( Throwable t) {
		return false;
	}
}
}

class PageSpec
{
	BookTab Tab;
	String Status;
	boolean IsOnTop = false;
	Notebook Master;
	int LayoutIndex;
	boolean Enabled = true;

public PageSpec ( int lIdx, String tabText, String status, Notebook bf) {
	Master = bf;
	LayoutIndex = lIdx;

	if (tabText != null) {
		if (! tabText.equals( " ") && ! tabText.equals( "*") )
			Tab = new BookTab( tabText, this);
	}
	Status = status;
}

boolean hasLabel( String label) {
	if ( Tab == null)
		return false;
	return Tab.TabLabel.equals( label);
}

boolean isEnabled() {
	return Enabled;
}

void setEnabled( boolean b) {
	Enabled = b;
}

void setLabel( String label) {
	if ( Tab == null)
		Tab = new BookTab( label, this);
	else
		Tab.TabLabel = label;
}

void toTop() {
	Master.setTopPage( this);
}
}
