package biss.awt;

import biss.Queue;
import java.awt.AWTEvent;
import java.awt.EventQueue;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.peer.ActiveEvent;

class PaneDrawEvent
  extends AWTEvent
  implements ActiveEvent
{
	PaneDrawer Drawer;

public PaneDrawEvent ( PaneDrawer pd) {
	super( pd, 0);
	Drawer = pd;
}

public void dispatch () {
	Drawer.Pane.drawPane();
	Awt.DefToolkit.sync();
	consume();
}
}

/**
 * Thread object used for asynchronous (accumulative) drawing in
 * panes. Requires the pane to implement the DrawablePane interface.
 * Useful to do time-consuming draw operations in the background
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author P.C.Mehlitz
 */
public class PaneDrawer
  extends Thread
{
	DrawablePane Pane;
	Queue DrawQueue;
	int Delay = 100;
	boolean StopIt = false;
	PaneDrawEvent Drawer;

public PaneDrawer ( DrawablePane pane, Queue queue ) {
	this( pane, queue, 0);
}

public PaneDrawer ( DrawablePane pane, Queue queue, int delay ) {
	Pane = pane;
	DrawQueue = queue;
	Delay = delay;
	Drawer = new PaneDrawEvent( this);

	setDaemon( true);
	setPriority( Thread.MIN_PRIORITY);
	start();
}

public void run () {
	EventQueue q = Awt.DefToolkit.getSystemEventQueue();
	long tLast = -1;
	
	try {
		while ( !StopIt ) {
			// if ( Pane.getPeer() == null ) break;
			
			if ( DrawQueue.empty() ) {
				synchronized ( DrawQueue ) {
					// termination safeguard. This is NOT the recommended way
					// to shut down a PaneDrawer thread because there might be
					// lengthy drawPane operations which might cause Graphics operations
					// after the Pane was closed (causing BadDrawable X errors).
					// However, most VMs seem to have problems with explicitly stopped
					// threads
					DrawQueue.wait(3000);
					tLast = -1;
				}
			}
			else {
				if ( q.peekEvent() != Drawer ) {
					while ( q.peekEvent() != null )
						sleep( 100);

					if ( (System.currentTimeMillis() - tLast) < 600 )
						sleep( 3000);
					
					q.postEvent( Drawer);
					if ( Delay > 0 ) sleep( Delay);
					
					tLast = System.currentTimeMillis();
				}
				else
					sleep( 200);
			}
		}
	}
	catch ( Exception x ) {
		x.printStackTrace();
	}
}

public void stopIt () {
	synchronized ( DrawQueue ) {
		StopIt = true;
		DrawQueue.notify();
	}
}
}
