package biss.awt;

import biss.CommandQueue;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Vector;

public class PopupMenu
  extends Window
  implements FocusListener, KeyListener, MouseListener, MouseMotionListener, SingleInstInterface
{
	Vector Contents;
	static int BorderWidth = 2;
	static int SepHeight = 4;
	int ItemHeight;
	int ItemOffset;
	MenuItem Selection;
	PopupMenu TopPane;
	PopupMenu SubPane;
	Component Owner;
	FontMetrics Fm;
	boolean MouseSelection;

public PopupMenu ( Component owner, Menu cont) {
	this( owner, cont.getItems() );
}

public PopupMenu ( Component owner, Vector v) {
	this( Awt.getFrame( owner));
	
	Owner = owner;
	Contents = v;
	
	GUIHandler.SingleInstWnd = this;
}

protected PopupMenu ( Frame fr) {
	super( fr);

	setFont( Awt.MenuFont);
	setCursor( Cursor.getPredefinedCursor( Awt.ActionCursor) );
	
	addFocusListener( this);
	addKeyListener( this);
	addMouseListener( this);
	addMouseMotionListener( this);
}

protected PopupMenu ( PopupMenu topPane) {
	this( Awt.getFrame( topPane) );
	
	Owner = topPane.Owner;
	TopPane = topPane;
}

public void dispose() {
	if ( TopPane == null){
		for ( PopupMenu p = SubPane; p != null; p = p.SubPane)
			p.dispose();
		GUIHandler.SingleInstWnd = null;
	}
	else {
		// disposing a focused window activates its parent and so disposes
		// the SingleInstWnd
		TopPane.requestFocus();
	}
	super.dispose();
	
	Owner.requestFocus();
//	CommandQueue.append( new SetFocusCmd( Owner));
}

public boolean disposeRequest( Component c) {
	dispose();
	return true;
}

protected void disposeSubMenus() {
	for ( PopupMenu p = SubPane; p != null; p = p.SubPane)
		p.dispose();
	SubPane = null;
	requestFocus();
}

public void focusGained( FocusEvent e) {
	if ( Selection == null){
		Selection = (MenuItem)Contents.firstElement();
		redrawItem( Selection, null, false);
	}
}

public void focusLost( FocusEvent e) {
}

protected int getHeight() {
	int    cy = 0;
	Vector v  = Contents;
	int    vs = v.size();

	for ( int i=0; i<vs; i++) {
		if (((MenuItem)v.elementAt( i)).isSeparator() )
			cy += SepHeight;
		else
			cy += ItemHeight;
	}

	return cy+ 2*BorderWidth + 1;  // add extra deco contour space
}

protected int getWidth() {
	int    cx = 0;
	Vector v  = Contents;
	int    vs = v.size();

	for ( int i=0; i<vs; i++) {
		int x = Fm.stringWidth( ((MenuItem)v.elementAt( i)).Text);
		if ( x > cx)
			cx = x;
	}

	return (cx + 2 * BorderWidth + 25) + 1;  // add extra deco contour space
}

protected int getYPos( MenuItem mi) {
	int    cy = 0;
	Vector v  = Contents;
	int    vs = v.size();

	for ( int i=0; i<vs; i++) {
		MenuItem mii = (MenuItem)v.elementAt( i);
		if ( mii == mi)
			return cy;
		if ( mii.isSeparator() )
			cy += SepHeight;
		else
			cy += ItemHeight;
	}

	return -1;
}

protected MenuItem itemAt( int yPos) {
	int    cy = ItemHeight;
	Vector v  = Contents;
	int    vs = v.size();

	for ( int i=0; i<vs; i++) {
		MenuItem mi = (MenuItem)v.elementAt( i);
		if ( mi.isSeparator() )
			cy += SepHeight;
		else {
			if ( yPos < cy)
				return mi;
			cy += ItemHeight;
		}
	}

	return null;
}

public void keyPressed( KeyEvent e) {
	switch( e.getKeyCode() ){
		case KeyEvent.VK_ESCAPE:
			Selection = null;
			rootPane().dispose();
			break;
		case KeyEvent.VK_RIGHT:
			if ( Selection instanceof MenuItemContainer)
				openSubMenu( Selection);
			break;
		case KeyEvent.VK_ENTER:
			processSelection();
			break;
		case KeyEvent.VK_UP:
			selectPrevItem();
			break;
		case KeyEvent.VK_DOWN:
			selectNextItem();
			break;
		case KeyEvent.VK_LEFT:
			// first item of subpane
			if ( (TopPane != null) && (Selection != null) && Contents.indexOf( Selection) == 0){
				disposeSubMenus();
				dispose();
				TopPane.SubPane = null;
				TopPane.requestFocus();
			}
	}
	
	e.consume();
}

public void keyReleased( KeyEvent e) {
	e.consume();
}

public void keyTyped( KeyEvent e) {
	e.consume();
}

public void mouseClicked( MouseEvent e) {
	e.consume();
}

public void mouseDragged( MouseEvent e) {
	e.consume();
}

public void mouseEntered( MouseEvent e) {
	e.consume();
}

public void mouseExited( MouseEvent e) {
	if ( Selection != null) {
		MenuItem mi = Selection;
		Selection = null;
		redrawItem( mi, null, false);
	}
	e.consume();
}

public void mouseMoved( MouseEvent e) {
	MenuItem mi = itemAt( e.getY());
	if ( mi != Selection)
		watchSelection(mi);
	e.consume();
}

public void mousePressed( MouseEvent e) {
	MenuItem mi = itemAt( e.getY() );
	MouseSelection = true;
	requestFocus();
	watchSelection( mi);
	e.consume();
}

public void mouseReleased( MouseEvent e) {

	e.consume();
	//fixes windows "action on opening" problem
	if ( MouseSelection && ((e.getModifiers() & e.BUTTON1_MASK) > 0 )) {
		if ( (Selection != null)&& (Selection instanceof MenuItemContainer) ) {
			redrawItem( Selection, null, true);
			openSubMenu( Selection);
		}
		else if (SubPane == null){
			if ( processSelection() )
				return;
		}
	}
	MouseSelection = true;
}

protected boolean openSubMenu( MenuItem mi) {
	if( ! mi.isEnabled() || ! (mi instanceof MenuItemContainer) ) 
		return false;

	Vector v = ((MenuItemContainer)mi).getItems();
	if ( (v == null) || (v.size() == 0) )
		return false;
		 
	if( SubPane != null ) {
		if ( SubPane.Contents == v ){
			SubPane.requestFocus();
			return true;
		}
		disposeSubMenus();
	}

	Rectangle rect = getBounds();

	SubPane = new PopupMenu( this );
	SubPane.Contents = v;
	SubPane.popUpAt( rect.x + rect.width, rect.y + getYPos( mi));

	return true;	
}

public void paint ( Graphics g) {
	Dimension	s = getSize();
	Vector		v = Contents;
	int		vs = v.size();
	int		y0 = 1;
	int		x0 = BorderWidth + 1;
	int		w  = s.width - 2*BorderWidth - 1;

	GraphicsLib.paintButton( g, 0, 0, s.width, s.height, false, Awt.MenuBackClr);

	for ( int i=0; i<vs; i++) {
		MenuItem mi = (MenuItem)v.elementAt( i);
		boolean sel = (mi == Selection);
		if ( sel)
			GraphicsLib.paint3dBox( g, x0, y0 + BorderWidth, w, ItemHeight, false, null);

		if ( mi.isSeparator() ){
			int yo = y0 + SepHeight / 2;
			g.setColor( Awt.ShadowDarkClr);
			g.drawLine( 4, yo+1, s.width - 4, yo+1);
			g.setColor( Awt.ShadowBrightClr);
			g.drawLine( 4, yo+2, s.width - 4, yo+2);
			y0 += SepHeight;
		}	
		else {
			mi.drawAt( g, 8, y0+ItemHeight-ItemOffset, false);
			if ( mi instanceof MenuItemContainer){
				GraphicsLib.drawRightTri( g, s.width-15, y0+ItemHeight/3,
			                          ItemHeight/2, ItemHeight/2, sel);
			}
			y0 += ItemHeight;
		}
	}
}

public void popUpAt ( int x, int y) {
	Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
	int w = getWidth();
	int h = getHeight();

	if ( x + w > d.width)		x = d.width - w;
	if ( y + h > d.height)	y = d.height - h;

	Selection = itemAt( 0);

	if ( Awt.AwtToolkit == Awt.TK_WIN ) {
		// WIN workaround: Window instances can only be reshaped if already visible
		setBounds( 0,0,0,0); setVisible( true); setBounds( x, y, w, h);
	}
	else {
		setBounds( x, y, w, h);
		setVisible( true);
	}

	CommandQueue.append( new ToFrontCmd( this));
	CommandQueue.append( new SetFocusCmd( this));
//	requestFocus();
}

protected boolean processSelection( ) {
	if ( (Selection == null) || ( ! Selection.isEnabled()) ||
			 (Selection instanceof MenuItemContainer) )
		return false;
		
	redrawItem( Selection, null, true);
	Awt.DefToolkit.sync();
	try { Thread.sleep( 50); } catch ( Exception x ) {}

	// make sure popup is closed before command is executed because
	// the command could bring up another topwindow (requesting the focus)
	// while the popup is trying to restore the focus to its owner
	rootPane().dispose();
	Awt.DefToolkit.sync();

	//invoke java.awt.MenuItem listeners
	Selection.process();	//may remove selection
	
	if ( (Owner instanceof CmdInterface) && (Selection != null) )
		((CmdInterface)Owner).getOsCommand().notifyObservers( Selection.Selector);
	Selection = null;

	return true;
}

public void redrawItem ( MenuItem mi, Graphics g, boolean hilight) {
	Dimension   s = getSize();
	int         y0 = getYPos( mi) + 1;
	int         x0 = BorderWidth + 1;
	int         w  = s.width - 2*BorderWidth - 1;
	boolean     sel = mi.equals( Selection);
	Graphics    dg = ( g != null) ? g : getGraphics();
	
	if ( dg == null)
		return;
		
	if ( sel)
		GraphicsLib.paint3dBox( dg, x0, y0 + BorderWidth, w, ItemHeight, false, null);
	else if ( ! mi.isSeparator() ){
		dg.setColor( getBackground() );
		dg.fillRect( x0, y0+BorderWidth, w, ItemHeight);
	}

	if ( mi.isSeparator() ){
		int yo = y0 + SepHeight / 2;
		dg.setColor( Awt.ShadowDarkClr);
		dg.drawLine( 4, yo+1, s.width - 4, yo+1);
		dg.setColor( Awt.ShadowBrightClr);
		dg.drawLine( 4, yo+2, s.width - 4, yo+2);
	}	
	else {
		mi.drawAt( dg, 8, y0+ItemHeight-ItemOffset, hilight);
		if ( mi instanceof MenuItemContainer){
			GraphicsLib.drawRightTri( dg, s.width-15, y0+ItemHeight/3,
			                          ItemHeight/2, ItemHeight/2, sel);
		}
	}
	
	if ( g == null)
		dg.dispose();
}

protected PopupMenu rootPane( ) {
	PopupMenu p = this;
	for(; p.TopPane != null; p = p.TopPane);
	return p;
}

public boolean selectItem( MenuItem mi) {
	MenuItem mit;

	if ( mi == null || mi.equals( Selection) || mi.isSeparator() )
		return false;

	if ( Selection != null) {
		mit = Selection;
		Selection = null;
		redrawItem( mit, null, false);
	}

	Selection = mi;
	redrawItem( Selection, null, false);

	return true;
}

public boolean selectNextItem() {
	MenuItem mi;
	Vector v = Contents;
	int    vs = v.size();
	
	int cs = (Selection != null) ? v.indexOf( Selection) : -1;

	if ( cs > vs - 2)
		return false;

	for ( int idx = cs + 1; idx < vs; idx++){
		mi = (MenuItem)v.elementAt( idx);
		if ( ! mi.isSeparator() ){
			selectItem( mi);    
			return true;
		}
	}

	return false;
}

public boolean selectPrevItem() {
	MenuItem mi;
	Vector v = Contents;
	int    vs = v.size();
	
	int cs = (Selection != null) ? v.indexOf( Selection) : vs;

	if ( cs > 0) {
		for ( int idx = cs - 1; idx > -1; idx--){
			mi = (MenuItem)v.elementAt( idx);
			if ( ! mi.isSeparator()){
				selectItem( mi);
				return true;
			}    
		}
	}
	return false;
}

public void setFont ( Font fnt) {
	Font uf = ( fnt != null) ? fnt : Awt.MenuFont;

	super.setFont( uf);
	Fm = Awt.getFontMetrics( uf);

	ItemHeight = 3*Fm.getHeight()/2;
	ItemOffset = ((ItemHeight - Fm.getHeight()) / 2) +  Fm.getDescent() - 1;
}

protected boolean watchSelection( MenuItem mi) {	
	if ( (mi == null) || (mi.equals( Selection)) )
		return false;

	selectItem( mi);
	if ( SubPane != null){
		if ( ! ( mi instanceof MenuItemContainer) || (((MenuItemContainer)mi).getItems() != SubPane.Contents) )
			disposeSubMenus();
	}

	return true;	
}
}
