package biss.awt;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;

/**
 * progress bar
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author Alex Boisvert
 */
public class ProgressBar
  extends DecoBox
{
	FontMetrics Fm;
	Font TextFont = Awt.StaticFont;
	Color TextColor = Color.black;
	Color ForeColor = Awt.ProgBarClr;
	Color BackColor;
	Color XorColor = Awt.ProgBarClr;
	boolean Carved = Awt.CarvedText;
	boolean XorMode = false;
	int Percent = 0;
	int MinValue = 0;
	int MaxValue = 100;
	int CurrentValue = 0;

public ProgressBar() {
	this( Border.BS_INSET);
}

public ProgressBar ( int style) {
	super(style);
	Fm = Awt.DefToolkit.getFontMetrics(TextFont);
}

public void blank(Graphics g, Color cb) {
	g.setColor(cb);
	g.fillRect( 0, 0, Width, Height);
}

public void drawBar(Graphics g) {
	int b2 = 2*BoxBorder.Ext;
	int w = Width - b2;
	int h = Height - b2;
	int p = Percent*w/100;

	g.setColor(ForeColor);	
	g.fillRect( BoxBorder.Ext, BoxBorder.Ext, p, h);
	
	if (BackColor != null)
		g.setColor(BackColor);
	else
		g.setColor(getParent().getBackground());
		
	g.fillRect(BoxBorder.Ext+p, BoxBorder.Ext, w-p, h);
}

public void drawText(Graphics g) {
	Color c = g.getColor();
	String str = String.valueOf(Percent)+"%";
	int x=(Width-Fm.stringWidth(str))/2;
	int y=Height-(Height-Fm.getHeight())/2-Fm.getDescent();

	g.setFont(TextFont);
	if (Carved) {
		g.setColor(Color.white);
		g.drawString(str, x+1, y+1);
	}
	if (XorMode) g.setXORMode(XorColor);
	else g.setColor(TextColor);
	g.drawString(str, x, y);

	g.setPaintMode(); // restore paint context
	g.setColor(c);
}

public int getValue() {
	return CurrentValue;
}

public void paint (Graphics g) {
	if ( isShowing() ) {
		drawBorder(g);
		drawBar(g);
		drawText(g);
	}
}

public void setBackground(Color c) {
	BackColor = c;
	repaint();
}

public void setCarved(boolean b) {
	Carved = b;
	XorMode = false; // mutually exclusive
	repaint();
}

public void setFont(Font f) {
	TextFont = f;
	Fm = Awt.DefToolkit.getFontMetrics(TextFont);
	repaint();
}

public void setForeground(Color c) {
	ForeColor = c;
	repaint();
}

public void setTextColor(Color c) {
	TextColor = c;
	repaint();
}

public void setValue(int value) {
	if (value < MinValue) CurrentValue = MinValue;
	else if (value > MaxValue) CurrentValue = MaxValue;
	else CurrentValue = value;
	Percent = 100*(CurrentValue-MinValue)/(MaxValue-MinValue);
	
	Graphics g = getGraphics();
	drawBar( g);
	drawText( g);
	g.dispose();
}

public void setXORColor(Color c) {
	XorColor = c;
	repaint();
}

public void setXORMode(boolean b) {
	XorMode = b;
	Carved = false; // mutually exclusive
	repaint();
}
}
