package biss.awt;

import biss.CommandQueue;
import biss.ObserverSocket;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Rectangle;

/**
 * scrollbar base ( HScrollBar / VScrollBar)
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
abstract public class ScrollBar
  extends Widget
  implements MouseMotionInterface
{
	int DragOffs;
	int LineInc = 1;
	int Max;
	int Min;
/**
 * triggered by any slider position change ( via slider or
 * positioning methods)
 * parameter: -
 */
	public ObserverSocket OsScroll = new ObserverSocket( this);
	int Pos;
	int ScrollOp;
	int StepPels = 1;
	int PageInc = 1;
	ScrollHandler Repeater = new ScrollHandler();
	Graphics ScrollGraph;
	final public static int UNIT_INCREMENT = 1;
	final public static int UNIT_DECREMENT = 2;
	final public static int BLOCK_DECREMENT = 3;
	final public static int BLOCK_INCREMENT = 4;
	final public static int TRACK = 5;
	int LastOp;
	Rectangle SliderRect = new Rectangle();
	static boolean IsSoftScroll;

class ScrollHandler
  extends RepeatCommand
{
	int ScrollCount;

public ScrollHandler () {
	super( null, true);
}

public void actionNotify() {
	
	if ( ScrollOp == 0){
		ScrollCount = 0;
		return;
	}
	ScrollCount++;	
	
	if ( Awt.SoftScroll && Mouse.isRightActive() ){
		CommandQueue.setDelay( 0);
		IsSoftScroll = true;
	}
		
	switch( ScrollOp) {
		case UNIT_INCREMENT:
			if ( Pos > Min)	scrollDown( LineInc);
			else						endScroll();
			break;
		case UNIT_DECREMENT:
			if ( Pos < Max)	scrollUp( LineInc);
			else						endScroll();
			break;
	}
	
	if ( Mouse.isAnyActive() ){
		repeat();
	}
	else {
		endScroll();
		ScrollCount = 0;
	}
		
	IsSoftScroll = false;
}

public void stop() {
	if ( ScrollCount == 0)
		actionNotify();
	ScrollOp = 0;
	super.stop();
}
}

public ScrollBar( int min, int max, int pos) {
	Min = min;
	Max = Math.max( max, min);
	Pos = pos;

	setBackground( Awt.ScrollBarClr);
	setBorder( Border.BS_INSET);
}

public boolean canGetFocus () {
	return false;
}

public void end( ) {
	setPosition( Max);
}

protected void endScroll() {
	int lastOp = ScrollOp;
	
	Repeater.stop();
	DragOffs = 0;
	ScrollOp = 0;

	if ( ( lastOp == UNIT_INCREMENT) || ( lastOp == UNIT_DECREMENT) )
		redrawSpins();
			
	if ( ScrollGraph != null) {
		ScrollGraph.dispose();
		ScrollGraph = null;
	}
}

public int getLastOp() {
	return LastOp;
}

public int getLineIncrement() {
	return LineInc;
}

public int getMaximum() {
	return Max;
}

public int getMinimum() {
	return Min;
}

public Dimension getMinimumSize () {
	return getPreferredSize();
}

public int getPageIncrement() {
	return PageInc;
}

public int getPosition() {
	return Pos;
}

public int getStepPels() {
	return StepPels;
}

public void home( ) {
	setPosition( Min);
}

public void mouse1Down () {
	ScrollGraph = getGraphics();
	mouseDownScroll();
}

public void mouse1Drag () {
}

public void mouse1Up () {
	mouseUp();
}

public void mouse2Down () {
	ScrollGraph = getGraphics();
	mouseDownScroll();
}

public void mouse2Drag () {
}

public void mouse2Up () {
	mouseUp();
}

void mouseDownScroll () {
}

public void mouseMove () {
}

void mouseUp() {

	switch( ScrollOp ) {
	case 1:
	case 2:
		Repeater.stop();
		redrawSpins();
		return;
	case 3:
		scrollDown( PageInc);
		break;
	case 4:
		scrollUp( PageInc);
		break;
	case 5:
		EventHandler.setCursor( Cursor.DEFAULT_CURSOR);
	}

	endScroll();
}

public void pageDown( ) {
	setPosition( Pos - PageInc);
}

public void pageUp( ) {
	setPosition( Pos + PageInc);
}

public void redraw( Graphics g) {
	if ( SliderRect.width == 0)
		updateSliderRect();
	redrawSlider( g);
	redrawSpins( g);
}

protected void redrawSlider () {
	Graphics g = ScrollGraph;
	if ( g != null )
		redrawSlider( g);
	else {
		g = getGraphics();
		if ( g != null) {
			redrawSlider( g);
			g.dispose();
		}
	}
}

abstract protected void redrawSlider( Graphics g);

protected void redrawSpins () {
	Graphics g = ScrollGraph;
	if ( g != null )
		redrawSpins( g);
	else {
		g = getGraphics();
		if ( g != null) {
			redrawSpins( g);
			g.dispose();
		}
	}
}

abstract protected void redrawSpins( Graphics g);

public void scrollDown( int steps) {
	setPosition( Pos - steps);
}

public void scrollUp( int steps) {
	setPosition( Pos + steps);
}

public void setLineIncrement( int inc) {
	LineInc = inc;
}

public void setMaximum( int max) {
	setMinMax( Min, max);
}

public void setMinMax( int min, int max) {
	Min = min;
	Max = Math.max(max, min);

	if ( Pos > Max)       Pos = Max;
	else if ( Pos < Min)  Pos = Min;

	updateSlider();
}

public void setMinimum( int min) {
	setMinMax( min, Max);
}

public void setPageIncrement( int inc) {
	setPageIncrement( inc, true);
}

public void setPageIncrement( int inc, boolean upd) {
	if ( PageInc != inc){
		PageInc = inc;
		if ( upd)
			updateSlider();
	}
}

public boolean setPosition( int sPos) {
	if ( sPos == Pos)
		return false;

	if ( sPos > Max){
		if ( Pos == Max )
			return false;
		sPos = Max;
	}
	else if ( sPos < Min){
		if ( Pos == Min)
			return false;
		sPos = Min;
	}

	Pos = sPos;
	updateSliderPos();
	redrawSlider();
	OsScroll.notifyObservers();

	return true;
}

public void setStepPels( int p) {
	StepPels = p;
}

public void setValue( int value) {
	//for java.awt.Scrollbar compatibility
	setPosition( value);
}

public void setValues( int pos, int min, int max) {
	setValues( pos, min, max, true);
}

public void setValues( int pos, int min, int max, boolean upd) {
	if ( (Pos == pos) && (Min == min) && (Max == max))
		return;
		
	Min = min;
	Max = Math.max( max, min);

	if ( pos < Min)      Pos = Min;
	else if ( pos > Max) Pos = Max;
	else                 Pos = pos;

	if ( upd)
		updateSlider();
}

public void setValues( int value, int visible, int minimum, int maximum) {
	//for java.awt.Scrollbar compatibility
	setValues( value, minimum, maximum, false);
	setPageIncrement( visible, false);
	updateSlider();
}

public String toString() {
	String s = "biss.awt.ScrollBar [Pos: " + Pos + " Min: " + Min +
	" Max: " + Max + " PageInc: " + PageInc + "]";
	return s;
}

boolean updatePosition( int sPos) {
	if ( sPos == Pos)
		return false;

	if ( sPos > Max){
		if ( Pos == Max )
			return false;
		sPos = Max;
	}
	else if ( sPos < Min){
		if ( Pos == Min)
			return false;
		sPos = Min;
	}

	Pos = sPos;
	updateSliderPos();
	redrawSlider();

	return true;
}

public void updateSlider () {
	updateSliderRect();
	if ( isShowing() )
		redrawSlider();
}

abstract protected void updateSliderPos();

abstract protected void updateSliderRect();
}
