package biss.awt;

import biss.ObserverSocket;
import java.awt.Color;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;

/**
 * slider widget
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class Slider
  extends Widget
  implements HotKeyProcessor, MouseMotionInterface
{
	int Min;
	int Max;
	int Pos;
	int DragOffs;
	FontMetrics Fm;
	String StPos;
	ObserverSocket OsPosChanged = new ObserverSocket( this);
	int Scales = 4;
	int YStPos;
	Rectangle SliderRect;
	Integer CPos;

public Slider () {
	this( 0, 100, 0);
}

public Slider ( int min, int max) {
	this( min, max, min);
}

public Slider ( int min, int max, int pos) {
	int d = Awt.BorderWidth;
	SliderRect = new Rectangle( d, d, 4*Awt.SysFontHeight, 4*Awt.SysFontHeight/3);
	setValues( min, max, pos);
	setFont( null);
	registerHotKeys();
}

void blankSlider ( Graphics g, int oldX ) {	
	int x, w;

	if ( oldX < SliderRect.x ) {
		x = oldX;
		w = SliderRect.x - x;
	}
	else {
		w = oldX - SliderRect.x + 1;
		x = SliderRect.x + SliderRect.width;
	}
	g.setClip( x, SliderRect.y, w, SliderRect.height);

	g.setColor( getBackground());
	g.fillRect( x, 0, w, SliderRect.height + Awt.BorderWidth);
	drawFrame( g);

	g.setClip( 0, 0, Width, Height);
}

public void charInput( char c, int mod) {
	if ( (c == '-') && (CPos != null) ){
		CPos = new Integer( -1 * CPos.intValue() );
		redraw();
	}
	else if ( Character.isDigit( c)){
		int pv = (CPos != null) ? CPos.intValue()*10 : 0;
		CPos = new Integer( pv + Character.digit( c, 10));
		redraw();
	}
}

void drawFocusMark ( Graphics g) {
		GraphicsLib.drawFocusMark( g, Width-Awt.FocusMarkWidth-1,
															SliderRect.height + 2*Awt.BorderWidth,
	                         		Awt.FocusMarkWidth);
}

void drawFrame ( Graphics g) {
	int h = SliderRect.height + 2*Awt.BorderWidth;
	int hh  = 2*Awt.BorderWidth + 4;
	int y0 = Awt.BorderWidth + (SliderRect.height - hh) / 2;
	GraphicsLib.paint3dBox( g, 0, y0, Width, hh, true, Awt.ScrollBarClr );

	if ( Scales < 1 )
		return;

	int x0		= SliderRect.width/2 + Awt.BorderWidth;
	int lPos	= Min;
	int ldx 	= (Max-Min) / Scales;
	double dx	= (double) (Width - 2*x0) / (double) Scales;
	int dh    = Awt.SysFontHeight / 4;

	g.setColor( Color.black);

	for ( int i=0; i<=Scales; i++){
		int xPos = x0 + (int) (i*dx);
		String pStr = String.valueOf( lPos);
		g.drawLine( xPos, h, xPos, h+dh);
		g.setFont( getFont() );
		g.drawString( pStr, xPos - Fm.stringWidth( pStr) / 2, h+Awt.SysFontHeight);
		lPos += ldx;
	}
}

void drawSlider ( Graphics g) {
	GraphicsLib.paint3dBox ( g, SliderRect.x, SliderRect.y,
			SliderRect.width, SliderRect.height, false, Awt.ButtonBackClr );

	g.setFont( getFont() );

	if ( CPos == null) {													
		g.setColor( Pos < 0 ? Color.red : Color.blue );
		g.drawString( StPos, SliderRect.x + ( SliderRect.width - Fm.stringWidth( StPos)) / 2, YStPos );
	}
	else {
		g.setColor( Color.black);
		String cs = "*" + CPos.intValue();
		g.drawString( cs, SliderRect.x + ( SliderRect.width - Fm.stringWidth( cs)) / 2, YStPos ); 
	}
	
	int x0 = SliderRect.x + SliderRect.width/2;
	int y0 = SliderRect.y + SliderRect.height - 6;
	int y1 = y0 + 5;
	
	g.setColor( Color.black);
	g.drawLine( x0-1, y0, x0-1, y1);
	g.drawLine( x0+1, y0, x0+1, y1);
	g.setColor( Color.white);
	g.drawLine( x0, y0, x0, y1);
}

void eraseFocusMark ( Graphics g) {
	g.setColor( getBackground() );
	g.fillRect( Width - Awt.FocusMarkWidth - 1, SliderRect.height + 2*Awt.BorderWidth,
							Awt.FocusMarkWidth + 1, Awt.FocusMarkWidth + 2 );
}

public int getPosition () {
	return Pos;
}

public void gotFocus () {
	Graphics g = getGraphics();
	if ( g != null) {
		drawFocusMark( g);
		g.dispose();
	}
}

public void lostFocus () {
	Graphics g = getGraphics();
	if ( g != null) {
		eraseFocusMark( g);
		g.dispose();
	}
}

public void mouse1Down () {
	int x = Mouse.Pos.x;
	int y = Mouse.Pos.y;
	if ( SliderRect.contains( x, y) )
		DragOffs = x - SliderRect.x;
	else
		DragOffs = 0;
}

public void mouse1Drag () {
	if ( DragOffs == 0)
		return;
	int x = Mouse.Pos.x;
	int np = ((x - DragOffs + 1)*(Max - Min)) / (Width - SliderRect.width - 2) + Min;
	setPosition( np);
}

public void mouse1Up () {
	DragOffs = 0;
}

public void mouse2Drag () {
}

public void mouseMove () {
}

protected void posChanged () {
	int d = Awt.BorderWidth;
	SliderRect.x = d + (Pos - Min) * (Width - SliderRect.width - 2*d) / (Max - Min);
}

public void processKey( Object sel) {

	if ( sel.equals( "updatePos")){
		if ( CPos != null){
			int i = CPos.intValue();
			CPos = null;
			setPosition( i );
		}
	}
	else if ( sel.equals( "clearPos")){
		CPos = null;
		redraw();
	}
	else if ( sel.equals( "left"))
		setPosition( Pos-1);
	else if ( sel.equals( "right"))
		setPosition( Pos+1);
	else if ( sel.equals( "pageRight"))
		setPosition( Pos+10);
	else if ( sel.equals( "pageLeft"))
		setPosition( Pos-10);
	else if ( sel.equals( "home"))
		setPosition( Min);
	else if ( sel.equals( "end"))
		setPosition( Max);
	else
		super.processKey( sel);
}

public void redraw ( Graphics g) {
	drawFrame( g);
	drawSlider( g);
	if ( EventHandler.HasFocus )
		drawFocusMark( g);

}

void registerHotKeys () {
	addHotKey( Key.Newline, HotKey.Lone, this, "updatePos");
	addHotKey( Key.Escape, HotKey.Lone, this, "clearPos");
	addHotKey( Key.CursorLeft, HotKey.Lone, this, "left");
	addHotKey( Key.CursorRight, HotKey.Lone, this, "right");
	addHotKey( Key.PageUp, HotKey.Lone, this, "pageRight");
	addHotKey( Key.PageDown, HotKey.Lone, this, "pageLeft");
	addHotKey( Key.Home, HotKey.Lone, this, "home");
	addHotKey( Key.End, HotKey.Lone, this, "end");
}

public synchronized void setFont ( Font fnt) {
	Font uf = ( fnt != null) ? fnt : Awt.MenuFont;
	
	if ( getFont() == uf)
		return;
		
	super.setFont( uf);
	Fm = Awt.getFontMetrics( uf);
	YStPos = SliderRect.y + Fm.getHeight() - Fm.getDescent() + 1;
	
	if ( isShowing() )
		redraw();
}

public void setPosition ( int p) {
	if (p > Max) {
		if ( Pos == Max)	return;
		Pos = Max;
	}
	else if (p < Min) {
		if ( Pos == Min)	return;
		Pos = Min;
	}
	else Pos = p;

	StPos = String.valueOf( Math.abs( Pos));
	
	if ( isShowing() ) {
		int oldX = SliderRect.x;
		Graphics g = getGraphics();
		posChanged();
		blankSlider( g, oldX);
		drawSlider( g);
		g.dispose();
	}

	OsPosChanged.notifyObservers();
}

public void setScales ( int s) {
	Scales = s;
	if ( isShowing() )
		redraw();
}

public void setValues ( int min, int max, int pos) {
	Min = min;
	Max = max;
	Pos = pos;
	
	StPos = String.valueOf( Math.abs( Pos));
	
	if ( isShowing() )
		redraw();
}
}
