package biss.awt;

import biss.CommandQueue;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.PaintEvent;
import java.util.Enumeration;
import java.util.Vector;

/**
 * abstract base class for Splitbars ( HSplitBar / VSplitBar)
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author P.C.Mehlitz
 */
abstract public class SplitBar
  extends Component
  implements MouseListener, MouseMotionListener
{
	boolean Dragging = false;
	int Height;
	int Id;
	int Max;
	int Min;
	Graphics PGr;
	Container Parent;
	boolean ResizeSelf;
	Vector Siblings;
	int Width;
	int X;
	int XPos;
	int Y;
	int YPos;
	PaintCmd RetainPainter = (Awt.FilterPaints > 0) ? new PaintCmd( this)  : null;

public void addNotify () {
	super.addNotify();
	Parent = getParent();
	addMouseListener( this);
	addMouseMotionListener( this);
}

public void addSibling ( Component c ) {
	if ( Siblings == null )
		Siblings = new Vector(4);
	Siblings.addElement( c);
}

abstract void computeSiblings ();

public Frame getFrame() {
	Container c;
	for ( c = Parent; c!= null; c=c.getParent() ){
		if ( c instanceof Frame)
			return (Frame)c;
	}
	return null;
}

public int getId () {
	return Id;
}

public Vector getSiblings () {
	return Siblings;
}

void initDrag () {
	Component c;
	Rectangle r;
	int i, n;
	LocalDispatcher ld = null;
	
	if ( Siblings == null )
		computeSiblings();
	n = Siblings.size();

	PGr = Parent.getGraphics();
	if ( PGr == null )
		return;
	PGr.setColor( Color.lightGray);

	Min = 100000;
	Max = 0;

	if ( Awt.AwtToolkit != Awt.TK_OTHER)
			ld = new LocalDispatcher();

	for ( i=0; i<n; i++ ) {
		c = (Component) Siblings.elementAt( i);
		r = c.getBounds();

		if ( ! ( c instanceof SplitBar ) ){
				c.setVisible( false);
				
				//wait for async repaint() caused by hiding
				if ( ld != null)
					ld.dispatchUntil( PaintEvent.UPDATE);
					
				PGr.fill3DRect( r.x, r.y, r.width, r.height, true);
		}
		
		updateMoveBoundaries( r);
	}

	PGr.setColor( Color.black);
	PGr.setXORMode( Color.lightGray);

}

public void mouseClicked ( MouseEvent e) {
}

public void mouseDragged ( MouseEvent e) {
}

public void mouseEntered ( MouseEvent e) {
}

public void mouseExited ( MouseEvent e) {
}

public void mouseMoved ( MouseEvent e) {
}

public void mousePressed ( MouseEvent e ) {
e.consume();
	Dragging = true;
	ResizeSelf = true;

	X = -1;
	Y = -1;

	initDrag();
}

public void mouseReleased ( MouseEvent e ) {
	Dragging = false;
	
	if ( ResizeSelf )
		rearrange();

	PGr.dispose();
	PGr = null;
}

abstract void rearrange ();

public void setBounds( int min, int max ) {
	Min = min;
	Max = max;
}

public void setBounds ( int x, int y, int width, int height ) {
	super.setBounds( x, y, width, height);

	XPos = x; YPos = y;
	Width = width; Height = height;
}

public void skipResize () {
	ResizeSelf = false;
}

abstract void updateMoveBoundaries ( Rectangle r );
}
