package biss.awt;

import biss.ObserverSocket;
import biss.awt.LineBuffer;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.util.Observable;

/**
 * replaces java.awt.TextField
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class TextField
  extends Widget
  implements HotKeyProcessor, MouseMotionInterface
{
/**
 * triggered by Enter key ( validate contents)
 * parameter: -
 */
	public ObserverSocket OsEnter = new ObserverSocket( this);
	public ObserverSocket OsModified = new ObserverSocket( this);
/**
 * triggered by ESC key ( clear contents)
 * parameter: -
 */
	public ObserverSocket OsReset = new ObserverSocket( this);
	LineBuffer Contents = new LineBuffer( null);
	int CursorIdx;
	int Mode;
	boolean ReadOnly;
	int SelEnd;
	int SelStart;
	int XBorder = 3;
	int XOffs;
	int YOffs;
	boolean RightAlign;
	EntryValidator Validator;
	EntryKeyFilter KeyFilter;
	public static int DefaultBorderStyle = Border.BS_STEP;
	TextCursor Cursor;
	boolean Modified;
	static FontMetrics DefFm;
	boolean Validated;
	LineBuffer RealContents;
	char EchoChar;
	boolean Modifed;

static {
	DefFm = Awt.getFontMetrics( Awt.SysFont);
}

public TextField(){
	this( null, null);
}

public TextField( Font fnt) {
	this( null, fnt);
}

public TextField( String defCont){
	this( defCont, null);
}

public TextField( String defCont, Font fnt) {
	super();
	CursorIdx = 0;
	SelStart  = 0;
	SelEnd    = 0;
	Modified  = false;

	Cursor = new TextCursor( Awt.EntryBackClr);
	setBackground( Awt.EntryBackClr);
	setForeground( Awt.EntryForeClr);
	setCursor( java.awt.Cursor.getPredefinedCursor( java.awt.Cursor.TEXT_CURSOR));

	setFont( fnt);
	FBorder.setBorder( DefaultBorderStyle);

	setContents( defCont);
	registerHotKeys();
}

public TextField( int xBorder, Font fnt, Color cf, Color cb) {
	this( null, fnt);
	setForeground( cf);
	setBackground( cb);
	XBorder   = xBorder;
}

void adjustContents() {

	if ( Height == 0)    //not yet initialized
		return;

	YOffs = (Height - Contents.CharHeight) / 2; // always vertically centered
	Cursor.setYPos( Height - YOffs);
	Cursor.setHeight( Contents.CharHeight);

	if ( ! RightAlign)
		XOffs = XBorder + FBorder.Ext;
	else
		XOffs = Width - 2*FBorder.Ext - Contents.charsWidth() - XBorder;
}

void adjustToCursor( boolean update) {

	int tw;

	if ( RightAlign) {
		tw = Contents.charsWidth();
		XOffs = Width - 2*FBorder.Ext - tw - XBorder;
		if ( update)
			redraw();
		return;
	}

	tw = Contents.charsWidth( 0, CursorIdx, 0);
	if ( tw + XOffs  >= Width - FBorder.Ext ){
		XOffs -= (tw - Width + 20 + XOffs - XBorder);
		if ( update)
			redraw();
	}	
	else if ( tw + XOffs < FBorder.Ext){
		XOffs -= (tw + XOffs - XBorder - FBorder.Ext);
		if ( update)
			redraw();
	}	
}

void backSpace() {
	if ( ReadOnly)
		return;

	if ( SelStart != SelEnd){
		setModified( true);
		deleteSelection();
	}
	else if ( CursorIdx > 0){
		CursorIdx--;
		if ( RealContents != null)
			RealContents.remove( CursorIdx, CursorIdx + 1);
		Contents.remove( CursorIdx, CursorIdx + 1);
		updateSel();
		setModified( true);
		redraw();
	}
}

void  blankCursor( Graphics g) {

	if ( YOffs == 0)
		adjustContents();

	Cursor.clearAtOffs( g, XOffs, 0);

	if ( Contents.Len > CursorIdx){
		g.setColor( getForeground());
		g.setFont( getFont() );
		g.drawChars( Contents.Text, CursorIdx, 1, XOffs + Cursor.XPos,
		             Height - YOffs - Contents.MaxDescent );
	}
}

public void charInput( char c, int mod) {

	if ( ReadOnly)
		return;

	if ( (KeyFilter != null) && ( ! KeyFilter.isValidKey( CursorIdx, c)) )
		return;

	if ( SelStart != SelEnd){
		replaceSelectionWith( c);
		return;
	}

	if ( RealContents != null ) {
		RealContents.insert( c, CursorIdx);
		Contents.insert( EchoChar, CursorIdx);
	}
	else
		Contents.insert( c, CursorIdx);
		
	CursorIdx++;
	updateSel();

	setModified( true);
	redraw();
}

void cursorLeft( boolean extentSel) {

	if ( CursorIdx <= 0)
		return;

	Graphics g = getGraphics();
	if ( g == null )
		return;
	
	clip( g);

	blankCursor( g);
	CursorIdx--;
	adjustToCursor( true);
	if ( Mode == 1){
		updateSel();
		redraw( g);
	}	
	else if ( ! extentSel){
		if ( hasSelection() ){
			updateSel();
			redraw( g);
		}
		else {
			updateSel();
			drawCursor( g);
		}    
	}    
	else {
		SelEnd--;
		redraw( g);
	}

	g.dispose();
}

void cursorRight( boolean extentSel) {

	if ( CursorIdx >= Contents.Len)
		return;

	Graphics g = getGraphics();
	if ( g == null )
		return;
	
	clip( g);

	blankCursor( g);
	CursorIdx++;
	adjustToCursor(true);
	if ( Mode == 1){
		updateSel();
		redraw(g);
	}	
	else if ( ! extentSel){
		if ( hasSelection() ){
			updateSel();
			redraw( g);
		}
		else {
			updateSel();
			drawCursor( g);
		}    
	}    
	else {
		SelEnd++;
		redraw( g);
	}

	g.dispose();
}

void del() {
	if ( ReadOnly)
		return;

	if ( SelStart != SelEnd) {
		setModified( true);
		deleteSelection();
	}
	else if ( CursorIdx < Contents.Len) {        	
		updateSel();
		if ( RealContents != null )
			RealContents.remove( CursorIdx, CursorIdx + 1);
		Contents.remove( CursorIdx, CursorIdx + 1);
		setModified( true);
		redraw();
	}
}

public void deleteSelection() {
	int sb  = Math.min( SelStart, SelEnd);
	int len = Math.abs( SelStart - SelEnd);

	if ( ReadOnly)
		return;

	if ( RealContents != null)
		RealContents.remove( sb, sb + len);
	Contents.remove( sb, sb + len);

	if ( SelEnd > SelStart)
		CursorIdx = Math.max( CursorIdx - len, 0);

	updateSel();
	redraw();
	setModified( true);
}

void drawCursor ( Graphics g) {

	if ( YOffs == 0)
		adjustContents();

	Cursor.setXPos( Contents.posFor( CursorIdx));
	Cursor.drawAtOffs( g, XOffs, 0);
}

public String getContents() {
	// return copy of contents
	if ( RealContents != null )
		return RealContents.toString();
	return Contents.toString();
}

public int getContentsLen() {
	return Contents.Len;
}

public int getCursorIndex() {
	return CursorIdx;
}

public char getEchoCharacter() {
	return EchoChar;
}

public Dimension getMinimumSize() {
	int cx = 15 + Contents.charsWidth();
	int cy = 10 + Contents.CharHeight;
	
	return new Dimension( cx, cy);
}

public Dimension getMinimumSize( int cols) {
	//for java.awt.TextComponent compatibility
	int cx = 15 + Contents.Fm.charWidth( '0') * cols;
	int cy = 10 + Contents.CharHeight;

	return new Dimension( cx, cy);
}

public Dimension getPreferredSize( int cols) {
	return getMinimumSize( cols);
}

public String getSelection() {
	int   sb    = Math.min( SelStart, SelEnd);
	int   len   = Math.abs( SelStart - SelEnd);
	
	if ( RealContents != null)
		return new String ( RealContents.Text, sb, len);
	return new String ( Contents.Text, sb, len);
}

public int getSelectionEnd() {
	//for java.awt.TextComponent compatibility
	return Math.max( SelStart, SelEnd);
}

public int getSelectionStart() {
	//for java.awt.TextComponent compatibility
	return Math.min( SelStart, SelEnd);
}

public String getText() {
	//for java.awt.TextComponent compatibility
	return getContents();
}

public void gotFocus() {
	updateCursor();
}

public boolean hasSelection() {
	return ( SelStart != SelEnd);
}

public boolean isModified() {
	return Modified;
}

public boolean isReadOnly(){
	return ReadOnly;
}

public void lostFocus() {
	if ( hasSelection() )
		updateSel();
	updateCursor();
}

public void mouse1Down() {
	setCursorPos( Mouse.Pos.x, true);
	SelStart = SelEnd = CursorIdx;
}

public void mouse1Drag() {
	int cp = CursorIdx;
	setCursorPos( Mouse.Pos.x, false);
	SelEnd = CursorIdx;
	if ( cp != CursorIdx)
		redraw();
}

public void mouse1Up(){
	redraw();
}

public void mouse2Drag() {
}

public void mouseMove() {
}

protected void posChanged() {
	adjustContents();
}

public void processKey( Object sel) {

	if ( sel.equals( "toggleMode") ) {
		Mode = (Mode == 0) ? 1 : 0;
		updateSel();
		redraw();
	}
	else if ( sel.equals( "cut") ) {
		StringTransfer.setContents( getSelection() );
		deleteSelection();
	}
	else if ( sel.equals( "copy") )
		StringTransfer.setContents( getSelection() );
	else if ( sel.equals( "paste") )
		replaceSelectionWith( StringTransfer.getContents() );
	else if ( sel.equals( "newL") ){
		Validated = validateContents();
		if ( Validated ){
			redraw();
			OsEnter.notifyObservers( Contents.toString() );
		}
	}
	else if ( sel.equals( "esc") ){
		if ( ! ReadOnly) {
			Contents.free();
			if ( RealContents != null)
				RealContents.free();
			CursorIdx = 0;
			updateSel();
			XOffs = XBorder + FBorder.Ext;
			redraw();
			OsReset.notifyObservers();
		}    
	}
	else
		super.processKey( sel);
}

protected void redraw( Graphics g){

	if ( YOffs == 0)
		adjustContents();

	adjustToCursor( false);

	if ( Contents.Len == 0) {
		blank( g);
		clip( g);
		if ( EventHandler.HasFocus)
			drawCursor( g);
		return;
	}	

	int bl = Height - YOffs - Contents.MaxDescent;

	blank( g);
	clip( g);
	g.setFont( getFont() );

	if ( Validated && !isModified()){
		g.setColor( Color.green);
		g.fillRect( Width - FBorder.Ext - 4, FBorder.Ext, 4, 4);
	}

	if ( SelStart == SelEnd) {
		g.setColor( EventHandler.IsDisabled ? Awt.InactiveClr : getForeground());
		Contents.draw( 0, Contents.Len, g, XOffs , bl, XOffs);
	}
	else {
		int offs = XOffs;
		int dx   = 0;
		int sb   = Math.min( SelStart, SelEnd);
		int se   = Math.max( SelStart, SelEnd);
		Color cf = getForeground();
		Color cb = getBackground();

		if ( sb > 0) {
			g.setColor( cb);
			dx = Contents.charsWidth( 0, sb, 0);
			g.fillRect( FBorder.Ext, FBorder.Ext, offs + dx,
			            Height - 2 * FBorder.Ext);
			g.setColor( cf);
			Contents.draw( 0, sb, g, offs , bl, XOffs);
			offs += dx;
		}

		g.setColor( Awt.SelBackClr);
		dx = Contents.charsWidth( sb, se - sb, 0);
		g.fillRect( offs,
		            Height - YOffs - Contents.CharHeight,
		            dx,
		            Contents.CharHeight + 1);
		g.setColor( Awt.SelForeClr);
		Contents.draw( sb, se - sb, g, offs , bl, XOffs);
		offs += dx;

		if ( se < Contents.Len) {
			g.setColor( cb );
			g.fillRect( offs, FBorder.Ext,
			            Width - offs,
			            Height - 2 * FBorder.Ext);
			g.setColor( cf );
			Contents.draw( se, Contents.Len - se, g, offs , bl, XOffs);
		}
	}

	if ( EventHandler.HasFocus)
		drawCursor( g);
}

void registerHotKeys() {
	addHotKey( Awt.TextFieldValidKeys[0], Awt.TextFieldValidKeys[1], this, "newL");
	addHotKey( Awt.TextFieldClearKeys[0], Awt.TextFieldClearKeys[1], this, "esc");
	
	addHotKey( Key.Insert, HotKey.Lone, this, "toggleMode" );

	addHotKey( Awt.EditCutKeys[0], Awt.EditCutKeys[1], this, "cut");
	addHotKey( Awt.EditCopyKeys[0], Awt.EditCopyKeys[1], this, "copy");
	addHotKey( Awt.EditPasteKeys[0], Awt.EditPasteKeys[1], this, "paste");

}

public boolean replaceSelectionWith( String s) {
	int  sb   = Math.min( SelStart, SelEnd);
	int  len  = Math.abs( SelStart - SelEnd);

	if ( ReadOnly)
		return false;

	if ( RealContents != null){
		RealContents.replace( sb, sb + len, s);
		Contents.replace( sb, sb + len, EchoChar);
	}
	else
		Contents.replace( sb, sb + len, s);
		
	if ( s != null)
		CursorIdx = sb + s.length();

	updateSel();

	redraw();
	setModified( true);
	return true;
}

public boolean replaceSelectionWith( char c) {
	return replaceSelectionWith( String.valueOf( c));
}

public void select( int selStart, int selEnd) {
	//for java.awt.TextComponent compatibility
	SelStart = Math.min( selStart, Contents.Len);
	SelEnd   = Math.min( selEnd, Contents.Len);
	setCursorPos( 0, true);
}

public void selectContents() {
	SelStart = 0;
	SelEnd   = Contents.Len;
	setCursorPos( 0, true);
}

public void setChecker( Object chk) {
	if ( chk instanceof EntryKeyFilter)
		KeyFilter = (EntryKeyFilter) chk;
	if ( chk instanceof EntryValidator)
		Validator = (EntryValidator) chk;
}

public void setContents( String s) {
	Contents.free();
	Contents.append( s);
	if ( RealContents != null){
		RealContents.free();
		RealContents.insert( EchoChar, Contents.Len, 0);
	}

	SelStart = SelEnd = CursorIdx = 0;  
	XOffs = XBorder + FBorder.Ext;

	setModified( false);
	Validated = false;

	setCursorPos( 0, true);
}

public void setCursorIndex( int idx) {
	CursorIdx = Math.min( idx, Contents.Len);
	if ( isShowing() )
		redraw();
}

boolean setCursorPos( int x, boolean upd) {
	int xo = XOffs;

	for( int idx = 0; idx < Contents.Len; idx++) {
		if ( x <= xo){
			CursorIdx = idx;
			if ( upd && isShowing() )
				redraw();
			return true;
		}
		xo += Contents.charsWidth( idx, 1, 0);
	}

	CursorIdx = Contents.Len;

	if ( upd && isShowing() )
		redraw();

	return false;

}

public static void setDefaultBorderStyle( int style) {
	DefaultBorderStyle = style;
}

public void setEchoCharacter( char c) {

	char pec = EchoChar;
	EchoChar = c;
	
	if ( EchoChar != 0) {
		if ( RealContents != null )
			Contents.replaceAll( pec, EchoChar);
		else {
			RealContents = new LineBuffer ( Contents.Fm);
			RealContents.append( Contents.Text, 0, Contents.Len);
			Contents.free();
			Contents.insert( EchoChar, RealContents.Len, 0);
		}
	}
	else {
		Contents.free();
		Contents.append( RealContents.Text, 0, RealContents.Len); 
		RealContents = null;
	}
	
	if ( isShowing() )
		redraw();
}

public void setEditable( boolean editable){
	//for java.awt.TextComponent compatibility
	setEnabled( editable);
}

public void setEnabled( boolean e){
	updateSel();
	super.setEnabled( e);
}

public synchronized void setFont( Font fnt) {
	Font uf = ( fnt != null) ? fnt : Awt.SysFont;
	super.setFont( uf);

	if ( uf.equals( Awt.SysFont))
		Contents.setMetrics( DefFm);
	else
		Contents.setMetrics( Awt.getFontMetrics( uf));

	if ( RealContents != null)
		RealContents.setMetrics( Contents.Fm); 

	adjustContents();
	if ( isShowing() )
		redraw();
}

public void setKeyFilter( EntryKeyFilter f) {
	KeyFilter = f;
}

public void setModified( boolean m) {
	if ( m && !Modified) OsModified.notifyObservers( this);
	Modified = m;
}

public void setReadOnly( boolean state){
	ReadOnly = state;
}

public void setRightAlign( boolean a) {
	RightAlign = a;
	if ( isShowing() )
		redraw();
}

public void setText( String l){
	//for java.awt.TextComponent compatibility
	setContents( l);
}

public void setValidator( EntryValidator v) {
	Validator = v;
}

protected void updateCursor() {
	Graphics g = getGraphics();
	if( g != null) {
		if ( EventHandler.HasFocus)
			drawCursor( g);
		else
			blankCursor( g);
		g.dispose();
	}
}

void updateSel() {
	SelStart = CursorIdx;
	SelEnd = Math.min( SelStart + Mode, Contents.Len);
}

public boolean vKeyInput( int k, int mod) {

	switch ( k) {
	case Key.Backspace:
		backSpace();
		break;                    
	case Key.Delete:
		del();
		break;
	case Key.CursorLeft:
		cursorLeft( (mod & Event.SHIFT_MASK) != 0 );
		break;
	case Key.CursorRight:
		cursorRight( (mod & Event.SHIFT_MASK) != 0 );
		break;
	case Key.Home:
		if ( (mod & Event.SHIFT_MASK) != 0) {
			SelStart = CursorIdx;
			SelEnd   = 0;
		}
		XOffs = XBorder + FBorder.Ext;
		setCursorPos( 0, true);
		break;
	case Key.End:
		if ( (mod & Event.SHIFT_MASK) != 0) {
			SelStart = CursorIdx;
			SelEnd   = Contents.Len;
		}
		setCursorPos( 10000, true);
		break;
	default:
		return false;
	}

	return false;
}

public boolean validateContents () {
	if ( Validator == null){
		setModified( false);
		return true;
	}

	String cont = getContents();
	String val = Validator.validateContents( cont);

	if ( val == cont){
		setModified( false);
		return true;
	}

	setContents( val);
	selectContents();

	return false;
}
}
