package biss.awt;

import biss.awt.Awt;
import biss.awt.DialogWindow;
import biss.awt.Label;
import biss.awt.Layouter;
import biss.awt.TextField;
import java.awt.AWTEvent;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.Rectangle;
import java.util.Observable;

/**
 * logical view for generic String propmting dialogs
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class TextPrompter
  extends DialogWindow
{
	Label TP = new Label();
	TextField Entry = new TextField();

public TextPrompter( String text, Frame owner){
	this( "Please enter:", text, null, false, null, owner);
}

public TextPrompter( String text, String defCont, Frame owner){
	this( "Please enter:", text, defCont, true, null, owner);
}

public TextPrompter( String title, String text, String defCont, Frame owner){
	this( title, text, defCont, true, null, owner);
}

public TextPrompter( String title, String text, String defCont, boolean defSel, Object checker, Frame owner ){
	super( owner, title, DISPATCHED);
	TP.setContents( text);

	if ( defCont != null) {
		Entry.setContents( defCont);
		if ( defSel)
			Entry.selectContents();
	}

	if ( checker != null)
		Entry.setChecker( checker);

	open();
}

public String getContents(){
	if ( WasCanceled)
		return null;
	return Entry.getContents();
}

public int getContentsAsInt() {
	try { return Integer.parseInt( Entry.getContents() ); }
	catch ( Exception e) { return -1; }
}

public String getNonEmptyContents(){
	if ( ! WasCanceled){
		String c = Entry.getContents();
		if ( ( c != null) && ( c.length() > 0) )
			return c;
	}
	return null;
}

public Dimension getPreferredSize() {
	int		tw = 30*Awt.SysFontWidth;
	int		th = TP.minHeight( tw) + 5*Awt.SysFontHeight;
	
	return new Dimension( tw, th);
}

void init() {
	TP.wrapContents();
	TP.setCarved( false);
	TP.setBorder( Border.BS_NONE);
	TP.setAlignment( Label.HA_CENTER, Label.VA_CENTER);

	Entry.FBorder.setBorder( Border.BS_STEP);
	Entry.OsEnter.addObserver( this);
	Entry.OsReset.addObserver( this);

	DefCom = Entry;
	setCenterBounds();
}

public static void main ( String args[]) {
	Frame fr = new Frame( "ModalOwner");
	fr.setBounds( 100, 100, 100, 100);
	fr.setVisible( true);
	String txt = "Please type your lastname, userID or password";
	String def = "Gonzo";
	TextPrompter tp = new TextPrompter( "My Prompter", txt, def, true, null, fr);
	System.out.println( tp.getNonEmptyContents() );
	System.exit( 0);
}

public void open() {
	Lo.add( TP, 0, 0, 1000, 1000, true, 20, 20, -20, -200, 0);
	Lo.add( Entry, 0, 1000, 1000, 1000, true, 20, -180, -20, -20, 0);

	init();

	Rectangle bounds = getBounds();

	if ( bounds.width > 0)
		setVisible( true);
	else {
		bounds.setBounds( 3, 2283, 1543, 650);
		Awt.toPixelUnits( bounds);
		setBounds( bounds);
		setVisible( true);
	}
}

public static String prompt( String title, String text, Frame owner) {
	TextPrompter tp = new TextPrompter( title, text, owner);
	if ( tp.WasCanceled)
		return null;
	return tp.getContents();
}

public static String prompt( String title, String text, String defCont, Frame owner) {
	TextPrompter tp = new TextPrompter( title, text, defCont, owner);
	if ( tp.WasCanceled)
		return null;
	return tp.getContents();
}

public void update( Observable o, Object arg){
	if ( o == Entry.OsEnter )
		process();
	else if ( o == Entry.OsReset ){
		WasCanceled = true;
		process();
	}
}
}
