package biss.awt;

import biss.Command;
import biss.CommandQueue;
import biss.ObserverSocket;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.ImageObserver;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

class SingleInstCloseCmd
  implements Command
{
	TopWindow Target;
	int Count;

public SingleInstCloseCmd ( TopWindow target ) {
	Target = target;
	Count = 5;
}

public boolean execute () {
	if ( Target != TopWindow.ActiveTopWindow ) {
		if ( --Count > 0 ) {
			CommandQueue.append( this);
			return true;
		}

		Target.disposeSingleInstWnd();
	}

	return true;
}
}

/**
 * enhancement of java.awt.Frame
 *
 * (C) 1996-1998 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz, M.Brauch
 */
public class TopWindow
  extends Frame
  implements Observer, FocusInterface, KbdInterface, MouseInterface, CmdInterface, WindowListener, GUIHandlerInterface, HotKeyProcessor
{
	boolean DisableDispose = false;
	static Vector Frames = new Vector();
/**
 * triggered by window creation ( show() )
 * parameter: this
 */
	public ObserverSocket OsOpen = new ObserverSocket( this, "Open");
/**
 * triggered by window closing ( dispose() )
 * parameter: this
 */
	public ObserverSocket OsClose = new ObserverSocket( this, "Close");
/**
 * externally triggered by the GUIHandler to notify about a
 * Menu or HotKey action
 * parameter: HotKey or MenuItem selector
 */
	public ObserverSocket OsCommand = new ObserverSocket( this, "Command");
/**
 * externally triggered by the GUIHandler to notify
 * that the window got the input focus
 * parameter: -
 */
	public ObserverSocket OsFocusGot = new ObserverSocket( this);
/**
 * externally triggered by the GUIHandler to notify
 * that the window lost the input focus
 * parameter: -
 */
	public ObserverSocket OsFocusLost = new ObserverSocket( this);
/**
 * externally triggered by the GUIHandler to notify
 * that the mouse pointer enters the window area
 * parameter: -
 */
	public ObserverSocket OsMouseEnter = new ObserverSocket( this);
/**
 * externally triggered by the GUIHandler to notify
 * that the mouse pointer exits the window area
 * parameter: -
 */
	public ObserverSocket OsMouseExit = new ObserverSocket( this);
/**
 * externally triggered by the GUIHandler to notify
 * about a MouseButton Event
 * parameter: Event
 */
	public ObserverSocket OsMouseBtn = new ObserverSocket( this);
	protected GUIHandler EventHandler = new GUIHandler( this);
	protected Window Overlay;
	protected Component DefComponent;
	PaintCmd RetainPainter = (Awt.FilterPaints > 0) ? new PaintCmd( this) : null;
	static TopWindow ActiveTopWindow;
	Image ImageBack;
	protected Menu PopupContents;
	public static boolean ExitOnZeroInst = true;
	protected TopWindow Owner;
	protected Vector Owned = null;
	Initializer Init;

public TopWindow(){
	this( "TopWindow");
}

public TopWindow( String aTitle){
	this( aTitle, null);
}

public TopWindow( String aTitle, TopWindow owner){
	this( aTitle, owner, null);
}

public TopWindow( String aTitle, TopWindow owner, Initializer init){
	super( aTitle);
	Owner = owner;
	Init  = init;
	
	if ( Owner != null) Owner.addOwned( this);

	OsOpen.addObserver( this);
	initEventHandler();
	setBackground( Awt.WindowBackClr);
}

public boolean addGlobalHotKey( int c, HotKeyProcessor p, Object sel){
	return EventHandler.addHotKey( c, Awt.GlobalKeyMod, p, sel);
}

public boolean addHotKey( int c, int mods, HotKeyProcessor p, Object sel){
	return EventHandler.addHotKey( c, mods, p, sel);
}

public boolean addHotKey ( int c, int mods, Object sel ) {
	return EventHandler.addHotKey( c, mods, null, sel);
}

protected void addOwned( TopWindow w){
	if ( Owned == null) Owned = new Vector();
	Owned.addElement( w);
}

public boolean canGetFocus () {
	return isEnabled() && !isOwner();
}

public Rectangle centerToOwnerBounds( Rectangle b){
	if ( Owner == null) return b;
	
	//--- center to owner's center
	Rectangle ob = getOwnerBounds();
	b.x = ob.x + ob.width/2 - b.width/2;
	b.y = ob.y + ob.height/2 - b.height/2;
	
	//--- keep window completely on screen
	if ( b.x < 0) b.x = 0;
	if ( b.y < 0) b.y = 0;
	if ( (b.x + b.width)  > Awt.ScreenWidth ) b.x = Awt.ScreenWidth  - b.width; 
	if ( (b.y + b.height) > Awt.ScreenHeight) b.y = Awt.ScreenHeight - b.height; 

	return b;
}

public void centerToScreen(){
	Dimension ss = Awt.DefToolkit.getScreenSize();
	Dimension d = getSize();

	setBounds( (ss.width - d.width) / 2, (ss.height - d.height) / 2,
	         d.width, d.height );
}

public Rectangle centeredBoundsFromDlgExtent ( int width, int height ) {
	Dimension ss = Awt.DefToolkit.getScreenSize();
	int cx = (width*Awt.DlgXUnit+50)/100;
	int cy = (height*Awt.DlgYUnit+50)/100;

	return new Rectangle( (ss.width - cx) / 2,
	                      (ss.height - cy) / 2,
	                      cx, cy );
}

public Rectangle centeredBoundsFromExtent ( int width, int height ) {
	Dimension ss = Awt.DefToolkit.getScreenSize();

	return new Rectangle( (ss.width - width) / 2,
	                      (ss.height - height) / 2,
	                      width, height );
}

public void charInput( char c, int mod){}

public void disableDispose() {
	DisableDispose = true;
}

public void dispose () {

	if ( isOwner()) return;

	DisableDispose = false;
	OsClose.notifyObservers( this);

	if ( DisableDispose )
		return;

	if ( Overlay != null )
		Overlay.dispose();

	setVisible( false);

	Frames.removeElement( this);
	super.dispose() ;

	if ( Owner != null) {
		Awt.enableComponents( Owner);
		Owner.removeOwned( this);
	}
	if(( !Awt.IsApplet) && (Frames.size() == 0) && ExitOnZeroInst )
		System.exit(0);
}

public boolean disposeSingleInstWnd () {
	if ( EventHandler.isSingleInstWndOwner( this) )
		return EventHandler.disposeSingleInstWnd();
	else
		return false;
}

public void focusNext(){
	FocusHandler.focusNext( this);
}

public void focusPrev( ){
	FocusHandler.focusPrev( this);
}

public GUIHandler getGUIHandler () {
	return EventHandler;
}

public Menu getMenu(){
	return PopupContents;
}

public ObserverSocket getOsCommand() {
	return OsCommand;
}

public ObserverSocket getOsFocusGot() {
	return OsFocusGot;
}

public ObserverSocket getOsFocusLost() {
	return OsFocusLost;
}

public ObserverSocket getOsMouseBtn() {
	return OsMouseBtn;
}

public ObserverSocket getOsMouseEnter() {
	return OsMouseEnter;
}

public ObserverSocket getOsMouseExit() {
	return OsMouseExit;
}

public Rectangle getOwnerBounds() {

	if ( Owner == null) return null;

	return Owner.getBounds();
}

public void gotFocus () {

	ActiveTopWindow = this;

	if ( Overlay != null ){
		CommandQueue.append( new ToFrontCmd( Overlay, 10));
		Awt.DefToolkit.sync();
	}
}

public boolean hasFocus () {
	return EventHandler.HasFocus;
}

void initEventHandler( ) {
	EventHandler.Wnd = this;
	addWindowListener( this);
	registerObserver();
}

public boolean isActive () {
	return (ActiveTopWindow == this);
}

public boolean isOwner(){
	return (Owned != null) && ( !Owned.isEmpty());
}

public boolean isSingleInstWndOwner () {
	return EventHandler.isSingleInstWndOwner( this);
}

public void lostFocus(){
	if ( (Awt.AwtToolkit != Awt.TK_WIN) && (Awt.AwtToolkit != Awt.TK_OS2) ) {
		if ( EventHandler.isSingleInstWndOwner( this))
			CommandQueue.append( new SingleInstCloseCmd( this));
	}
}

public void mouse1Click(){}

public void mouse1DblClick(){}

public void mouse1Down(){}

public void mouse1Up(){}

public void mouse2Click(){
	if ( PopupContents != null) {
		PopupMenu p = new PopupMenu( this, PopupContents);
		Point pt = Awt.getDesktopLocation( this, Mouse.Pos.x, Mouse.Pos.y);
		p.popUpAt( pt.x, pt.y);
	}
}

public void mouse2DblClick(){}

public void mouse2Down(){}

public void mouse2Up(){}

public void mouseEnter() {}

public void mouseExit(){
}

public void openCenteredInDlgUnits ( int width, int height ) {
	int cx = Awt.DlgXUnit;
	int cy = Awt.DlgYUnit;

	if ( ! (getLayout() instanceof Layouter) )
		pack();

	setSize( (width*cx+50)/100, (height*cy+50)/100 );
	centerToScreen();
	setVisible( true);
}

public void openIn ( Rectangle rect ) {
	openIn( rect.x, rect.y, rect.width, rect.height );
}

public void openIn ( int x, int y, int width, int height ) {
	if ( ! (getLayout() instanceof Layouter) )
		pack();
	setBounds( x, y, width, height );
	setVisible( true);
}

public void openInDlgUnits ( Rectangle rect ) {
	if ( ! (getLayout() instanceof Layouter) )
		pack();
	setBounds( Awt.toPixelUnits( rect));
	setVisible( true);
}

public void openInDlgUnits ( int x, int y, int width, int height ) {
	openInDlgUnits( new Rectangle( x, y, width, height));
}

public void paint ( Graphics g ) {

//	if ( (RetainPainter != null) && RetainPainter.checkPaintSeries() )
//		return;
	
	if ( (ImageBack != null) && isShowing() ){
		Dimension d = getSize();
		int x0 = 0;
		int y0 = 0;
		
		while ( y0 < d.height) {
			x0 = 0;
			while ( x0 < d.width) {
				g.drawImage( ImageBack, x0, y0, this);
				x0 += ImageBack.getWidth( null);
			}
			y0 += ImageBack.getHeight( null);
		}
	}

	super.paint( g);
}

public void processKey( Object sel){
	if ( sel instanceof MenuItem)
		OsCommand.notifyObservers( ((MenuItem)sel).Selector);
}

public void registerObserver( ){
	OsCommand.addObserver( this);
}

protected void removeOwned( TopWindow w){
	if ( Owned == null) return;
	Owned.removeElement( w);
	if ( Owned.isEmpty()) Owned = null;
}

public void repaintAll() {
	Component [] childs = getComponents();

	repaint();
	for ( int i=0; i<childs.length; i++ )
		childs[i].repaint();
}

public void resetOverlay () {
	if ( Overlay != null ) {
		Awt.enableComponents( this);
		Overlay = null;
	}
}

public void setBackgroundImage ( Image img) {
	ImageBack = img;
	if ( isShowing() )
		repaint();
}

public void setDefaultComponent ( Component c ) {
	DefComponent = c;
}

public void setEnabled ( boolean b) {
	super.setEnabled( b);
	EventHandler.IsDisabled = !b;
}

public boolean setFocus(){

	if ( canGetFocus() ) {
		requestFocus();
		return true;
	}
	return false;
}

public void setMenu( Menu m){
	PopupContents = m;
	EventHandler.addHotKeys( m.getHotKeys( this, HotKey.Control));
}

public void setOverlay ( Window tw ) {
	if ( Overlay != null )
		Overlay.dispose();

	Overlay = tw;

	if ( Overlay != null)
		Awt.disableComponents( this);
	else
		Awt.enableComponents( this);
}

public int tbHeight(){
	return getInsets().top;
}

public void toFront(){

	if ( isOwner() ) return;
	
	super.toFront();
}

public TopWindow topMostOwned(){

	if ( !isOwner()) return this;
	
	TopWindow o = (TopWindow)(Owned.lastElement());
	return o.topMostOwned();
}

public void unregisterObserver( ){
	OsCommand.deleteObserver( this);
}

public void update( Observable slot, Object arg){

	if ( arg instanceof Command ){
		((Command)arg).execute();
		return;
	}
}

public boolean vKeyInput( int c, int mod){
	return false;
}

public void windowActivated( WindowEvent e) {
	if ( isOwner()) {
		TopWindow o = topMostOwned();
		o.toFront();
		o.setFocus();
	} else {
		EventHandler.disposeSingleInstWnd();
		ActiveTopWindow = this;
	}
}

public void windowClosed( WindowEvent e) {
}

public void windowClosing( WindowEvent e) {
	dispose();
}

public void windowDeactivated( WindowEvent e) {
}

public void windowDeiconified( WindowEvent e) {
}

public void windowIconified( WindowEvent e) {
}

public void windowOpened( WindowEvent e) {

	Frames.addElement( this);
	
	if ( Owner != null) Awt.disableComponents( Owner);

	if ( Init != null) Init.init( this);

	OsOpen.notifyObservers( this);
	
	if ( DefComponent != null)
		DefComponent.requestFocus();
	else if ( getComponentCount() > 0)
		getComponent(0).requestFocus();
}
}
