package biss.awt;

import biss.CommandQueue;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Scrollbar;
import java.awt.event.MouseEvent;

/**
 * vertical scrollbar ( replaces java.awt.Scrollbar)
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class VScrollBar
  extends ScrollBar
{
public VScrollBar () {
	super( 0, 0, 0);
}

public VScrollBar ( int min, int max, int pos) {
	super( min, max, pos);
}

public Dimension getPreferredSize(){
	return new Dimension( Awt.ScrollBarWidth, 10*Awt.SysFontHeight);
}

public void mouse1Drag() {
	if ( DragOffs > 0){
		int w  = Width - FBorder.Ext;
		int tl = Height - 2*w;
		int y  = Mouse.Pos.y - DragOffs;

		setPosition( Min + ((y - w)*(Max - Min)) / (tl - SliderRect.height) );
	}
}

public void mouseDownScroll () {

	int x = Mouse.Pos.x;
	int y = Mouse.Pos.y;
	
	if ( SliderRect.contains( x, y)){
		DragOffs = y - SliderRect.y;
		ScrollOp = 5;
		LastOp = TRACK;
	}
	else if ( (y < Width) && (Pos > Min) ){
		ScrollOp = 1;
		LastOp = UNIT_DECREMENT;
		redrawSpins();
		Repeater.start();
	}
	else if ( (y > Height - Width) && (Pos < Max) ){
		ScrollOp = 2;
		LastOp = UNIT_INCREMENT;
		redrawSpins();
		Repeater.start();
	}
	else if ( SliderRect.height > 0) {
		if ( y < SliderRect.y){
			ScrollOp = 3;
			LastOp = BLOCK_DECREMENT;
		}
		else if ( y > SliderRect.y + SliderRect.height){
			ScrollOp = 4;
			LastOp = BLOCK_INCREMENT;
		}
	}

}

protected void redrawSlider( Graphics g) {
	if ( Height <= 0)
		return;

	int b = FBorder.Ext;
	int w = Width - b;
	int h = Height - 2*w;

	if (SliderRect.height > 0) {
		int y, d = SliderRect.y - w;
		int ww = w-b;
		g.setColor( getBackground() );
		if ( d > 0 )
			g.fillRect( b, w, ww, d);
		y = SliderRect.y + SliderRect.height;
		d = Height - w - y;
		if ( d > 0 )
			g.fillRect( b, y, ww, d);

		GraphicsLib.paint3dBox( g, SliderRect.x, SliderRect.y,
		                        SliderRect.width, SliderRect.height,
		                        false, Awt.ButtonBackClr);
	}
	else {
		g.setColor( getBackground() );
		g.fillRect( b, w, w-b, h);
	}
}

protected void redrawSpins( Graphics g) {
	int b = FBorder.Ext;
	int w = Width - 2*b;

	if ( Mouse.isAnyActive() && (ScrollOp == UNIT_INCREMENT) )
		GraphicsLib.paintButton( g, b, b, w, w, false, Awt.ButtonBackClr);
	else
		GraphicsLib.paint3dBox( g, b, b, w, w, false, Awt.ButtonBackClr);

	if ( Mouse.isAnyActive() && (ScrollOp == UNIT_DECREMENT) )
		GraphicsLib.paintButton( g, b, Height-b-w, w, w, false, Awt.ButtonBackClr);
	else
		GraphicsLib.paint3dBox( g, b, Height-b-w, w, w, false, Awt.ButtonBackClr);
}

protected void updateSliderPos() {
	int w  = Width - FBorder.Ext;
	int tl = Height - 2*w;
	SliderRect.y = w + (Pos-Min) * (tl-SliderRect.height)/(Max-Min);
}

protected void updateSliderRect() {
	if ( (Width == 0) || (Height == 0) )
		return;
		
	int b = FBorder.Ext;
	int w = Width - b;
	int tl = Height - 2*w;

	if ( Max - Min <= 0){
		SliderRect.setBounds( 0, 0, 0, 0);
	}
	else {
		int s = Math.max((int)((double)tl * PageInc / (PageInc+Max-Min)), Width);
		int yPos = w + (Pos-Min) * (tl-s)/(Max-Min);
		SliderRect.setBounds( b, yPos, w-b, s);
	}
}
}
