package biss.awt;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.Enumeration;

/**
 * vertical splitbar (moveable boundary between widgets to change the
 * layout of a Container)
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author P.C.Mehlitz
 */
public class VSplitBar
  extends SplitBar
{
public VSplitBar () {
	setCursor( Cursor.getPredefinedCursor( Cursor.N_RESIZE_CURSOR));
}

public VSplitBar ( int id ) {
	this();
	Id = id;
}

void computeSiblings () {
	Component childs[] = getParent().getComponents();
	Rectangle b = getBounds();
	int i;
	Rectangle r;

	for ( i=0; i<childs.length; i++ ) {
		if ( (childs[i] != this) && (childs[i].isVisible()) ) {
			r = childs[i].getBounds();
			if ( r.x < (b.x + b.width) && (r.x + r.width) > r.x ) {
				if ( (Math.abs( b.y - (r.y + r.height)) <= 1) ||
				     (Math.abs( r.y - (b.y + b.height)) <= 1) ) {
					addSibling( childs[i]);
				}
			}
		}
	}
}

public Dimension getPreferredSize () {
	return new Dimension( Parent.getSize().width, 5);
}

public void mouseDragged ( MouseEvent e ) {
	int y  = e.getY();
	int y1 = YPos;
	int x2 = XPos+Width - 1;
	
	if ( X < 0 ) {
		X = e.getX();
	}
	else {
		y1 += Y;
		PGr.drawLine( XPos, y1, x2, y1);
	}

	y1 = YPos + y;

	if ( y1 <= Max && y1 >= Min ) {
		Y = y;
		PGr.drawLine( XPos, y1, x2, y1);
	}

}

public void paint ( Graphics g ) {
	g.setColor( getBackground());
	g.fillRect( 0, 0, Width, Height);

	// draw handle (to indicate moveable boundary)
	int b = 12;
	int h = Height -1;
	g.setColor( Color.white);
	g.drawLine( 0, 0, b, 0);
	g.drawLine( 0, 0, 0, h);
	g.setColor( Color.black);
	g.drawLine( 0, h, b, h);
	g.drawLine( b, 0, b, h);
}

void rearrange () {
	Component c;
	Rectangle r;
	int       yNew, hNew;
	int       y = YPos + Y;
	int       y2 = YPos + Height;
	int       d;

	setVisible( false);
	
	for ( Enumeration e=Siblings.elements(); e.hasMoreElements(); ) {
		c = (Component) e.nextElement();
		r = c.getBounds();

		if ( r.y >= y2 ) {   // below, adjust top
			d    = r.y - y2;
			yNew = y + Height + d;
			hNew = r.height + (r.y - yNew);
		}
		else {              // above, adjust bottom
			d    = YPos - (r.y + r.height);
			yNew = r.y;
			hNew = y - r.y - d;
		}
		c.setBounds( r.x, yNew, r.width, hNew);

		/*** panel bug workaround ***/
		if ( c instanceof Container)
			c.doLayout();
			
		c.setVisible( true);
	}

	setLocation( XPos, y);
	setVisible( true);
}

void updateMoveBoundaries ( Rectangle r ) {
	if ( r.y >= YPos ) {
		int y = r.y+r.height - Height;
		if ( y > Max )
			Max = y;
	}
	if ( r.y < Min )
		Min = r.y;
}
}
