package biss.awt;

import biss.Command;
import biss.ObserverSocket;
import java.awt.AWTEvent;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.*;
import java.awt.image.ImageObserver;
import java.util.Observable;
import java.util.Observer;

/**
 * abstract base used to implement various low level widgets
 * (java.awt.Canvas replacement)
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class Widget
  extends Component
  implements Observer, FocusInterface, MouseInterface, KbdInterface, CmdInterface, GUIHandlerInterface, HotKeyProcessor
{
	protected Rectangle DrawRect = new Rectangle(0,0,0,0);
/**
 * externally triggered by the GUIHandler to notify about a
 * Menu or HotKey action
 * parameter: HotKey or MenuItem selector
 */
	public ObserverSocket OsCommand = new ObserverSocket( this, "Command");
/**
 * externally triggered by the GUIHandler to notify
 * that the Widget got the input focus
 * parameter: -
 */
	public ObserverSocket OsFocusGot = new ObserverSocket( this);
/**
 * externally triggered by the GUIHandler to notify
 * that the Widget lost the input focus
 * parameter: -
 */
	public ObserverSocket OsFocusLost = new ObserverSocket( this);
/**
 * externally triggered by the GUIHandler to notify
 * that the mouse pointer enters the Widget area
 * parameter: -
 */
	public ObserverSocket OsMouseEnter = new ObserverSocket( this);
/**
 * externally triggered by the GUIHandler to notify
 * that the mouse pointer exits the Widget area
 * parameter: -
 */
	public ObserverSocket OsMouseExit = new ObserverSocket( this);
/**
 * externally triggered by the GUIHandler to notify
 * about a MouseButton Event
 * parameter: Event
 */
	public ObserverSocket OsMouseBtn = new ObserverSocket( this);
	protected GUIHandler EventHandler = new GUIHandler( this);
	protected Border FBorder = new Border();
	protected int Width = 0;
	protected int Height = 0;
	protected int XPos = 0;
	protected int YPos = 0;
	PaintCmd RetainPainter = (Awt.FilterPaints > 0) ? new PaintCmd( this)  : null;
	protected Menu PopupContents;

public boolean addHotKey( int c, int mods, HotKeyProcessor p, Object sel){
	return EventHandler.addHotKey( c, mods, p, sel);
}

public boolean addHotKey ( int c, int mods, Object sel ) {
	return EventHandler.addHotKey( c, mods, null, sel);
}

public synchronized void addNotify () {
	super.addNotify();
	Window w = getWindow();
	EventHandler.Wnd = w;
	if ( w instanceof GUIHandlerInterface)
		EventHandler.Super = ((GUIHandlerInterface)w).getGUIHandler();
}

public void blank(){
	Graphics g = getGraphics();
	if ( g != null ) {
		blank( g);
		g.dispose();
	}
}

public void blank( Graphics g){
	g.setColor( getBackground() );
	g.fillRect( DrawRect.x, DrawRect.y, DrawRect.width, DrawRect.height);
}

public boolean canGetFocus () {
	return isEnabled() && isShowing();
}

public void charInput( char c, int mod) {}

protected void clip( Graphics g){
	g.setClip( DrawRect.x, DrawRect.y, DrawRect.width, DrawRect.height);
}

public boolean contains( int x, int y){
	if ( isShowing() )
		return super.contains( x, y);
	return false;
}

public Rectangle getDrawRect(){
	return DrawRect;
}

public Frame getFrame() {
	for ( Container so = getParent(); so != null; so =so.getParent() ) {
		if ( so instanceof Frame)
			return (Frame)so;
	}
	return null;
}

public GUIHandler getGUIHandler() {
	return EventHandler;
}

public Graphics getGraphics(){
	/* resolved to prevent drawing in invisible state */
	if ( isShowing() )
		return super.getGraphics();
	return null;
}

public int getHeight(){
	return Height;
}

public Menu getMenu(){
	return PopupContents;
}

public ObserverSocket getOsCommand() {
	return OsCommand;
}

public ObserverSocket getOsFocusGot() {
	return OsFocusGot;
}

public ObserverSocket getOsFocusLost() {
	return OsFocusLost;
}

public ObserverSocket getOsMouseBtn() {
	return OsMouseBtn;
}

public ObserverSocket getOsMouseEnter() {
	return OsMouseEnter;
}

public ObserverSocket getOsMouseExit() {
	return OsMouseExit;
}

public Dimension getPreferredSize(){
	return getMinimumSize();
}

public TopWindow getTopWindow() {
	for ( Container so = getParent(); so != null; so =so.getParent() ) {
		if ( so instanceof TopWindow)
			return (TopWindow)so;
	}
	return null;
}

public int getWidth(){
	return Width;
}

public Window getWindow() {
	for ( Container so = getParent(); so != null; so =so.getParent() ) {
		if ( so instanceof Window)
			return (Window)so;
	}
	return null;
}

public int getXPos(){
	return XPos;
}

public int getYPos(){
	return YPos;
}

public void gotFocus() {}

public boolean hasFocus () {
	return (EventHandler.HasFocus && isShowing() );
}

public boolean imageUpdate ( Image img, int infoflags, int x, int y,
		      int width, int height) {

	if ( infoflags == (ImageObserver.WIDTH | ImageObserver.HEIGHT))
		Awt.DefToolkit.prepareImage( img, width, height, this);

	boolean ready = ( (infoflags & ImageObserver.ALLBITS) > 0);

	if (ready) // standard repaint() sometimes seems to blank (causing flicker)
		redraw();

	return !ready;
}

public boolean isEnabled(){
	return !EventHandler.IsDisabled;
}

public void lostFocus() {}

public void mouse1Click() {}

public void mouse1DblClick() {}

public void mouse1Down() {}

public void mouse1Up() {}

public void mouse2Click() {
	if ( PopupContents != null) {
		PopupMenu p = new PopupMenu( this, PopupContents);
		Point pt = Awt.getDesktopLocation( this, Mouse.Pos.x, Mouse.Pos.y);
		p.popUpAt( pt.x, pt.y);
	}
}

public void mouse2DblClick() {}

public void mouse2Down() {}

public void mouse2Up() {}

public void mouseEnter() {}

public void mouseExit() {}

public void paint() {
	if ( isShowing() ) {
		Graphics g = getGraphics();
		if ( g != null){
			paint( g);
			g.dispose();
		}
	}
}

public void paint( Graphics g) {

	// infamous ScreenUpdater neutralizer
//	if ( RetainPainter != null && RetainPainter.checkPaintSeries() )
//		return;

	//windows bug with graphics initialization ( clipRect)
	if ( Awt.AwtToolkit == Awt.TK_WIN) {
		Graphics grr = getGraphics();
		if ( grr != null) {
			FBorder.drawBorder( grr, Width, Height);
			grr.dispose();
		}
	}
	else
		FBorder.drawBorder( g, Width, Height);

	redraw( g);
}

protected void posChanged() {}

void preProcessKeyEvent ( KeyEvent e ) {
	// to speed up, skip parent barMenu shortcuts
}

public void processKey( Object sel){
	if ( sel instanceof MenuItem)
		OsCommand.notifyObservers( ((MenuItem)sel).Selector);
}

public void redraw () {
	if ( isShowing() ) {
		Graphics g = getGraphics();
		if ( g != null ) {
			redraw( g);
			g.dispose();
		}
	}
}

protected void redraw( Graphics g) {
	// redraw contents (without borders)
}

public void redrawImmediate(){
	redraw();
	Awt.DefToolkit.sync();
}

public void registerObserver(){
}

public void setBackground( Color c){
	super.setBackground( c);
	if ( isShowing() )
		redraw();
}

public void setBorder( int bs){
	FBorder.setBorder( bs);

	DrawRect.setBounds( FBorder.Ext, FBorder.Ext,
	                  Width-2*FBorder.Ext, Height-2*FBorder.Ext);
}

public void setBounds( int x, int y, int width, int height) {

	if ( (width < 0) || (height < 0) )
		return;

	boolean change = ((XPos != x) ||
	                  (YPos != y) ||
	                  (Width != width) ||
	                  (Height != height));

	if ( ! change)
		return;

	XPos   = x;
	YPos   = y;
	Width  = width;
	Height = height;

	DrawRect.setBounds( FBorder.Ext, FBorder.Ext,
	                  Width-2*FBorder.Ext, Height-2*FBorder.Ext);

	super.setBounds( x, y, width, height);
	posChanged();
}

public void setEnabled( boolean b){
	Frame fr;

	EventHandler.IsDisabled = !b;

	if ( isShowing() ){
		if ( (fr = getFrame()) instanceof TopWindow ) {
			if ( ((TopWindow)fr).Overlay == null ) { // otherwise Overlay has focus
				fr.requestFocus();  // don't break tab switching
				redraw();
			}
		}
		else
			redraw();
	}
	
	super.setEnabled( b);
}

public boolean setFocus () {
	if ( canGetFocus() ) {
		requestFocus();
		return true;
	}
	return false;
}

public void setFont( Font f){
	super.setFont( f);
	if ( isShowing() )
		redraw();
}

public void setForeground( Color c){
	super.setForeground( c);
	if ( isShowing() )
		redraw();
}

public void setMenu( Menu m){
	PopupContents = m;
	EventHandler.addHotKeys( m.getHotKeys( this, HotKey.Control));
	
}

public void sync(){
	Awt.DefToolkit.sync();
}

public void unregisterObserver(){
}

public void update( Observable o, Object arg){
	if ( arg instanceof Command )
		((Command)arg).execute();
}

public boolean vKeyInput ( int code, int mod) {
	return false;
}
}
