package java.util;


/**
 * Reimplemented java.util.Observable class to avoid memory waste
 * and improve performance ( 4 times faster) on notifyObservers()
 *
 * (C) 1996,97 BISS GmbH Germany, see file 'LICENSE.BISS-AWT' for details
 * @author J.H.Mehlitz
 */
public class Observable
{
	protected boolean Changed;
	protected Observer Obs;

public Observable() {
}

public synchronized void addObserver( Observer o){
	if ( Obs == null)
		Obs = o;
	else if ( Obs != o){
		if ( !( Obs instanceof ObserverItem))
			Obs = new ObserverItem( Obs);
		((ObserverItem)Obs).addObserver( o);
	}
}

protected synchronized void clearChanged(){
	Changed = false;
}

public synchronized int countObservers(){
	if ( Obs == null)
		return 0;
	if ( !( Obs instanceof ObserverItem))
		return 1;
	return ((ObserverItem)Obs).countObservers();
}

public synchronized void deleteObserver( Observer o){
	if ( Obs == null)
		return;
	else if ( !( Obs instanceof ObserverItem))
		Obs = null;
	else {
		ObserverItem oi = (ObserverItem)Obs;
		if ( oi.countObservers() == 1)
			Obs = oi.Obs;
		else
			oi.deleteObserver( o);
	}
}

public synchronized void deleteObservers(){
	Obs = null;
}

public synchronized boolean hasChanged(){
	return Changed;
}

public void notifyObservers(){
	notifyObservers( null);
}

public void notifyObservers ( Object arg ) {
	if ( ! Changed)
		return;
	if ( Obs != null)
		Obs.update( this, arg);
	Changed = false;
}

protected synchronized void setChanged(){
	Changed = true;
}
}

class ObserverItem
  implements Observer
{
	Observer Obs;
	ObserverItem Next;

public ObserverItem ( Observer obs) {
	Obs = obs;
}

void addObserver( Observer o){
	ObserverItem oi = this;
	for ( ; oi.Next != null; oi = oi.Next) {
		if ( oi.Obs == o)
			return;
	}
	if ( oi.Obs != o)
		oi.Next = new ObserverItem( o);
}

int countObservers(){
	int count = 0;
	for ( ObserverItem oi = this; oi != null; oi = oi.Next)
		count++;
	return count;
}

void deleteObserver( Observer o){
	for ( ObserverItem oi = this; oi.Next != null; oi = oi.Next){
		if ( oi.Next.Obs == o){
			oi.Next = oi.Next.Next;
			break;
		}
	}
}

public void update ( Observable obs, Object arg ) {
	for ( ObserverItem oi = this; oi != null; oi = oi.Next)
		oi.Obs.update( obs, arg);
}
}
